<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\API\Entities;

/**
 * Representation of a Certificate
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Certificate implements Entity
{
    private $caReference;
    private $tbsReference;
    private $customerReference;
    private $serialNumber;
    
    public function __construct(
            $caReference = null,
            $tbsReference = null,
            $customerReference = null,
            $serialNumber = null
    ) {
        $this->caReference = $caReference;
        $this->tbsReference = $tbsReference;
        $this->customerReference = $customerReference;
        $this->serialNumber = $serialNumber;
    }
    
    public function getCAReference()
    {
        return $this->caReference;
    }
    
    public function getTBSReference()
    {
        return $this->tbsReference;
    }
    
    public function getCustomerReference()
    {
        return $this->customerReference;
    }
    
    public function getSerialNumber()
    {
        return $this->serialNumber;
    }
    
    private function checkAttributes()
    {
        if (!empty($this->caReference) || !empty($this->tbsReference)) {
            $check = true;
        } else {
            $check = false;
        }
        return $check;
    }
    /**
     * Function used to validate the values of properties.
     * @todo Error management
     */
    public function validate()
    {
        while (!$this->checkAttributes()) {
            \cli\line('An issue occured with the CERTIFICATE. Please provide or correct the following information');
        
            if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                $this->interactiveFallback();
            } else {
                throw new \Exception('An issue occured with the CERTIFICATE.');
            }
        }
    }
    
    public function interactiveFallback()
    {
        if (empty($this->caReference)) {
            $this->caReference = \cli\prompt('   - CA reference', null);
        }
        if (empty($this->caReference) && empty($this->tbsReference)) {
            $this->tbsReference = \cli\prompt('   - TBS reference', null);
        }
    }
    
    public function save()
    {
        $info = array();
        
        if (isset($this->caReference) && !is_null($this->caReference)) {
            $info['caReference'] = $this->caReference;
        }
        if (isset($this->tbsReference) && !is_null($this->tbsReference)) {
            $info['tbsReference'] = $this->tbsReference;
        }
        if (isset($this->customerReference) && !is_null($this->customerReference)) {
            $info['customerReference'] = $this->customerReference;
        }
        if (isset($this->serialNumber) && !is_null($this->serialNumber)) {
            $info['serialNumber'] = $this->serialNumber;
        }
        
        return $info;
    }
    
    public function getXmlRpc()
    {
        $subXmlRpc = array(
            'refCA' => $this->caReference,
            'refTBS' => $this->tbsReference
        );
        
        if (isset($this->customerReference) && !is_null($this->customerReference)) {
            array_push($subXmlRpc, array('refClient' => $this->customerReference));
        }
        if (isset($this->serialNumber) && !is_null($this->serialNumber)) {
            array_push($subXmlRpc, array('nrSerie' => $this->serialNumber));
        }
        
        return $subXmlRpc;
    }
    
    public static function setCLIoptions(\cli\Arguments &$arguments)
    {
        $arguments->addOption('ca-ref', array('description' => 'Certification Authority\'s reference.'));
        $arguments->addOption('tbs-ref', array('description' => 'TBS Internet\'s reference.'));
        $arguments->addOption('customer-ref', array('description' => 'Customer\'s reference.'));
        $arguments->addOption('serial', array('description' => 'Certificate\'s serial number'));
    }

    public static function mergeOptionsWithConfig(array $options)
    {
        $config = array();
        
        if (isset($options['ca-ref']) && !is_null($options['ca-ref'])) {
            $config['CERTIFICATE']['caReference'] = $options['ca-ref'];
        }
        if (isset($options['tbs-ref']) && !is_null($options['tbs-ref'])) {
            $config['CERTIFICATE']['tbsReference'] = $options['tbs-ref'];
        }
        if (isset($options['customer-ref']) && !is_null($options['customer-ref'])) {
            $config['CERTIFICATE']['customerReference'] = $options['customer-ref'];
        }
        if (isset($options['serial']) && !is_null($options['serial'])) {
            $config['CERTIFICATE']['serialNumber'] = $options['serial'];
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    public static function load(array $info)
    {
        if (isset($info['caReference']) && !is_null($info['caReference'])) {
            $caReference = $info['caReference'];
        }
        if (isset($info['tbsReference']) && !is_null($info['tbsReference'])) {
            $tbsReference = $info['tbsReference'];
        }
        if (isset($info['customerReference']) && !is_null($info['customerReference'])) {
            $customerReference = $info['customerReference'];
        }
        if (isset($info['serialNumber']) && !is_null($info['serialNumber'])) {
            $serialNumber = $info['serialNumber'];
        }
        
        return new Certificate(
                $caReference,
                $tbsReference,
                $customerReference,
                $serialNumber
        );
    }
}
