<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\API\Entities;

/**
 * Abastrct representation of a generic Contact
 * Contacts used in TBSCertBot inherit from this "template"
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
abstract class Contact implements Entity
{
    protected $title;
    protected $firstName;
    protected $lastName;
    protected $function;
    protected $phoneNumber;
    protected $emailAddress;
    protected $language;
    
    public function __construct(
            $title,
            $firstName,
            $lastName,
            $function,
            $phoneNumber,
            $emailAddress,
            $language
            ) {
        $this->title = $title;
        $this->firstName = $firstName;
        $this->lastName = $lastName;
        $this->function = $function;
        $this->phoneNumber = $phoneNumber;
        $this->emailAddress = $emailAddress;
        $this->language = $language;
    }
    
    public function getTitle()
    {
        return $this->title;
    }
    
    public function getFirstName()
    {
        return $this->firstName;
    }
    
    public function getLastName()
    {
        return $this->lastName;
    }
    
    public function getFunction()
    {
        return $this->function;
    }
    
    public function getPhoneNumber()
    {
        return $this->phoneNumber;
    }
    
    public function getEmailAddress()
    {
        return $this->emailAddress;
    }
    
    public function getLanguage()
    {
        return $this->language;
    }
    

    /**
     * Checks the Title
     * 
     * @param boolean $checkLanguage If we want to check titles based on the language
     * @return boolean
     * @todo Centralize language/title information
     */
    private function checkTitle($checkLanguage = true)
    {
        if ($checkLanguage) {
            if ($this->language === 'EN') {
                if (in_array($this->title, array('Mr', 'Mrs', 'Miss', 'Ms', 'Dr', 'Mx'))) {
                    $check = true;
                } else {
                    $check = false;
                }
            } elseif (in_array($this->language, array('FR', 'ES'))) {
                if (in_array($this->title, array('Mr', 'Mrs', 'Miss', 'Dr'))) {
                    $check = true;
                } else {
                    $check = false;
                }
            } else {
                $check = false;
            }
        } else {
            if (in_array($this->title, array('Mr', 'Mrs', 'Miss', 'Ms', 'Dr', 'Mx'))) {
                $check = true;
            } else {
                $check = false;
            }
        }
        
        return $check;
    }
    
    /**
     * Checks if the language is accepted
     * 
     * @return boolean
     * @todo Centralize language information
     */
    private function checkLanguage()
    {
        if(in_array($this->language, array('FR', 'EN', 'ES'))) {
            $check = true;
        } else {
            $check = false;
        }
        
        return $check;
    }
    
    /**
     * Checks the attributes conformity
     * @return boolean
     * @todo Handle additional titles
     */
    private function checkAttributes()
    {
        if ($this->checkLanguage()
                && $this->checkTitle(true)
                && !empty($this->title)
                && !empty($this->firstName)
                && !empty($this->lastName)
                && !empty($this->function)
                && !empty($this->phoneNumber)
                && !empty($this->emailAddress)
            ) {
            $check = true;
        } else {
            $check = false;
        }
        return $check;
    }
    
    /**
     * Function used to validate the values of properties.
     * @todo WARNING => Hardcoded class names  in this method.
     * @todo Error management
     */
    public function validate()
    {   
        while (!$this->checkAttributes()) {
            if (get_class($this) == 'TBSCertBot\API\Entities\CorporateContact') {
                \cli\line('An issue occured with the CORPORATE CONTACT.');
            
                $contactType = 'corporate';
            } elseif (get_class($this) == 'TBSCertBot\API\Entities\LegalContact') {
                \cli\line('An issue occured with the LEGAL CONTACT.');
                
                $contactType = 'legal';
            } elseif (get_class($this) == 'TBSCertBot\API\Entities\TechnicalContact') {
                \cli\line('An issue occured with the TECHNICAL CONTACT.');
                
                $contactType = 'technical';
            }
                     
            if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                \cli\line('Please provide or correct the following information:');
                
                $this->interactiveFallback();
            } else {
                throw new \Exception('Please check and correct the ' . $contactType . ' contact information');
            }
        }
    }
    
    /**
     * Interactive fallback for the title, to be used by the public interactiveFallback()
     * @todo Add additional titles
     * @todo Centralize language/title information
     */
    private function interactiveFallbackTitle()
    {
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
            \cli\line('Please in put the contact title:');
            
            if (in_array($this->language, array('FR', 'ES'))) {
                \cli\line('Possible values: Mr, Mrs, Miss, Dr');
            } elseif ($this->language === 'EN') {
                \cli\line('Possible Values: Mr, Mrs, Miss, Ms, Dr, Mx');
            } else {
                \cli\line('Possible values: Mr, Mrs, Miss, Dr');
            }
        }
        
        if (empty($this->title) || !in_array($this->title, array('Mr', 'Mrs', 'Miss'))) {
            $this->title = \cli\prompt('   - Title');
        } else {
            $this->title = \cli\prompt('   - Title [' . $this->title . ']', $this->title);
        }
    }
    
    /**
     * Interactive fallback for the language, to be used by the public interactiveFallback()
     * @todo Improve language management
     */
    private function interactiveFallbackLanguage()
    {
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
            \cli\line('Please input the contact language:');
            \cli\line('Possible values: EN, FR');
        }
        
        if (empty($this->language)) {
            $this->language = \cli\prompt('   - Language');
        } else {
            $this->language = \cli\prompt('   - Language [' . $this->language . ']', $this->language);
        }
    }
    
    /**
     * Interactive fallback that can be called if the values of the entity are missing or incorrect
     * @todo Check phone number syntax?
     * @todo Check email syntax?
     * @todo Handle additional titles
     */
    public function interactiveFallback()
    {
        //We ask for the language first so that we can check the title
        $this->interactiveFallbackLanguage();
        
        $this->interactiveFallbackTitle();
        
        if (empty($this->firstName)) {
            $this->firstName = \cli\prompt('   - First name');
        } else {
            $this->firstName = \cli\prompt('   - First name [' . $this->firstName . ']', $this->firstName);
        }
        
        if (empty($this->lastName)) {
            $this->lastName = \cli\prompt('   - Last name');
        } else {
            $this->lastName = \cli\prompt('   - Last name [' . $this->lastName . ']', $this->lastName);
        }
        
        if (empty($this->function)) {
            $this->function = \cli\prompt('   - Function');
        } else {
            $this->function = \cli\prompt('   - Function [' . $this->function . ']', $this->function);
        }
        
        if (empty($this->phoneNumber)) {
            $this->phoneNumber = \cli\prompt('   - Phone number');
        } else {
            $this->phoneNumber = \cli\prompt('   - Phone number [' . $this->phoneNumber . ']', $this->phoneNumber);
        }
        
        if (empty($this->emailAddress)) {
            $this->emailAddress = \cli\prompt('   - Email address');
        } else {
            $this->emailAddress = \cli\prompt('   - Email address [' . $this->emailAddress . ']', $this->emailAddress);
        }
        
    }
    
    public function save()
    {
        $info = array();
        
        if (isset($this->title) && !is_null($this->title)) {
            $info['title'] = $this->title;
        }
        if (isset($this->firstName) && !is_null($this->firstName)) {
            $info['firstName'] = $this->firstName;
        }
        if (isset($this->lastName) && !is_null($this->lastName)) {
            $info['lastName'] = $this->lastName;
        }
        if (isset($this->function) && !is_null($this->function)) {
            $info['function'] = $this->function;
        }
        if (isset($this->phoneNumber) && !is_null($this->phoneNumber)) {
            $info['phoneNumber'] = $this->phoneNumber;
        }
        if (isset($this->emailAddress) && !is_null($this->emailAddress)) {
            $info['emailAddress'] = $this->emailAddress;
        }
        if (isset($this->language) && !is_null($this->language)) {
            $info['language'] = $this->language;
        }
        
        return $info;
    }
    
    abstract public function getXmlRpc();
    
    /**
     * Deserialization from the stored file
     * @param string $json
     * @todo Implement? Would this even be useful?
     */
    public static function load(array $json) 
    {
        //Do nothing
    }
}
