<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\API\Entities;

/**
 * Representation of an Organization
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Organization implements Entity
{
    private static $DEFAULT_COUNTRIES_JSON;
   
    private static $DEFAULT_ORGANIZATION_TYPES_JSON;
    
    private static $DEFAULT_NAF_CODES_JSON;
    
    private $name;
    private $address = array();
    private $city;
    private $postalCode;
    private $state;
    private $country;
    private $phoneNumber;
    
    // Optional values.
    private $type;
    private $number;
    private $nafCode;
    private $website; 
    private $vatNumber;
    private $accountingEmailAddress;
    
    public function __construct(
            $name,
            $address,
            $city,
            $postalCode,
            $state,
            $country,
            $phoneNumber,
            $type = null,
            $number = null,
            $nafCode = null,
            $website = null,
            $vatNumber = null,
            $accountingEmailAddress = null
            ) {

        $this->name = $name;
        $this->address = $address;
        $this->city = $city;
        $this->postalCode = $postalCode;
        $this->state = $state;
        $this->country = $country;
        $this->phoneNumber = $phoneNumber;
        
        // Optional values.
        $this->type = $type;
        $this->number = $number;
        $this->nafCode = $this->translateNafCode($nafCode);
        $this->website = $website;
        $this->vatNumber = $vatNumber;
        $this->accountingEmailAddress = $accountingEmailAddress;
    }
    
    private static function getDefaultCountriesJson()
    {
        if (empty(self::$DEFAULT_COUNTRIES_JSON)) {
            self::$DEFAULT_COUNTRIES_JSON = 'data' . DIRECTORY_SEPARATOR . 'internal'
            . DIRECTORY_SEPARATOR . 'values' . DIRECTORY_SEPARATOR . 'Countries.json';
        }
        
        return self::$DEFAULT_COUNTRIES_JSON;
    }
    
    private static function getDefaultOrganizationTypesJson()
    {
        if (empty(self::$DEFAULT_ORGANIZATION_TYPES_JSON)) {
            self::$DEFAULT_ORGANIZATION_TYPES_JSON = 'data' . DIRECTORY_SEPARATOR . 'internal'
            . DIRECTORY_SEPARATOR . 'values' . DIRECTORY_SEPARATOR . 'OrganizationTypes.json';
        }
        
        return self::$DEFAULT_ORGANIZATION_TYPES_JSON;
    }
    
    private static function getDefaultNafCodesJson()
    {
        if (empty(self::$DEFAULT_NAF_CODES_JSON)) {
            self::$DEFAULT_NAF_CODES_JSON = 'data' . DIRECTORY_SEPARATOR . 'internal'
            . DIRECTORY_SEPARATOR . 'values' . DIRECTORY_SEPARATOR . 'NafCodes.json';
        }
        
        return self::$DEFAULT_NAF_CODES_JSON;
    }
    
    public function getName()
    {
        return $this->name;
    }
    
    public function getAddress()
    {
        return $this->address;
    }
    
    public function getCity()
    {
        return $this->city;
    }
    
    public function getPostalCode()
    {
        return $this->postalCode;
    }
    
    public function getState()
    {
        return $this->state;
    }
    
    public function getCountry()
    {
        return $this->country;
    }
    
    public function getPhoneNumber()
    {
        return $this->phoneNumber;
    }
    
    public function getType()
    {
        return $this->type;
    }
    
    public function getNumber()
    {
        return $this->number;
    }
    
    public function getNafCode()
    {
        return $this->nafCode;
    }
    
    public function getWebsite()
    {
        return $this->website;
    }
    
    public function getVATNumber()
    {
        return $this->vatNumber;
    }
    
    public function getAccountingEmailAddress()
    {
        return $this->accountingEmailAddress;
    }
    
    private function translateNafCode($nafCode)
    {
        if (is_numeric($nafCode) && (int)$nafCode == $nafCode) {
            return $nafCode;
        } else {
            $nafCodes = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultNafCodesJson()),
                        true
            );
            
            if (in_array($nafCode, array_keys($nafCodes))) {
                return $nafCodes[$nafCode];
            } else {
                return $nafCode;
            }
        }
    }
    
    private function checkCountry()
    {
        if (!empty($this->country) && key_exists(
                $this->country,
                \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultCountriesJson()),
                        true
                        )
                )) {
            $check = true;
        } else {
            $check = false;
        }
        return $check;
    }
    
    /**
     * 
     * Checks the concordance between the organization type and its country
     * 
     * @return boolean
     */
    private function checkTypeByCountry()
    {
        if (!empty($this->type) && !empty($this->country)) {
            $organizationTypes = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultOrganizationTypesJson()),
                        true
                        );
          $typeCheck = false;
          foreach ($organizationTypes as $type => $countryAndDescription) {
               if ($this->type == $type && $this->country == $countryAndDescription['country']) {
                   $typeCheck |= true;
               }
           }
        } else {
            $typeCheck = true;
        }
        return $typeCheck;
    }
    
    /**
     * 
     * @return bool Whether the check was successful
     * @todo Check phone number syntax?
     */
    private function checkAttributes()
    {   
        $check = true;
        
        if (empty($this->name)
                || empty($this->address[0])
                || empty($this->city)
                || !$this->checkCountry()
                || empty($this->phoneNumber)
                ) {
            $check = false;
        }
        
        // Complementary test on type.
        $check &= $this->checkTypeByCountry();
        
        return $check;
    }
    
    /**
     * Function used to validate the values of properties.
     * WARNING => Hardcoded class names in this method.
     * @todo Add state to mandatory fields?
     * @todo Error management
     */
    public function validate()
    {
        while (!$this->checkAttributes()) {
            if (get_class($this) == 'TBSCertBot\API\Entities\Organization') {
                \cli\line('An issue occured with the ORGANIZATION.');
                
                $organizationType = 'organization';
            } elseif (get_class($this) == 'TBSCertBot\API\Entities\BillingOrganization') {
                \cli\line('An issue occured with the BILLING ORGANIZATION.');
            
                $organizationType = 'billing organization';
            }
            
            if (!\TBSCertBot\Config::getConfig()->getBatch()) {
               \cli\line('Please provide or correct the following information');
                $this->interactiveFallback();
            } else {
                throw new \Exception('Please check and correct the ' . $organizationType . '.');
            }
        }
    }
    
    /**
     * Interactive fallback for the address, to be used by the public interactiveFallback()
     * 
     * @todo Handle the variation of addresses fields properly
     */
    private function interactiveFallbackAddress()
    {
        if (empty($this->address[0])) {
            $this->address[0] = \cli\prompt('   - Address (1/3)');
            $this->address[1] = \cli\prompt('   - Address (2/3)', '');
            $this->address[2] = \cli\prompt('   - Address (3/3)', '');
        } else {
            $this->address[0] = \cli\prompt('   - Address (1/3) [' . $this->address[0] . ']',
                    $this->address[0]);
            
          if (empty($this->address[1])) {
              $this->address[1] = \cli\prompt('   - Address (2/3)', '');
          } else {
              $this->address[1] = \cli\prompt('   - Address (2/3) [' . $this->address[1] . ']',
                      $this->address[1]);
          }
          
            if (empty($this->address[2])) {
                $this->address[2] = \cli\prompt('   - Address (3/3)', '');
            } else {
                $this->address[2] = \cli\prompt('   - Address (3/3) [' . $this->address[2] . ']',
                        $this->address[2]);
            }
        }
    }
    
    /**
     * Interactive fallback for the NAF Code, to be used by the public interactiveFallback
     * 
     * @todo Add a NAF Code help and display it here
     */
    private function interactiveFallbackNafCode()
    {
        if (empty($this->nafCode)) {
            $this->nafCode = \cli\prompt('   - Naf Code', '');
        } else {
            $this->nafCode = \cli\prompt('   - Naf Code [' . $this->nafCode . ']', $this->translateNafCode($this->nafCode));
        }
    }
    
    /**
     * Interactive fallback for the Accounting Email, to be used by the public interactiveFallback
     */
    private function interactiveFallbackAccountingEmail()
    {
        if (get_class($this) == 'TBSCertBot\API\Entities\BillingOrganization') {
            if (empty($this->accountingEmailAddress)) {
                $this->accountingEmailAddress = \cli\prompt('   - Accounting Email Address');
            } else {
                 $this->accountingEmailAddress = \cli\prompt('   - Accounting Email Address [' 
                         . $this->accountingEmailAddress . ']', 
                         $this->accountingEmailAddress);
            }
        }
    }
    
    /**
     * Interactive fallback that can be called if the values of the entity are missing or incorrect
     * 
     * @todo Check phone number syntax?
     */
    public function interactiveFallback()
    {
        if (empty($this->name)) {
            $this->name = \cli\prompt('   - Name');
        } else {
             $this->name = \cli\prompt('   - Name [' . $this->name . ']', $this->name);
        }
        
        $this->interactiveFallbackAddress();
        
        if (empty($this->city)) {
            $this->city = \cli\prompt('   - City');
        } else {
            $this->city = \cli\prompt('   - City [' . $this->city . ']', $this->city);
        }
        
        if (empty($this->postalCode)) {
            $this->postalCode = \cli\prompt('   - Postal Code');
        } else {
            $this->postalCode = \cli\prompt('   - Postal Code [' . $this->postalCode . ']', $this->postalCode);
        }
        
        if (empty($this->state)) {
            $this->state = \cli\prompt('   - State');
        } else {
            $this->state = \cli\prompt('   - State [' . $this->state . ']', $this->state);
        }
        
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
            \cli\line('Please input the country code:');
            self::countryHelp();
        }
        if (empty($this->country)) {
            $this->country = \cli\prompt('   - Country');
        } else {
            $this->country = \cli\prompt('   - Country [' . $this->country . ']', $this->country);
        }
        
        if (empty($this->phoneNumber)) {
            $this->phoneNumber = \cli\prompt('   - Phone number');
        } else {
            $this->phoneNumber = \cli\prompt('   - Phone number [' . $this->phoneNumber . ']',
                    $this->phoneNumber);
        }
        
        // Optional values.
        //Note the default empty string as a prompt parameter
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
            \cli\line('Please input the organization type:');
            self::typeHelp();
        }
        if (empty($this->type)) {
            $this->type = \cli\prompt('   - Type', '');
        } else {
            $this->type = \cli\prompt('   - Type [' . $this->type . ']', $this->type);
        }
        
        if (empty($this->number)) {
            $this->number = \cli\prompt('   - Number', '');
        } else {
            $this->number = \cli\prompt('   - Number [' . $this->number . ']', $this->number);
        }
        
        $this->interactiveFallbackNafCode();
        
        if (empty($this->website)) {
            $this->website = \cli\prompt('   - Website', '');
        } else {
            $this->website = \cli\prompt('   - Website [' . $this->website . ']', $this->website);
        }
        
        if (empty($this->vatNumber)) {
            //VAT Number is mandatory for Billing organizations
            if (get_class($this) == 'TBSCertBot\API\Entities\BillingOrganization') {
                $this->vatNumber = \cli\prompt('   - VAT Number');
            } else {
                $this->vatNumber = \cli\prompt('   - VAT Number','');
            }
        } else {
            $this->vatNumber = \cli\prompt('   - VAT Number [' . $this->vatNumber . ']', $this->vatNumber);
        }
        

        $this->interactiveFallbackAccountingEmail();
    }
    
    public function save()
    {
        $info = array();
        
        if (isset($this->name) && !is_null($this->name)) {
            $info['name'] = $this->name;
        }
        if (isset($this->address) && !is_null($this->address)) {
            $info['address'] = $this->address;
        }
        if (isset($this->city) && !is_null($this->city)) {
            $info['city'] = $this->city;
        }
        if (isset($this->postalCode) && !is_null($this->postalCode)) {
            $info['postalCode'] = $this->postalCode;
        }
        if (isset($this->state) && !is_null($this->state)) {
            $info['state'] = $this->state;
        }
        if (isset($this->country) && !is_null($this->country)) {
            $info['country'] = $this->country;
        }
        if (isset($this->phoneNumber) && !is_null($this->phoneNumber)) {
            $info['phoneNumber'] = $this->phoneNumber;
        }
        if (isset($this->type) && !is_null($this->type)) {
            $info['type'] = $this->type;
        }
        if (isset($this->number) && !is_null($this->number)) {
            $info['number'] = $this->number;
        }
        if (isset($this->nafCode) && !is_null($this->nafCode)) {
            $info['nafCode'] = $this->nafCode;
        }
        if (isset($this->website) && !is_null($this->website)) {
            $info['website'] = $this->website;
        }
        if (isset($this->vatNumber) && !is_null($this->vatNumber)) {
            $info['vatNumber'] = $this->vatNumber;
        }
        if (isset($this->accountingEmailAddress) && !is_null($this->accountingEmailAddress)) {
            $info['accountingEmailAddress'] = $this->accountingEmailAddress;
        }
        
        return $info;
    }
    
    public function getXmlRpc()
    {
        $return = array(
          'nom' => $this->name,
          'ville' => $this->city,
          'codePostal' => $this->postalCode,
          'state' => $this->state,
          'pays' => $this->country,
          'telephone' => $this->phoneNumber,
          'typeOrg' => $this->type,
          'siren' => $this->number
        );
        
        //Only one of those three is necessary.
        //Validation is ensured in the validation method.
        if (is_array($this->address) && array_key_exists(0, $this->address) && !empty($this->address[0])) {
            $return['adresse1'] = $this->address[0];
        }
        if (is_array($this->address) && array_key_exists(1, $this->address) && !empty($this->address[1])) {
            $return['adresse2'] = $this->address[1];
        }
        if (is_array($this->address) && array_key_exists(2, $this->address) && !empty($this->address[2])) {
            $return['adresse3'] = $this->address[2];
        }
        
        //Optional fields
        if ($this->nafCode) $return['codeNaf'] = $this->nafCode;
        if ($this->website) $return['siteweb'] = $this->website;

        //Only set for billing organizations
        if ($this->vatNumber) $return['numTVA'] = $this->vatNumber;
        if ($this->accountingEmailAddress) $return['melcompta'] = $this->accountingEmailAddress;
        
        return $return;
    }
    
    public static function setCLIoptions(\cli\Arguments &$arguments)
    {
        $arguments->addOption('org-name', array('description' => 'Organization\'s name.'));
        $arguments->addOption('org-address-1', array('description' => 'Organization\'s address.'));
        $arguments->addOption('org-address-2', array('description' => 'Organization\'s address.'));
        $arguments->addOption('org-address-3', array('description' => 'Organization\'s address.'));
        $arguments->addOption('org-city', array('description' => 'Organization\'s city.'));
        $arguments->addOption('org-postalcode', array('description' => 'Organization\'s postal code.'));
        $arguments->addOption('org-state', array('description' => 'Organization\'s state.'));
        $arguments->addOption('org-country', array('description' => 'Organization\'s country.'));
        $arguments->addOption('org-phone', array('description' => 'Organization\'s phone number.'));
        $arguments->addOption('org-type', array('description' => 'Organization\'s type.'));
        $arguments->addOption('org-number', array('description' => 'Organization\'s identification number.'));
        $arguments->addOption('org-nafcode', array('description' => 'Organization\'s NAF code (France only).'));
        $arguments->addOption('org-website', array('description' => 'Organization\'s website.'));
        $arguments->addOption('org-vat', array('description' => 'Organization\'s VAT number.'));
        $arguments->addOption('org-accounting-email', array('description' => 'Organization\'s accounting e-mail address.'));
    }
    
    public static function mergeOptionsWithConfig(array $options)
    {
        $config = array();
        
        if (isset($options['org-name']) && !is_null($options['org-name'])) {
            $config['ORGANIZATION']['name'] = $options['org-name'];
        }
        if (isset($options['org-address-1']) && !is_null($options['org-address-1'])) {
            $config['ORGANIZATION']['address'][0] = $options['org-address-1'];
        }
        if (isset($options['org-address-2']) && !is_null($options['org-address-2'])) {
            $config['ORGANIZATION']['address'][1] = $options['org-address-2'];
        }
        if (isset($options['org-address-3']) && !is_null($options['org-address-3'])) {
            $config['ORGANIZATION']['address'][2] = $options['org-address-3'];
        }
        if (isset($options['org-city']) && !is_null($options['org-city'])) {
            $config['ORGANIZATION']['city'] = $options['org-city'];
        }
        if (isset($options['org-postalcode']) && !is_null($options['org-postalcode'])) {
            $config['ORGANIZATION']['postalCode'] = $options['org-postalcode'];
        }
        if (isset($options['org-state']) && !is_null($options['org-state'])) {
            $config['ORGANIZATION']['state'] = $options['org-state'];
        }
        if (isset($options['org-country']) && !is_null($options['org-country'])) {
            $config['ORGANIZATION']['country'] = $options['org-country'];
        }
        if (isset($options['org-phone']) && !is_null($options['org-phone'])) {
            $config['ORGANIZATION']['phoneNumber'] = $options['org-phone'];
        }
        if (isset($options['org-type']) && !is_null($options['org-type'])) {
            $config['ORGANIZATION']['type'] = $options['org-type'];
        }
        if (isset($options['org-number']) && !is_null($options['org-number'])) {
            $config['ORGANIZATION']['number'] = $options['org-number'];
        }
        if (isset($options['org-nafcode']) && !is_null($options['org-nafcode'])) {
            $config['ORGANIZATION']['nafCode'] = $options['org-nafcode'];
        }
        if (isset($options['org-website']) && !is_null($options['org-website'])) {
            $config['ORGANIZATION']['website'] = $options['org-website'];
        }
        if (isset($options['org-vat']) && !is_null($options['org-vat'])) {
            $config['ORGANIZATION']['vatNumber'] = $options['org-vat'];
        }
        if (isset($options['org-accounting-email']) && !is_null($options['org-accounting-email'])) {
            $config['ORGANIZATION']['accountingEmailAddress'] = $options['org-accounting-email'];
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    public static function countryHelp()
    {   
        $table = new \cli\Table();
        $table->setHeaders(array('code', 'country'));
        $rows = array();
        $countries = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultCountriesJson()),
                        true
        );
        
        foreach ($countries as $code => $country) {
            array_push($rows, array($code, $country));
        }
        
        $table->setRows($rows);
        $table->display();
    }
    
    public static function nafCodeHelp()
    {
        $table = new \cli\Table();
        $table->setHeaders(array('NAF Code', 'TBSCertBot code'));
        $rows = array();
        $nafCodes = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultNafCodesJson()),
                        true
        );
        
        foreach ($nafCodes as $nafCode => $tbsCode) {
            array_push($rows, array($nafCode, $tbsCode));
        }
        
        $table->setRows($rows);
        $table->display();
    }
    
    public static function typeHelp()
    {
        $table = new \cli\Table();
        $table->setHeaders(array('type', 'country', 'description'));
        $rows = array();
        $types = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultOrganizationTypesJson()),
                        true
                );
        
        foreach ($types as $type => $details) {
            array_push($rows, array(
                $type,
                $details['country'],
                $details['description']
                )
            );
        }
        
        $table->setRows($rows);
        $table->display();
    }
    
    public static function load(array $info)
    {
        if (isset($info['name']) && !is_null($info['name'])) {
            $name = $info['name'];
        }
        if (isset($info['address']) && !is_null($info['address'])) {
            $address = $info['address'];
        }
        if (isset($info['city']) && !is_null($info['city'])) {
            $city = $info['city'];
        }
        if (isset($info['postalCode']) && !is_null($info['postalCode'])) {
            $postalCode = $info['postalCode'];
        }
        if (isset($info['state']) && !is_null($info['state'])) {
            $state = $info['state'];
        }
        if (isset($info['country']) && !is_null($info['country'])) {
            $country = $info['country'];
        }
        if (isset($info['phoneNumber']) && !is_null($info['phoneNumber'])) {
            $phoneNumber = $info['phoneNumber'];
        }
        if (isset($info['type']) && !is_null($info['type'])) {
            $type = $info['type'];
        }
        if (isset($info['number']) && !is_null($info['number'])) {
            $number = $info['number'];
        }
        if (isset($info['nafCode']) && !is_null($info['nafCode'])) {
            $nafCode = $info['nafCode'];
        }
        if (isset($info['website']) && !is_null($info['website'])) {
            $website = $info['website'];
        }
        if (isset($info['vatNumber']) && !is_null($info['vatNumber'])) {
            $vatNumber = $info['vatNumber'];
        }
        if (isset($info['accountingEmailAddress']) && !is_null($info['accountingEmailAddress'])) {
            $accountingEmailAddress = $info['accountingEmailAddress'];
        }
        
        return new Organization(
                $name,
                $address,
                $city,
                $postalCode,
                $state,
                $country,
                $phoneNumber,
                $type,
                $number,
                $nafCode,
                $website,
                $vatNumber,
                $accountingEmailAddress
        );
    }
}
