<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\API\Entities;

/**
 * Representation of a Payment
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Payment implements Entity
{
    private static $DEFAULT_PAYMENT_MODES_JSON;
    
    /**
     *
     * @var string
     */
    private $mode;
    
    /**
     *
     * @var bool
     */
    private $quotation;
    
    public function __construct($mode, $quotation = false)
    {
        $this->mode = $mode;
        $this->quotation = (bool) $quotation;
    }
    
    private static function getDefaultPaymentModes()
    {
        if (empty(self::$DEFAULT_PAYMENT_MODES_JSON)) {
            self::$DEFAULT_PAYMENT_MODES_JSON = 'data' . DIRECTORY_SEPARATOR . 'internal'
             . DIRECTORY_SEPARATOR . 'values' . DIRECTORY_SEPARATOR . 'PaymentModes.json';
        }
        
        return self::$DEFAULT_PAYMENT_MODES_JSON;
    }
    
    public function getMode()
    {
        return $this->mode;
    }
    
    public function getQuotation()
    {
        return $this->quotation;
    }

    private static function checkMode($mode)
    {
        if (!empty($mode) && key_exists(
                $mode,
                \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultPaymentModes()),
                        true
                        )
                )) {
            $check = true;
        } else {
            $check = false;
        }
        return $check;
    }
    
    private function checkAttributes()
    {
        return self::checkMode($this->mode);
    }
    
    /**
     * Function used to validate the values of properties.
     * @todo Error management
     */
    public function validate()
    {
        while (!$this->checkAttributes()) {
            \cli\line('An issue occured with the PAYMENT.');
            if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                \cli\line('Please provide or correct the following information:');
                $this->interactiveFallback();
            } else {
                throw new\Exception('Please check and correct the payment information.');
            }
        }
    }

    public function interactiveFallback()
    {
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
            \cli\line('Please select your payment mode:');
            self::modeHelp();
        }
        
        if (!$this->checkMode()) {
                $this->mode = \cli\prompt('   - Mode');
        }
        
        //A question mark will be added
        $quotationChoice = \cli\choose('   -  Quotation', $choice = 'yn', $default = 'n');
        
        if($quotationChoice == 'y') $this->quotation = true;
        else $this->quotation = false;
    }
    
    /**
     * 
     * Builds the array that will be used for serialization
     * 
     * @return array
     * @todo Secure Quotation
     */
    public function save()
    {
        $info = array();
        
        if (isset($this->mode) && !is_null($this->mode)) {
            $info['mode'] = $this->mode;
        }
        if (isset($this->quotation) && !is_null($this->quotation)) {
            $info['quotation'] = $this->quotation;
        }
        
        return $info;
    }
    
    public function getXmlRpc()
    {
        return array(
            'modePaiement' => $this->mode,
            'devis' => \TBSCertBot\API\Helper::mapBooleanToFrench($this->quotation)
        );
    }
    
    public static function setCLIoptions(\cli\Arguments &$arguments)
    {
        $arguments->addOption('payment', array('description' => 'Payment mode.'));
        
        $arguments->addFlag('quotation', ['description' => 'Quotation requested for the order.',
                                          'default' => \TBSCertBot\Config::getConfig()->getPayment()->getQuotation()]);
    }
    
    public static function mergeOptionsWithConfig(array $options)
    {
        $config = array();
        
        if (isset($options['payment']) && !is_null($options['payment'])) {
            $config['PAYMENT']['mode'] = $options['payment'];
        }
        if (isset($options['quotation']) && !is_null($options['quotation'])) {
            $config['PAYMENT']['quotation'] = $options['quotation'];
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    public static function modeHelp()
    {   
        $table = new \cli\Table();
        $table->setHeaders(array('mode', 'description'));
        $rows = array();
        $modes = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultPaymentModes()),
                        true
                );
        
        foreach ($modes as $mode => $description) {
            array_push($rows, array($mode, $description));
        }
        
        $table->setRows($rows);
        $table->display();
    }
    
    /**
     * Deserialization from the stored file
     * 
     * @param array $info
     * @return \TBSCertBot\API\Entities\Payment
     * @todo Security
     */
    public static function load(array $info)
    {
        if (isset($info['mode']) && !is_null($info['mode']) && self::checkMode($info['mode'])) {
            $mode = $info['mode'];
        } else {
            $mode = \TBSCertBot\Config::getConfig()->getPayment()->getMode();
        }
        
        if (isset($info['quotation']) && !is_null($info['quotation'])) {
            $quotation = $info['quotation'];
        } else {
            $quotation = \TBSCertBot\Config::getConfig()->getPayment()->getQuotation();
        }
        
        return new Payment(
                $mode,
                $quotation
        );
    }
}
