<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\API\Entities;

/**
 * Representation of a Product
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Product implements Entity
{
    private static $DEFAULT_PRODUCTS_JSON;
    
    private $code;
    private $validity;
    
    public function __construct($code, $validity)
    {
        $this->code = $code;
        $this->validity = $validity;
    }
    
    private function getDefaultProductsJson()
    {
        if (empty(self::$DEFAULT_PRODUCTS_JSON)) {
            self::$DEFAULT_PRODUCTS_JSON = 'data' . DIRECTORY_SEPARATOR . 'internal' 
            . DIRECTORY_SEPARATOR . 'values' . DIRECTORY_SEPARATOR . 'Products.json';
        }
        
        return self::$DEFAULT_PRODUCTS_JSON;
    }
    
    public function getCode()
    {
        return $this->code;
    }
    
    public function getValidity()
    {
        return $this->validity;
    }
    
    public function needsDcv()
    {
        if (!empty($this->code)) {
            $products = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultProductsJson()),
                        true
            );
            
            if ($products[$this->code]['dcv'] == true) {
                $dcvNeeded = true;
            } else {
                $dcvNeeded = false;
            }
        } else {
            $dcvNeeded = false;
        }
        
        return $dcvNeeded;
    }
    
    public function isDcvMandatoryToOrder()
    {
        if (!empty($this->code)) {
            $products = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultProductsJson()),
                        true
            );
            
            if (isset($products[$this->code]['dcvMandatoryToOrder'])
                && $products[$this->code]['dcvMandatoryToOrder'] == true) {
                
                $dcvMandatoryToOrder = true;
            } else {
                $dcvMandatoryToOrder = false;
            }
        } else {
            $dcvMandatoryToOrder = false;
        }
        
        return $dcvMandatoryToOrder;
    }
    
    private function checkCodeAndValidity()
    {
        if (!empty($this->code) && !empty($this->validity)) {
            $products = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultProductsJson()),
                        true
                        );
          $check = false;
          foreach ($products as $code => $details) {
               if ($this->code == $code && in_array($this->validity, $details['validity'])) {
                   $check = true;
               }
           }
        } else {
            $check = false;
        }
        return $check;
    }
    
    private function checkAttributes()
    {
        return $this->checkCodeAndValidity();
    }
    
    /**
     * 
     * Function used to validate the values of properties.
     * 
     * @todo Error management
     */
    public function validate()
    {
        while (!$this->checkAttributes()) {
            if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                \cli\line('An issue occured with the PRODUCT. Please provide or correct the following information:');
                               
                $this->interactiveFallback();
            } else {
                throw new \Exception('The product or its validity has not been correctly specified. Please provide a product.');
            }
        }
    }

    public function interactiveFallback()
    {
        //If the help flag is set, we'll display the whole product table
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
            self::productHelp();

            \cli\line('Please input a product:');
        }
        
        $this->code = \cli\prompt('   - Code');
        
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
            \cli\line('Please input the desired validity periods in years, or with the suffix d for days');
            \cli\line('Valid period examples: 1, 2, 30d, 90d');
        }
        
        $this->validity = \cli\prompt('   - Validity');
    }
    
    public function save()
    {
        $info = array();
        
        if (isset($this->code) && !is_null($this->code)) {
            $info['code'] = $this->code;
        }
        if (isset($this->validity) && !is_null($this->validity)) {
            $info['validity'] = $this->validity;
        }
        
        return $info;
    }
    
    public function getXmlRpc()
    {
        return array(
            'code' => $this->code,
            'dureeVie' => $this->validity
        );
    }
    
    public static function setCLIoptions(\cli\Arguments &$arguments)
    {
        $arguments->addOption('product', array('description' => 'Product code.'));
        $arguments->addOption('validity', array('description' => 'Requested validity for the ordered product.'));
    }
    
    public static function mergeOptionsWithConfig(array $options)
    {
        $config = array();
        
        if (isset($options['product']) && !is_null($options['product'])) {
            $config['PRODUCT']['code'] = $options['product'];
        }
        if (isset($options['validity']) && !is_null($options['validity'])) {
            $config['PRODUCT']['validity'] = $options['validity'];
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    public static function producthelp()
    {
        $table = new \cli\Table();
        $table->setHeaders(array('code', 'description', 'validity', 'license'));
        $rows = array();
        $products = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultProductsJson()),
                        true
                );
        
        foreach ($products as $code => $details) {
            array_push($rows, array(
                $code,
                $details['description'],
                (!empty($details['validity']) && is_array($details['validity'])) ? implode(', ', $details['validity']) : '',
                (string)\TBSCertBot\API\Helper::mapBooleanToInt($details['license'])
                )
            );
        }
        
        $table->setRows($rows);
        $table->display();
    }
    
    public static function load(array $info)
    {
        if (isset($info['code']) && !is_null($info['code'])) {
            $code = $info['code'];
        }
        if (isset($info['validity']) && !is_null($info['validity'])) {
            $validity = $info['validity'];
        }
        
        return new Product(
                $code,
                $validity
        );
    }
}
