<?php

/*
 * This file is part of TBSCertBot.
 *
 * Copyright (c) 1996-2016 TBS Internet
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\API\Entities;

/**
 * Representation of a Request
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Request implements Entity
{
    private static $DEFAULT_SOFTWARES_JSON;

    private $software;
    private $csr;
    private $mainDomain;
    private $domains;

    private $domainControlValidation;

    private $licenseNumber;
    private $cloningReferences;
    private $guidanceOption;
    private $hashAlgorithm;
    
    private $uniqueIdentifier;
    
    private $domainControlValidationUniqueValue;

    public function __construct(
            $software,
            $csr,
            $mainDomain,
            $domains,
            $domainControlValidation,
            $licenseNumber = null,
            $cloningReferences = null,
            $guidanceOption = false,
            $hashAlgorithm = null,
            $domainControlValidationUniqueValue = null
            ) {
        $this->software = $software;
        $this->csr = $csr;
        $this->mainDomain = $mainDomain;
        $this->domains = $domains;
        $this->domainControlValidation = $domainControlValidation;
        $this->licenseNumber = $licenseNumber;
        $this->cloningReferences = $cloningReferences;
        $this->guidanceOption = (bool)$guidanceOption;
        $this->hashAlgorithm = $hashAlgorithm;
        
        try {
            $this->uniqueIdentifier = \TBSCertBot\Crypto\Helper::generateUuid();
        } catch (\Exception $e) {
            throw $e;
        }
        
        $this->domainControlValidationUniqueValue = $domainControlValidationUniqueValue;
        
    }
    
    private static function getDefaultSoftwaresJson()
    {
        if (empty(self::$DEFAULT_SOFTWARES_JSON)) {
            self::$DEFAULT_SOFTWARES_JSON = 'data' . DIRECTORY_SEPARATOR . 'internal'
            . DIRECTORY_SEPARATOR . 'values' . DIRECTORY_SEPARATOR . 'Softwares.json';
        }
        
        return self::$DEFAULT_SOFTWARES_JSON;
    }

    public function getSoftware()
    {
        return $this->software;
    }

    public function getCSR()
    {
        return $this->csr;
    }

    public function setCSR($csr)
    {
        $this->csr = $csr;
    }
    
    public function setDomainControlValidationUniqueValue($domainControlValidationUniqueValue)
    {
        $this->domainControlValidationUniqueValue = $domainControlValidationUniqueValue;
    }

    /**
     * @todo Implement this method?
     */
    public function getKey()
    {

    }

    public function getMainDomain()
    {
        return $this->mainDomain;
    }

    public function setMainDomain($mainDomain)
    {
        $this->mainDomain = $mainDomain;
    }

    public function getDomains()
    {
        return $this->domains;
    }

    public function setDomains($domains)
    {
        $this->domains = $domains;
    }

    public function getDomainControlValidation()
    {
        return $this->domainControlValidation;
    }

    public function setDomainControlValidation($domainControlValidation)
    {
        $this->domainControlValidation = $domainControlValidation;
    }

    public function getLicenseNumber()
    {
        return $this->licenseNumber;
    }

    public function getCloningReference()
    {
        return $this->cloningReferences;
    }

    public function getGuidanceOption()
    {
        return $this->guidanceOption;
    }

    public function getHashAlgorithm()
    {
        return $this->hashAlgorithm;
    }

    public function getUniqueIdentifier()
    {
        return $this->uniqueIdentifier;
    }
    
    public function getDomainControlValidationUniqueValue()
    {
        return $this->domainControlValidationUniqueValue;
    }
    
    private function checkSoftware()
    {
        if (!empty($this->software) && key_exists($this->software, \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                file_get_contents(self::getDefaultSoftwaresJson()),
                true
                ))
         ) {
            $check = true;
        } else {
            $check = false;
        }
        return $check;
    }

    private function checkUniqueIdentifier()
    {
        return \TBSCertBot\Crypto\Helper::isUuid($this->uniqueIdentifier);
    }
    
    /**
     * 
     * Checks if the DCV unique value is an alphanumeric string of less than 20 characters
     * 
     * @return boolean
     */
    private function checkDomainControlValidationUniqueValue()
    {
        //Note: Strlen is permissible because we're testing for alphanumeric characters
        if (empty($this->domainControlValidationUniqueValue)
                || !is_string($this->domainControlValidationUniqueValue)
                || (is_string($this->domainControlValidationUniqueValue) 
                        && !\ctype_alnum($this->domainControlValidationUniqueValue))
                || (strlen($this->domainControlValidationUniqueValue) > 20)
            ) {
            $check = false;
        } else {
            $check = true;
        }
        
        return $check;   
    }
    
    private function checkAttributes()
    {
        if ($this->checkSoftware()
                && $this->checkUniqueIdentifier()
                && $this->checkDomainControlValidationUniqueValue()
            ) {
            $check = true;
        } else {
            $check = false;
        }
        return $check;
    }

    /**
     * Function used to validate the values of properties.
     * 
     * @todo Error management
     */
    public function validate()
    {
        while (!$this->checkAttributes()) {
            if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                \cli\line('An issue occured with the REQUEST. Please provide or correct the following information:');
                
                $this->interactiveFallback();
            } else {
                throw new \Exception('The request has not been correctly specified. Please correct the values.');
            }
        }
    }
    
    /**
     * Intefactive fallback for the software. To be used in the public intefactiveFallback()
     */
    private function interactiveFallbackSoftware()
    {
        if(!$this->checkSoftware()) {
            if(\TBSCertBot\Config::getConfig()->getHelp()) {
                \cli\line('Please input the software using the product:');
                self::softwareHelp();
            }

            $this->software = \cli\prompt('   - Software');
        }
    }
    
    /**
     * Intefactive fallback for the main domain. To be used in the public intefactiveFallback()
     */
    private function interactiveFallbackMainDomain()
    {
        if(empty($this->mainDomain)) {
            $this->mainDomain = \cli\prompt('   - Main Domain');
        } else {
            $this->mainDomain = \cli\prompt('   - Main Domain[' . $this->mainDomain . ']',
                    $this->mainDomain);
        }
    }
    
    /**
     * Intefactive fallback for the domain list. To be used in the public intefactiveFallback()
     */
    private function interactiveFallbackDomains()
    {
        \cli\line('If pertinent, please fill the domain list.');
        \cli\line('This is, for instance, not necessary for SSL Standard products.');
        
        if(empty($this->domains)) {
            //We default to an empty value as this value is not mandatory
            $this->domains = \cli\prompt('   - Domains', '');
        } else {
            \cli\line('Previously provided domains will be re-used if you do not enter new ones: ' . $this->domains);
            $this->domains = \cli\prompt('   - Domains', $this->domains);
        }
    }
    
    /**
     * Intefactive fallback for the DCV. To be used in the public intefactiveFallback()
     * 
     * @todo Proper validation
     */
    private function interactiveFallbackDomainControlValidation()
    {
        \cli\line('If pertinent, please fill the domain control validation method list:');
        
        if(\TBSCertBot\Config::getConfig()->getHelp()) {
                \cli\line('This value is only necessary for Comodo, TBS X509, and PositiveSSL products.');
                \cli\line('Please select HTTP_CSR_HASH, HTTPS_CSR_HASH, or an accepted email, per domain, separated by spaces.');
                \cli\line('You can find more information on DCV methods at https://www.tbs-certificats.com/FAQ/fr/challenge_dcv.html');
            }
        
        if(empty($this->domainControlValidation)) {
            $this->domainControlValidation = \cli\prompt('   - DCV', '');
        } else {
            \cli\line('If left empty, these previous values will be re-used: ' . $this->domainControlValidation);
            $this->domainControlValidation = \cli\prompt('   - DCV', $this->domainControlValidation);
        }
    }

    /**
     * Interactive fallback that can be called if the values of the entity are missing or incorrect
     * 
     * @todo Handle CSR and DCV interactive fallback.
     * @todo Handle Unique Value ?
     */
    public function interactiveFallback()
    {
        $this->interactiveFallbackSoftware();
        $this->interactiveFallbackMainDomain();
        $this->interactiveFallbackDomains();
        $this->interactiveFallbackDomainControlValidation();
         
    }

    public function save()
    {
        $info = array();

        if (isset($this->software) && !is_null($this->software)) {
            $info['software'] = $this->software;
        }
        if (isset($this->csr) && !is_null($this->csr)) {
            $info['csr'] = $this->csr;
        }
        if (isset($this->mainDomain) && !is_null($this->mainDomain)) {
            $info['mainDomain'] = $this->mainDomain;
        }
        if (isset($this->domainControlValidation) && !is_null($this->domainControlValidation)) {
            $info['domainControlValidation'] = $this->domainControlValidation;
        }
        if (isset($this->licenseNumber) && !is_null($this->licenseNumber)) {
            $info['licenseNumber'] = $this->licenseNumber;
        }
        if (isset($this->cloningReferences) && !is_null($this->cloningReferences)) {
            $info['cloningReferences'] = $this->cloningReferences;
        }
        if (isset($this->guidanceOption) && !is_null($this->guidanceOption)) {
            $info['guidanceOption'] = $this->guidanceOption;
        }

        if (isset($this->hashAlgorithm) && !is_null($this->hashAlgorithm)) {
            $info['hashAlgorithm'] = $this->hashAlgorithm;
        }
        
        if (isset($this->uniqueIdentifier) && !is_null($this->uniqueIdentifier)) {
            $info['uniqueIdentifier'] = $this->uniqueIdentifier;
        }
        
        if (isset($this->domainControlValidationUniqueValue) 
                && !is_null($this->domainControlValidationUniqueValue)) {
            $info['domainControlValidationUniqueValue'] = $this->domainControlValidationUniqueValue;
        }

        return $info;
    }

    public function getXmlRpc()
    {
        return array(
            'logiciel' => $this->software,
            'nbLicence' => $this->licenseNumber,
            'CSR' => $this->csr,
            'domainePrincipal' => $this->mainDomain,
            'domaines' => $this->domains,
            'cloneTBS' => $this->cloningReferences,
            'accompagnement' => \TBSCertBot\API\Helper::mapBooleanToFrench($this->guidanceOption),
            'adresseDVC' => $this->domainControlValidation,
            'hashage'   => $this->hashAlgorithm,
            'identifiantUnique' => $this->uniqueIdentifier,
            'valeurUniqueDCV' => $this->domainControlValidationUniqueValue
        );
    }

    public static function setCLIoptions(\cli\Arguments &$arguments)
    {
        $arguments->addFlag('guidance', ['description' => 'Guidance option requested for the order.',
                                          'default' => \TBSCertBot\Config::getConfig()->getRequest()->getGuidanceOption()]);
        
        $arguments->addOption('software', array('description' => 'Software code.'));
        $arguments->addOption('csr', array('description' => 'CSR file path.'));
        $arguments->addOption('key', array('description' => 'Private key file path.'));
        $arguments->addOption('maindomain', array('description' => 'Main domain for the order.'));
        $arguments->addOption('domain', array('description' => 'Domains list for the order.'));
        $arguments->addOption('license', array('description' => 'Number of licenses requested.'));
        $arguments->addOption('dcv', array('description' => 'DCV list matching the domains list order.'));
        $arguments->addOption('clone-ref', array('description' => 'Not documented yet.'));
        $arguments->addOption('hashAlgorithm', array('description' => 'Hash Algorithm for Thawte/Symanec products.'));
    }

    public static function mergeOptionsWithConfig(array $options)
    {
        $config = array();
        
        if (isset($options['software']) && !is_null($options['software'])) {
            $config['REQUEST']['software'] = $options['software'];
        }
        if (isset($options['csr']) && !is_null($options['csr'])) {
            $config['REQUEST']['csr'] = $options['csr'];
        }
        if (isset($options['key']) && !is_null($options['key'])) {
            $config['REQUEST']['key'] = $options['key'];
        }
        if (isset($options['maindomain']) && !is_null($options['maindomain'])) {
            $config['REQUEST']['mainDomain'] = $options['maindomain'];
        }
        
        if (isset($options['domain']) && !is_null($options['domain'])) {
            $config['REQUEST']['domains'] = $options['domain'];
        }
        if (isset($options['license']) && !is_null($options['license'])) {
            $config['REQUEST']['licenseNumber'] = $options['license'];
        }
        if (isset($options['dcv']) && !is_null($options['dcv'])) {
            $config['REQUEST']['domainControlValidation'] = $options['dcv'];
        }
        if (isset($options['clone-ref']) && !is_null($options['clone-ref'])) {
            $config['REQUEST']['cloningReferences'] = $options['clone-ref'];
        }
        if (isset($options['guidance']) && !is_null($options['guidance'])) {
            $config['REQUEST']['guidanceOption'] = $options['guidance'];
        }

        if (isset($options['hashAlgorithm']) && !is_null($options['hashAlgorithm'])) {
            $config['REQUEST']['hashAlgorithm'] = $options['hashAlgorithm'];
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }

    public static function softwareHelp()
    {
        $table = new \cli\Table();
        $table->setHeaders(array('code', 'name'));
        $rows = array();
        $softwares = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
                        file_get_contents(self::getDefaultSoftwaresJson()),
                        true
                );

        foreach ($softwares as $code => $name) {
            array_push($rows, array($code, $name));
        }

        $table->setRows($rows);
        $table->display();
    }

    public static function load(array $info)
    {
        if (isset($info['software']) && !is_null($info['software'])) {
            $software = $info['software'];
        }
        if (isset($info['csr']) && !is_null($info['csr'])) {
            $csr = $info['csr'];
        }
        if (isset($info['mainDomain']) && !is_null($info['mainDomain'])) {
            $mainDomain = $info['mainDomain'];
        }
        if (isset($info['domains']) && !is_null($info['domains'])) {
            $domains = $info['domains'];
        }
        if (isset($info['domainControlValidation']) && !is_null($info['domainControlValidation'])) {
            $domainControlValidation = $info['domainControlValidation'];
        }
        if (isset($info['licenseNumber']) && !is_null($info['licenseNumber'])) {
            $licenseNumber = $info['licenseNumber'];
        }
        if (isset($info['cloningReferences']) && !is_null($info['cloningReferences'])) {
            $cloningReferences = $info['cloningReferences'];
        }
        if (isset($info['guidanceOption']) && !is_null($info['guidanceOption'])) {
            $guidanceOption = $info['guidanceOption'];
        }

        if (isset($info['hashAlgorithm']) && !is_null($info['hashAlgorithm'])) {
            $hashAlgorithm = $info['hashAlgorithm'];
        }
        
        return new Request(
                $software,
                $csr,
                $mainDomain,
                $domains,
                $domainControlValidation,
                $licenseNumber,
                $cloningReferences,
                $guidanceOption,
                $hashAlgorithm
        );
    }
}
