<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\API;

/**
 * Helper "static" class containing utility methods
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
abstract class Helper
{
    private static $ERRORS_JSON;
    
    private static function getErrorsJson()
    {
        if (empty(self::$ERRORS_JSON)) {
            self::$ERRORS_JSON = 'data' . DIRECTORY_SEPARATOR . 'internal'
            . DIRECTORY_SEPARATOR . 'values' . DIRECTORY_SEPARATOR . 'Errors.json';
        }
        
        return self::$ERRORS_JSON;
    }
    
    /**
     * 
     * Maps booleans used by TBSCertBot to the french 
     * "oui" (yes) or "non" (no) expected by the API
     * 
     * @param bool $boolean
     * @return string contains "oui" (yes) or "non" (no)
     */
    public static function mapBooleanToFrench($boolean)
    {
        if ($boolean) {
            $french = 'oui';
        } else {
            $french = 'non';
        }
        return $french;
    }
    
    /**
     * 
     * Maps the title (Mr, Mrs, Miss) to its id.
     * 
     * @param string $title
     * @return string containing the id
     * @todo Add more titles
     */
    public static function mapTitleToId($title)
    {
        if ($title === 'Mr') {
            $id = '1';
        } elseif ($title === 'Mrs') {
            $id = '2';
        } elseif ($title === 'Miss'){
            $id = '3';
        } elseif ($title === 'Ms') {
            $id = '4';
        } elseif ($title === 'Dr') {
            $id = '5';
        } elseif ($title === 'Mx') {
            $id = '6';
        } else {
            //Fallback that should not be reachable
            $id = $title;
        }
        return $id;
    }
    
    /**
     * 
     * Maps true to 1 and false to 0
     * 
     * @param bool $boolean
     * @return int
     */
    public static function mapBooleanToInt($boolean)
    {
        if ($boolean) {
            $int = 1;
        } else {
            $int = 0;
        }
        return $int;
    }
    
    /**
     * 
     * Generates an error message based on the error code.
     * Error includes success.
     * 
     * @param int $errorCode
     * @return string
     */
    public static function getErrorMessage($errorCode)
    {
        $errors = \TBSCertBot\JsonDriver::getJsonDriver()->decode(
            file_get_contents(self::getErrorsJson()),
            true
        );
        
        if (array_key_exists($errorCode, $errors)) {
            $message = $errors[$errorCode];
        } elseif ($errorCode >= 0) {
            $message = 'Successful';
        } else {
            $message = 'Unknown error';
        }
        
        return $message;
    }
    
    /**
     * Tries to use getErrorMessage() and uses the fallback in case of failure
     * 
     * @param int $errorCode
     * @param string $fallbackMessage
     * @return string
     */
    public static function selectErrorMessage($errorCode, $fallbackMessage)
    {
        $messageAttempt = self::getErrorMessage($errorCode);
        
        if ($messageAttempt !== 'Unknown error') {
            return $messageAttempt;
        } else {
            return $fallbackMessage;
        }
    }
    
    /**
     * The API does not always return an numeric error code. Sometimes, it can return values like "-8a".
     * This code checks if the value is "negative" and returns a boolean.
     * 
     * @param int $errorCode
     * @return boolean
     */
    public static function isErrorCodeNegative($errorCode)
    {
        //We first need to exclude types we cannot work on, just in case
        if ($errorCode === null || $errorCode === false
                || is_array($errorCode) || is_object($errorCode)) {
            return false;
        }
        
        //If the variable is numeric we can use a simple comparison to 0
        if (is_numeric($errorCode)) {
            if ($errorCode < 0) return true;
            else return false;
        } else {
            //If not, we'll have to resort to string comparison
            if (strpos($errorCode, '-') === 0) return true;
            else return false;
        }
    }
}
