<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\CLI\Commands;

/**
 * Definition of the main common logic for commands
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
abstract class Command
{
    protected $arguments;
    protected $logger;
    
    public function __construct()
    {
        $this->arguments = new \cli\Arguments();
        $this->arguments->addOption('config' , array('description' => 'configuration file'));
        $this->arguments->addFlag('select-config', array('description' => 'select config from config dir'));
        $this->arguments->parse();
        
        //We explicitely check the class name in a elseif. We need to try to find a better solution if possible.

        if (isset($this->arguments['config']) && !empty($this->arguments['select-config'])) {
            throw new \Exception("Parameters config and select-config cannot be used together");
        } elseif (isset($this->arguments['config'])) {
            \TBSCertBot\Config::getConfig($this->arguments['config']);
        } elseif (!empty($this->arguments['select-config']) && get_class($this) === "TBSCertBot\CLI\Commands\Order") {
            if (!empty($this->arguments['batch'])) {
                throw new Exception("Parameter select-config requires interactive mode");
            }
            
            //We check that batch mode is not enabled;
            if (\TBSCertBot\Config::getConfig()->getBatch()) {
                throw new Exception("Parameter select-config requires interactive mode");
            }
            
            $configs = \TBSCertBot\Config::readConfigs();
            $nbConfigs = count($configs);
            
            
            $table = new \cli\Table();
            $table->setHeaders(['Number', 'File Name', 'Product', 'Organization', 'mainDomain', 'domains']);
            $rows = [];
            
            
            for ($i = 0; $i < $nbConfigs; $i++) {
                $rows[$i] = [
                    $i, 
                    $configs[$i]['fileName'],
                    $configs[$i]['PRODUCT']['code'],
                    $configs[$i]['ORGANIZATION']['name'], 
                    $configs[$i]['REQUEST']['mainDomain'],
                    (array_key_exists('domains', $configs[$i]['REQUEST']) ? $configs[$i]['REQUEST']['domains'] : '')
                ];
            }
            
            $table->setRows($rows);
            $table->display();
            
            do {
                $choice = \cli\prompt("Please select a file or type \"c\" to cancel");
                
            } while (!in_array($choice, array_merge(array_keys($configs), ['c'])));
            
            if ($choice === 'c') exit();
            
            \TBSCertBot\Config::getConfig(\TBSCertBot\Config::getConfig()->getConfigurationDirectoryPath() . DIRECTORY_SEPARATOR . $configs[$choice]['fileName']);
            
            //We unset select-config to be sure we will not get back into this condition at the next command execution
            $this->arguments['select-config'] = false;
            
        } else {
            \TBSCertBot\Config::getConfig();
        }
        unset($this->arguments);
        
        $this->arguments = new \cli\Arguments();
        self::setArguments($this->arguments);
        
        $this->parseArguments();
        self::mergeOptionsWithConfig($this->arguments->getArguments());
        
        $this->logger = new \TBSCertBot\Logger();
    }
    
    /**
     * Contains the main execution logic of the command
     */
    abstract public function proceed();
    
    protected function parseArguments()
    {
        $this->arguments->parse();
    }
    
    /**
     * 
     * Sets arguments from the command line
     * 
     * @param \cli\Arguments $arguments
     */
    public static function setArguments(\cli\Arguments &$arguments)
    {   

        $arguments->addFlag('debug', ['description' => 'Turn on debug mode.',
                                       'default' => \TBSCertBot\Config::getConfig()->getDebug()]);
        
        $arguments->addFlag('batch', ['description' => 'Turn non-interactive mode on.',
                                       'default' => \TBSCertBot\Config::getConfig()->getBatch()]);
        
        
        $arguments->addFlag('help', ['description' => 'Turn interfactive help mode on', 
                                      'default' => \TBSCertBot\Config::getConfig()->getHelp()]);
        

        $arguments->addOption('api-protocol', array('description' => 'Protocol used for API calls.'));
        $arguments->addOption('api-timeout', array('description' => 'Time-out used for API calls.'));
        $arguments->addOption('api-address', array('description' => 'Address used for API calls.'));
        $arguments->addOption('api-port', array('description' => 'Port used for API calls.'));
        $arguments->addOption('api-username', array('description' => 'API username.'));
        $arguments->addOption('api-password', array('description' => 'API password.'));
        
        $arguments->addOption('directory-certs', array('description' => 'Directory used to store certificates.'));
        $arguments->addOption('directory-keys', array('description' => 'Directory used to store cryptographic keys.'));
        $arguments->addOption('directory-reqs', array('description' => 'Directory used to store cryptographic requests.'));
        
        $arguments->addOption('hook-order', array('description' => 'Hook for orders'));
        $arguments->addOption('hook-renewal', array('description' => 'Hook for renewals'));
        $arguments->addOption('hook-download', array('description' => 'Hook for download'));
    }
    
    /**
     * 
     * Merges options from cli with th config file
     * 
     * @param array $options
     */
    private static function mergeOptionsWithConfig($options)
    {
        $config = array();
        
        if (isset($options['debug']) && !is_null($options['debug'])) {
            $config['debug'] = $options['debug'];
        }
        if (isset($options['batch']) && !is_null($options['batch'])) {
            $config['batch'] = $options['batch'];
        }
        if (isset($options['help']) && !is_null($options['help'])) {
            $config['help'] = $options['help'];
        }
        
        if (isset($options['api-protocol']) && !is_null($options['api-protocol'])) {
            $config['API']['protocol'] = $options['api-protocol'];
        }
        if (isset($options['api-timeout']) && !is_null($options['api-timeout'])) {
            $config['API']['timeout'] = $options['api-timeout'];
        }
        if (isset($options['api-address']) && !is_null($options['api-address'])) {
            $config['API']['address'] = $options['api-address'];
        }
        if (isset($options['api-port']) && !is_null($options['api-port'])) {
            $config['API']['port'] = $options['api-port'];
        }
        if (isset($options['api-username']) && !is_null($options['api-username'])) {
            $config['API']['username'] = $options['api-username'];
        }
        if (isset($options['api-password']) && !is_null($options['api-password'])) {
            $config['API']['api-password'] = $options['api-password'];
        }
        
        if (isset($options['directory-certs']) && !is_null($options['directory-certs'])) {
            $config['DIRECTORIES']['certificates'] = $options['directory-certs'];
        }
        if (isset($options['directory-keys']) && !is_null($options['directory-keys'])) {
            $config['DIRECTORIES']['keys'] = $options['directory-keys'];
        }
        if (isset($options['directory-reqs']) && !is_null($options['directory-reqs'])) {
            $config['DIRECTORIES']['requests'] = $options['directory-reqs'];
        }
        
        if (isset($options['hook-order']) && !is_null($options['hook-order'])) {
            $config['HOOKS']['order'] = $options['hook-order'];
        }
        if (isset($options['hook-renewal']) && !is_null($options['hook-renewal'])) {
            $config['HOOKS']['renewal'] = $options['hook-renewal'];
        }
        if (isset($options['hook-download']) && !is_null($options['hook-download'])) {
            $config['HOOKS']['download'] = $options['hook-download'];
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    /**
     *  Runs user-defined hooks on some commands
     */
    protected abstract function runHook($tbsReference = null);
}
