<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\CLI\Commands;

/**
 * Renews one or more certificates
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Copy extends Order
{
    //Note: If you edit these values, please edit the user messages, which are hardcoded
    const RENEWAL_MINIMUM_DATE_90d = 1728000;
    const RENEWAL_MINIMUM_DATE = 7780000;
    private $hook;
    private $reuseKeys;

    /**
     * 
     * @param type $tbsReference optional TBS Reference to only one certificate
     */
    public function __construct($tbsReference)
    {
        parent::__construct();
        
        self::setArguments($this->arguments);
        $this->parseArguments();
        self::mergeOptionsWithConfig($this->arguments->getArguments());
        
        $this->profiles = \TBSCertBot\Controllers\Profile::getProfiles($tbsReference);
        
        $this->hook = \TBSCertBot\Config::getConfig()->getRenewalHook();
        $this->reuseKeys = \TBSCertBot\Config::getConfig()->getReuseKeys();
    }

    /**
     * 
     * Contains the main execution logic of the command
     * 
     * @throws \Exception
     * @todo Handle organization identifier in the DN for CSR?
     * @todo Handle reference in the request
     * @todo Improve error handling after the api request
     */
    public function proceed()
    {  
        foreach ($this->profiles as $profile) {

            //We create the keys to avoid warnings. Content and type checks are run using validation functions.
            $indicies = ['CERTIFICATE', 'PRODUCT', 'REQUEST', 'ORGANIZATION',
                'CORPORATE_CONTACT', 'PAYMENT', 'TECHNICAL_CONTACT', 'LEGAL_CONTACT', 
                'BILLING_ORGANIZATION', 'MISCELLANEOUS'];
            
            foreach($indicies as $index) {
                if (!array_key_exists($index, $profile) || !is_array($profile[$index])) {
                    $profile[$index] = [];
                }
            }
            
            \cli\line();

            //TODO Merge with options/config?
            
            if (!$this->reuseKeys) {
                unset($profile['REQUEST']['csr']);
            }
            
            $profile = $this->overwriteProfileWithOptions($profile);
            
            $this->product = \TBSCertBot\API\Entities\Product::load($profile['PRODUCT']);
            $this->request = \TBSCertBot\API\Entities\Request::load($profile['REQUEST']);
            $this->organization = \TBSCertBot\API\Entities\Organization::load($profile['ORGANIZATION']);
            $this->corporateContact = \TBSCertBot\API\Entities\CorporateContact::load($profile['CORPORATE_CONTACT']);
            $this->payment = \TBSCertBot\API\Entities\Payment::load($profile['PAYMENT']);
            $this->technicalContact = \TBSCertBot\API\Entities\TechnicalContact::load($profile['TECHNICAL_CONTACT']);
            $this->legalContact = \TBSCertBot\API\Entities\LegalContact::load($profile['LEGAL_CONTACT']);
            $this->billingOrganization = \TBSCertBot\API\Entities\BillingOrganization::load($profile['BILLING_ORGANIZATION']);
            $this->miscellaneous = \TBSCertBot\API\Entities\Miscellaneous::load($profile['MISCELLANEOUS']);

            //Generating the DCV unique value and setting it into the request
            //This generation needs to happen here so that we can send it to the request
            $dcvUniqueValue = \TBSCertBot\Crypto\Helper::getAlphanumericToken(20);
            $this->request->setDomainControlValidationUniqueValue($dcvUniqueValue);

            \cli\line('Validating all parameters...');
            $this->logger->appendToBuffer('Validating all parameters...', 'info');
            
            try {
                $this->validate();
            } catch (\Exception $e) {
                $this->logger->appendToBuffer($e->getMessage(), 'error');
                throw $e;
            }
            
            \cli\line('All parameters are OK!');
            $this->logger->appendToBuffer('All parameters are OK!', 'info');
            
            \cli\line();

            $cn = \TBSCertBot\Controllers\Validation::getCN($this->request);

            if ($cn === null) {
                $this->logger->appendToBuffer('No domain name provided.', 'error');
                throw new \Exception('No domain name provided.');
            }

            if ($this->reuseKeys) {
                $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                    \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                    . DIRECTORY_SEPARATOR
                    . $cn . '-' . date('Y-m-d') . '.csr'
                );

                //Importing previous CSR
                $this->csr = new \TBSCertBot\Crypto\ExternalCSR(array(), 
                        file_get_contents($profile['CSR']['fileName']), 
                        $csrFileName, $dcvUniqueValue);

                //Display the HTTP DCV File if needed
                if ($this->request->getDomainControlValidation() === 'HTTP_CSR_HASH'
                            || $this->request->getDomainControlValidation() === 'HTTPS_CSR_HASH') {
                        \cli\line("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "");
                        $this->logger->appendToBuffer("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "", 'info');
                }  
            } elseif ($this->request->getCsr()) {
                $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                    \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                    . DIRECTORY_SEPARATOR
                    . $cn . '-' . date('Y-m-d') . '.csr'
                );

                try {
                    $this->csr = new \TBSCertBot\Crypto\ExternalCSR(array(), 
                            $this->request->getCsr(), 
                            $csrFileName, $dcvUniqueValue);

                    if ($this->csr->getCN() && $this->csr->getCN() != $cn) {
                        $this->logger->appendToBuffer('The CN provided in the CSR does not match the CN of the request', 'error');
                        throw new \Exception('The CN provided in the CSR does not match the CN of the request');
                    }
                } catch (\Exception $e) {
                    throw $e;
                }
            } else {
                $dn = array(
                    "CN" => $cn,
                    "O" => $this->organization->getName(),
                    "L" => $this->organization->getCity(),
                    "ST" => $this->organization->getState(),
                    "C" => $this->organization->getCountry()
                );

                // Key generation.
                $keyFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                                \TBSCertBot\Config::getConfig()->getKeysDirectoryPath()
                                . DIRECTORY_SEPARATOR
                                . $cn . '-' . date('Y-m-d') . '.key'
                );
                $this->key = new \TBSCertBot\Crypto\RSA($keyFileName);
                try {
                    \cli\line('Generating new cryptographic key...');
                    $this->logger->appendToBuffer('Generating new cryptographic key...', 'info');
                    \cli\line('   - Type = ' . \TBSCertBot\Crypto\RSA::TYPE);
                    $this->logger->appendToBuffer('   - Type = ' . \TBSCertBot\Crypto\RSA::TYPE, 'info');
                    \cli\line('   - Length = ' . \TBSCertBot\Crypto\RSA::DEFAULT_KEY_SIZE);
                    $this->logger->appendToBuffer('   - Length = ' . \TBSCertBot\Crypto\RSA::DEFAULT_KEY_SIZE, 'info');
                    $this->key->generate($dcvUniqueValue);
                    \cli\line('New cryptographic key generated!');
                    $this->logger->appendToBuffer('New cryptographic key generated!', 'info');
                } catch (\Exception $e) {
                    $this->logger->appendToBuffer($e->getMessage(), 'error');
                    throw $e;
                }
                \cli\line();

                // CSR generation.
                $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                                \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                                . DIRECTORY_SEPARATOR
                                . $cn . '-' . date('Y-m-d') . '.csr'
                );
                $this->csr = new \TBSCertBot\Crypto\CSR($dn, $csrFileName, $this->key);
                try {
                    \cli\line('Generating new CSR...');
                    $this->logger->appendToBuffer('Generating new CSR...', 'info');
                    \cli\line('   - CN = ' . $dn['CN']);
                    $this->logger->appendToBuffer('   - CN = ' . $dn['CN'], 'info');
                    \cli\line('   - O = ' . $dn['O']);
                    $this->logger->appendToBuffer('   - O = ' . $dn['O'], 'info');
                    \cli\line('   - L = ' . $dn['L']);
                    $this->logger->appendToBuffer('   - L = ' . $dn['L'], 'info');
                    \cli\line('   - ST = ' . $dn['ST']);
                    $this->logger->appendToBuffer('   - ST = ' . $dn['ST'], 'info');
                    \cli\line('   - C = ' . $dn['C']);
                    $this->logger->appendToBuffer('   - C = ' . $dn['C'], 'info');

                    $this->csr->generate($dcvUniqueValue);
                    \cli\line('New CSR generated!');
                    $this->logger->appendToBuffer('New CSR generated!', 'info');

                    //Display the HTTP DCV File if needed
                    if ($this->request->getDomainControlValidation() === 'HTTP_CSR_HASH'
                            || $this->request->getDomainControlValidation() === 'HTTPS_CSR_HASH') {
                        \cli\line("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "");
                        $this->logger->appendToBuffer("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "", 'info');
                    }
                } catch (\Exception $e) {
                    $this->logger->appendToBuffer($e->getMessage(), 'error');
                    throw $e;
                }
                \cli\line();   
            }

            // Affect the newly generated CSR to the Request.
            $this->request->setCSR($this->csr->getASCII());

            $validation = new \TBSCertBot\Controllers\Validation($this->request, $this->csr);
            $validation->validate($this->product->needsDcv());

            $validation->checkDCV();

            // TODO: Handle reference (last argument).
            $order = new \TBSCertBot\API\Methods\Order\Request(
                $this->product,
                $this->request,
                $this->organization,
                $this->corporateContact,
                $this->payment,
                $this->technicalContact,
                $this->legalContact,
                $this->billingOrganization,
                $this->miscellaneous,
                null
            );

            $orderInformation = $order->save();
            $orderInformation['CSR'] = $this->csr->save();
            if ($this->key) $orderInformation['PRIVATE_KEY'] = $this->key->save();

            $this->runDcvHook(null, $orderInformation);

            if ($this->product->needsDcv() && $this->product->isDcvMandatoryToOrder()
                    && !\TBSCertBot\Config::getConfig()->getBatch()) {
                \cli\line("The product you are ordering requires the DCV challenge to be successful beforehand.");
                $this->logger->appendToBuffer('The product you are ordering requires the DCV challenge to be successful beforehand.', 'info');

                //Use of two variable for PHP 5.3 compatibility
                $dcvRelativeFileNameArr = array_slice(explode('/', $this->csr->getDcvHttpFileName()), -1, 1, false);
                $dcvRelativeFileName = $dcvRelativeFileNameArr[0];

                \cli\line("Please make sure the DCV file is accessible at this address for each requested domain:");
                $this->logger->appendToBuffer('Please make sure the DCV file is accessible at this address for each requested domain:', 'info');
                \cli\line("http://domain/.well-known/pki-validation/" . $dcvRelativeFileName);
                $this->logger->appendToBuffer("http://domain/.well-known/pki-validation/" . $dcvRelativeFileName, 'info');

                \cli\line("The DCV file is available at: " . $this->csr->getDcvHttpFileName());
                $this->logger->appendToBuffer("The DCV file is available at: " . $this->csr->getDcvHttpFileName(), 'info');

                $dcvContinueChoice = "n";
                while ($dcvContinueChoice !== "y") {
                    $dcvContinueChoice = \cli\choose("Would you like to resume the order?", "yn", "n");
                }
            }

            \cli\line('Placing order...');
            $this->logger->appendToBuffer('Placing order...', 'info');
            $response = $order->send();

            //Let's check if we could get a response

            //Pre PHP 5 empty limitation
            $tmpReference = $response->getTBSReference();

            //PHP 5.3 compat: empty needs a reference
            $xmlRpcCode = $response->getXmlRpcCode();
            $xmlRpcMessage = $response->getXmlRpcMessage();

            if (empty($tmpReference) && is_int($xmlRpcCode) && !empty($xmlRpcCode)) {
                \cli\line("Could not get a reponse.");
                $this->logger->appendToBuffer('Could not get a reponse.', 'warn');

                if (!empty($xmlRpcCode) && !empty($xmlRpcMessage)) {
                    \cli\line('Network Error No ' . $xmlRpcCode
                        . ': ' . $xmlRpcMessage);
                    
                    $this->logger->appendToBuffer('Network Error No ' . $xmlRpcCode
                        . ': ' . $xmlRpcMessage, 'warn');
                }

                \cli\line("Waiting for the order to be processed before trying to recover it.");
                $this->logger->appendToBuffer('Waiting for the order to be processed before trying to recover it.', 'warn');

                //We have to wait because an order can take a long time to be processed.
                sleep(30);

                try {
                    $recoverable = new \TBSCertBot\Controllers\Recoverable($this->request->getUniqueIdentifier());

                    $recoverable->recover();

                    $tbsReference = $recoverable->getTbsReference();
                    $caReference = '';

                    /*
                     * If we could not get a TBS Reference,
                     * it means there was a network error we could not identify.
                     * 
                     * We could throw an exception but we carry on in order to save the order.
                     */
                    if (empty($tbsReference)) {
                        \cli\line("Could not recover order \"" 
                                . $this->request->getUniqueIdentifier()
                                . "\".");
                        
                        $this->logger->appendToBuffer("Could not recover order \"" 
                                . $this->request->getUniqueIdentifier()
                                . "\".", 'error');
                        
                        \cli\line("Please try using the recover command in a few minutes.");
                        
                        $this->logger->setFileName($this->request->getUniqueIdentifier() . '.log');
                    } else {
                        $this->logger->setFileName($tbsReference . '.log');
                    }

                } catch (\Exception $e) {
                    \cli\line("Could not get a response after ordering");
                    $this->logger->appendToBuffer('Could not get a response after ordering', 'error');
                    \cli\line("Error: " . $e->getMessage());
                    $this->logger->appendToBuffer("Error: " . $e->getMessage(), 'error');
                    
                    \cli\line("Could not recover order \"" 
                        . $this->request->getUniqueIdentifier()
                        . "\".");
                    
                    $this->logger->appendToBuffer("Could not recover order \"" 
                        . $this->request->getUniqueIdentifier()
                        . "\".", 'error');
                    
                    \cli\line("Please try using the recover command in a few minutes.");
                    \cli\line();

                    // Update and save profile
                    $orderInformation = array_replace_recursive($orderInformation, $order->save());
                    $certificateInformation = new \TBSCertBot\API\Entities\Certificate($caReference, $tbsReference);
                    $orderInformation['CERTIFICATE'] = $certificateInformation->save();

                    \TBSCertBot\Controllers\Profile::setProfile($orderInformation);

                    exit();
                }
            } elseif (!\TBSCertBot\API\Helper::isErrorCodeNegative($response->getCode())) {
                $tbsReference = $response->getTBSReference();
                $this->logger->setFileName($tbsReference . '.log');
            } else {
                \cli\line('Order failed');
                $this->logger->appendToBuffer('Order failed', 'error');
                throw new \Exception(\TBSCertBot\API\Helper::selectErrorMessage($response->getMessage(), $response->getCode()),
                        $response->getCode());
            }

            \cli\line('Order taken into account under the reference ' . $tbsReference . '.');
            $this->logger->appendToBuffer('Order taken into account under the reference ' . $tbsReference . '.', 'info');
            \cli\line();

            // Update and save profile
            $orderInformation = array_replace_recursive($orderInformation, $order->save());
            $certificateInformation = new \TBSCertBot\API\Entities\Certificate($caReference, $tbsReference);
            $orderInformation['CERTIFICATE'] = $certificateInformation->save();

            \TBSCertBot\Controllers\Profile::setProfile($orderInformation);

            //Running order hook before the download (that might trigger the download hook) 
            $this->runHook($tbsReference);

            $download = new Download($tbsReference);
            $download->proceed();
        }
    }
    
    public static function setArguments(\cli\Arguments &$arguments) 
    {
        $arguments->addFlag('reuse-keys', array('description' => 'Reuses keys from previous orders when renewing or reissueing'));
        $arguments->addFlag('no-reuse-keys', array('description' => 'Prevents reusing keys from previous orders when renewing or reissueing'));
    }
    
    private static function mergeOptionsWithConfig($options)
    {
        $config = array();
        
        if (!empty($options['reuse-keys'])) {
            $config['reuseKeys'] = true;
        } elseif ($options['no-reuse-keys']) {
            $config['reuseKeys'] = false;
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    private function validate()
    {   
        $this->product->validate();
        $this->request->validate();
        $this->organization->validate();
        $this->payment->validate();
        
        $corporateContact = array_filter((array)$this->corporateContact);
        $legalContact = array_filter((array)$this->legalContact);
        $technicalContact = array_filter((array)$this->technicalContact);
        
        $billingOrganization = array_filter(\TBSCertBot\Helper::array_flatten((array)$this->billingOrganization));
        
        $miscellaneous = array_filter((array)$this->miscellaneous);
        
        if ($corporateContact) {
            $this->corporateContact->validate();
        } elseif ($legalContact) {
            $this->legalContact->validate();
        } else { //Falllback to the corporate contact
            $this->corporateContact->validate();
        }
        
        if ($billingOrganization) {
            $this->billingOrganization->validate();
        }
        if ($technicalContact) {
            $this->technicalContact->validate();
        }
        
        if ($miscellaneous) {
            $this->miscellaneous->validate();
        }
    }
    
    private function overwriteProfileWithOptions($profile)
    {
        $options = $this->arguments->getArguments();
        
        //product
        if (!empty($options['product'])) $profile['PRODUCT']['code'] = $options['product'];
        if (!empty($options['validity'])) $profile['PRODUCT']['validity'] = $options['validity'];
        
        //request
        if (!empty($options['software'])) $profile['REQUEST']['software'] = $options['software'];
        if (!empty($options['maindomain'])) $profile['REQUEST']['mainDomain'] = $options['maindomain'];
        if (!empty($options['domain'])) $profile['REQUEST']['domains'] = $options['domains'];
        if (!empty($options['dcv'])) $profile['REQUEST']['domainControlValidation'] = $options['dcv'];
        if (!empty($options['hashAlgorithm'])) $profile['REQUEST']['hashAlgorithm'] = $options['hashAlgorithm'];
        if (!empty($options['csr'])) $profile['REQUEST']['csr'] = $options['csr'];
        if (!empty($options['license'])) $profile['REQUEST']['licenseNumber'] = $options['license'];
        if (!empty($options['clone-ref'])) $profile['REQUEST']['cloningReferences'] = $options['clone-ref'];
        if (!empty($options['guidance'])) $profile['REQUEST']['guidanceOption'] = $options['guidance'];
        
        //organization
        if (!empty($options['org-name'])) $profile['ORGANIZATION']['name'] = $options['org-name'];
        
        if (!empty($options['org-address-1'])) $profile['ORGANIZATION']['address'][0] = $options['org-address-1'];
        if (!empty($options['org-address-2'])) $profile['ORGANIZATION']['address'][1] = $options['org-address-2'];
        if (!empty($options['org-address-3'])) $profile['ORGANIZATION']['address'][2] = $options['org-address-3'];
        
        if (!empty($options['org-city'])) $profile['ORGANIZATION']['city'] = $options['org-city'];
        if (!empty($options['org-postalCode'])) $profile['ORGANIZATION']['postalCode'] = $options['org-postalCode'];
        if (!empty($options['org-state'])) $profile['ORGANIZATION']['state'] = $options['org-state'];
        if (!empty($options['org-country'])) $profile['ORGANIZATION']['country'] = $options['org-country'];
        if (!empty($options['org-phone'])) $profile['ORGANIZATION']['phoneNumber'] = $options['org-phone'];
        if (!empty($options['org-type'])) $profile['ORGANIZATION']['type'] = $options['org-type'];
        if (!empty($options['org-number'])) $profile['ORGANIZATION']['number'] = $options['org-number'];
        if (!empty($options['org-nafcode'])) $profile['ORGANIZATION']['nafCode'] = $options['org-nafcode'];
        if (!empty($options['org-website'])) $profile['ORGANIZATION']['website'] = $options['org-website'];
        if (!empty($options['org-vat'])) $profile['ORGANIZATION']['vatNumber'] = $options['org-vat'];
        if (!empty($options['org-accounting-email'])) $profile['ORGANIZATION']['accountingEmailAddress'] = $options['org-accounting-email'];

        
        //corporate contact
        if (!empty($options['cc-title'])) $profile['CORPORATE_CONTACT']['title'] = $options['cc-title'];
        if (!empty($options['cc-firstname'])) $profile['CORPORATE_CONTACT']['firstName'] = $options['cc-firstname'];
        if (!empty($options['cc-lastname'])) $profile['CORPORATE_CONTACT']['lastName'] = $options['cc-lastname'];
        if (!empty($options['cc-function'])) $profile['CORPORATE_CONTACT']['function'] = $options['cc-function'];
        if (!empty($options['cc-phone'])) $profile['CORPORATE_CONTACT']['phoneNumber'] = $options['cc-phone'];
        if (!empty($options['cc-email'])) $profile['CORPORATE_CONTACT']['emailAddress'] = $options['cc-email'];
        if (!empty($options['cc-language'])) $profile['CORPORATE_CONTACT']['language'] = $options['cc-language'];
        
        
        //technical contact
        if (!empty($options['tc-title'])) $profile['TECHNICAL_CONTACT']['title'] = $options['tc-title'];
        if (!empty($options['tc-firstname'])) $profile['TECHNICAL_CONTACT']['firstName'] = $options['tc-firstname'];
        if (!empty($options['tc-lastname'])) $profile['TECHNICAL_CONTACT']['lastName'] = $options['tc-lastname'];
        if (!empty($options['tc-function'])) $profile['TECHNICAL_CONTACT']['function'] = $options['tc-function'];
        if (!empty($options['tc-phone'])) $profile['TECHNICAL_CONTACT']['phoneNumber'] = $options['tc-phone'];
        if (!empty($options['tc-email'])) $profile['TECHNICAL_CONTACT']['emailAddress'] = $options['tc-email'];
        if (!empty($options['tc-language'])) $profile['TECHNICAL_CONTACT']['language'] = $options['tc-language'];
        
        //legal contact
        if (!empty($options['legal-title'])) $profile['LEGAL_CONTACT']['title'] = $options['legal-title'];
        if (!empty($options['legal-firstname'])) $profile['LEGAL_CONTACT']['firstName'] = $options['legal-firstname'];
        if (!empty($options['legal-lastname'])) $profile['LEGAL_CONTACT']['lastName'] = $options['legal-lastname'];
        if (!empty($options['legal-function'])) $profile['LEGAL_CONTACT']['function'] = $options['legal-function'];
        if (!empty($options['legal-phone'])) $profile['LEGAL_CONTACT']['phoneNumber'] = $options['legal-phone'];
        if (!empty($options['legal-email'])) $profile['LEGAL_CONTACT']['emailAddress'] = $options['legal-email'];
        if (!empty($options['legal-language'])) $profile['LEGAL_CONTACT']['language'] = $options['legal-language'];
        
        //payment
        if (!empty($options['payment'])) $profile['PAYMENT']['mode'] = $options['payment'];
        if (!empty($options['quotation'])) $profile['PAYMENT']['quotation'] = $options['quotation'];
        
        //billing organization
        if (!empty($options['billing-name'])) $profile['BILLING_ORGANIZATION']['name'] = $options['billing-name'];
        
        if (!empty($options['billing-address-1'])) $profile['BILLING_ORGANIZATION']['address'][0] = $options['billing-address-1'];
        if (!empty($options['billing-address-2'])) $profile['BILLING_ORGANIZATION']['address'][1] = $options['billing-address-2'];
        if (!empty($options['billing-address-3'])) $profile['BILLING_ORGANIZATION']['address'][2] = $options['billing-address-3'];
        
        if (!empty($options['billing-city'])) $profile['BILLING_ORGANIZATION']['city'] = $options['billing-city'];
        if (!empty($options['billing-postalCode'])) $profile['BILLING_ORGANIZATION']['postalCode'] = $options['billing-postalCode'];
        if (!empty($options['billing-state'])) $profile['BILLING_ORGANIZATION']['state'] = $options['billing-state'];
        if (!empty($options['billing-country'])) $profile['BILLING_ORGANIZATION']['country'] = $options['billing-country'];
        if (!empty($options['billing-phone'])) $profile['BILLING_ORGANIZATION']['phoneNumber'] = $options['billing-phone'];
        if (!empty($options['billing-type'])) $profile['BILLING_ORGANIZATION']['type'] = $options['billing-type'];
        if (!empty($options['billing-number'])) $profile['BILLING_ORGANIZATION']['number'] = $options['billing-number'];
        if (!empty($options['billing-nafcode'])) $profile['BILLING_ORGANIZATION']['nafCode'] = $options['billing-nafcode'];
        if (!empty($options['billing-website'])) $profile['BILLING_ORGANIZATION']['website'] = $options['billing-website'];
        if (!empty($options['billing-vat'])) $profile['BILLING_ORGANIZATION']['vatNumber'] = $options['billing-vat'];
        if (!empty($options['billing-accounting-email'])) $profile['BILLING_ORGANIZATION']['accountingEmailAddress'] = $options['billing-accounting-email'];

        
        //misc
        if (!empty($options['promo'])) $profile['MISCELLANEOUS']['promotionalCode'] = $options['promo'];
        if (!empty($options['note'])) $profile['MISCELLANEOUS']['customerNote'] = $options['note'];
        if (!empty($options['customer-ref'])) $profile['MISCELLANEOUS']['customerReference'] = $options['customer-ref'];
        //No reissue reference as this is not a reissue
        
        return $profile;
    }
}
