<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\CLI\Commands;

/**
 * Downloads updates about certificates
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 * @todo Handle reissues
 */
class Download extends Command
{
    private $profiles = array();
    private $hook;

    public function __construct($tbsReference = null)
    {        
        parent::__construct();
        
        //$this->setArguments();
        //$this->parseArguments();
        //$this->mergeOptionsWithConfig($this->arguments->getArguments());
            
        $this->profiles = \TBSCertBot\Controllers\Profile::getProfiles($tbsReference);
        
        $this->hook = \TBSCertBot\Config::getConfig()->getDownloadHook();
    }

    public function proceed()
    {
        foreach ($this->profiles as $profile) {
            
            //Assigning to empty string if unset to quiet implicit creation warning
            if (!isset($profile['CERTIFICATE']['caReference'])) {
                $profile['CERTIFICATE']['caReference'] = '';
            } 

            $status = new \TBSCertBot\API\Methods\Status\Request(
                    new \TBSCertBot\API\Entities\Certificate(
                    $profile['CERTIFICATE']['caReference'], $profile['CERTIFICATE']['tbsReference']
            ));

            // We don't want to download certificates that are revoked or have expired
            if (array_key_exists('LAST_KNOWN_STATUS', $profile) 
                    && is_array($profile['LAST_KNOWN_STATUS']) &&
                    array_key_exists('code', $profile['LAST_KNOWN_STATUS'])
                    && in_array($profile['LAST_KNOWN_STATUS']['code'], [-1, 2, 3])) {
               continue;
            }
            
            \cli\line();
            
            if (empty($profile['CERTIFICATE']['certificatePEM'])) {
                \cli\line('Checking if certificate reference ' . $profile['CERTIFICATE']['tbsReference'] 
                        . ' (CN = ' . $profile['CSR']['commonName'] . ') is available...');
                
                $this->logger->appendToBuffer('Checking if certificate reference ' . $profile['CERTIFICATE']['tbsReference'] 
                        . ' (CN = ' . $profile['CSR']['commonName'] . ') is available...', 'info');
            } else {
                \cli\line('Updating status for certificate reference ' . $profile['CERTIFICATE']['tbsReference'] 
                        . ' (CN = ' . $profile['CSR']['commonName'] . ')');
                
                $this->logger->appendToBuffer('Updating status for certificate reference ' . $profile['CERTIFICATE']['tbsReference'] 
                        . ' (CN = ' . $profile['CSR']['commonName'] . ')', 'info');
            }
            
            $statusResponse = $status->send();

            if ($statusResponse->getCode() === 0) {

                if ($statusResponse->getCAReference() !== null) {
                    $profile['CERTIFICATE']['caReference'] = $statusResponse->getCAReference();
                }
                
                $profile['LAST_KNOWN_STATUS']['code'] = $statusResponse->getStatusCode();
                $profile['LAST_KNOWN_STATUS']['message'] = $statusResponse->getStatusMessage();
                $profile['LAST_KNOWN_STATUS']['messageExtra'] = $statusResponse->getStatusMessageExtra();
                
                //If the certificate has been renewed, we need to check if it was by TBSCertBot
                if ($statusResponse->isRenewed()
                        && $profile['CERTIFICATE']['renewed'] != $statusResponse->isRenewed()) {
                    //First, we try to recover orders
                    $recover = new Recover();
                    $recover->proceed();
                    
                    $allProfiles = \TBSCertBot\Controllers\Profile::getProfiles();
                    
                    //We try to find a corresponding TBS Reference in our profiles
                    $match = false;
                    foreach ($allProfiles as $tmpProfile) {
                        if ($tmpProfile['CERTIFICATE']['tbsReference'] == $statusResponse->getRenewedByTbsReference()) {
                            $match = true;
                        }
                    }
                    
                    //If we didn't find any, we send a warning to the user on stderr
                    if ($match == false) {
                        \cli\err('Certificate ' . $profile['CERTIFICATE']['tbsReference']
                                . ' was renewed without TBSCertBot by certificate '
                                . $statusResponse->getRenewedByTbsReference());
                        
                        $this->logger->appendToBuffer('Certificate ' . $profile['CERTIFICATE']['tbsReference']
                                . ' was renewed without TBSCertBot by certificate '
                                . $statusResponse->getRenewedByTbsReference(), 'error');
                    }
                }
                
                //Then, no matter what, we make sure that the correct state is logged
                $profile['CERTIFICATE']['renewed'] = $statusResponse->isRenewed();

                // We don't want to download certificates that have already been downloaded
                if (empty($profile['CERTIFICATE']['certificatePEM'])) {
                    $newCert = false;
                    if ($statusResponse->getStatusCode() == 1 
                            && $statusResponse->getCertificatePEM() !== null) {
                        $newCert = true;
                        \cli\line('   -> Certificate has been successfully issued!');
                        $this->logger->appendToBuffer('   -> Certificate has been successfully issued!', 'info');
                        
                        $certFile = \TBSCertBot\Helper::getOrIncrementFileName(\TBSCertBot\Config::getConfig()->getCertificatesDirectoryPath()
                                        . DIRECTORY_SEPARATOR . 'cert-' . $statusResponse->getTBSReference() . '.cer');
                        file_put_contents($certFile, $statusResponse->getCertificatePEM());
                        \cli\line('      - Certificate downloaded and saved in file ' . $certFile);
                        $this->logger->appendToBuffer('      - Certificate downloaded and saved in file ' . $certFile, 'info');

                        $chainFile = \TBSCertBot\Helper::getOrIncrementFileName(\TBSCertBot\Config::getConfig()->getCertificatesDirectoryPath()
                                        . DIRECTORY_SEPARATOR . 'chain-' . $statusResponse->getTBSReference() . '.txt');
                        file_put_contents($chainFile, $statusResponse->getChainPEM());
                        \cli\line('      - Certificate chain downloaded and saved in file ' . $chainFile);
                        $this->logger->appendToBuffer('      - Certificate chain downloaded and saved in file ' . $chainFile, 'info');

                        //We move the private key if we generated it
                        if ($profile['PRIVATE_KEY']) {
                            $keyFile = \TBSCertBot\Helper::getOrIncrementFileName(\TBSCertBot\Config::getConfig()->getKeysDirectoryPath()
                                . DIRECTORY_SEPARATOR . 'key-' . $statusResponse->getTBSReference() . '.pkey');

                            rename($profile['PRIVATE_KEY']['fileName'], $keyFile);

                            \cli\line('      - Moving and renaming private key file to ' . $keyFile);
                            $this->logger->appendToBuffer('      - Moving and renaming private key file to ' . $keyFile, 'info');
                        }
                        \cli\line();

                        $profile['CERTIFICATE']['certificatePEM'] = $statusResponse->getCertificatePEM();
                        $profile['CERTIFICATE']['certificatePEMFile'] = $certFile;

                        $profile['CERTIFICATE']['chainPEM'] = $statusResponse->getChainPEM();
                        $profile['CERTIFICATE']['chainPEMFile'] = $chainFile;

                        $profile['CERTIFICATE']['beginDate'] = $statusResponse->getBeginDate();
                        $profile['CERTIFICATE']['endDate'] = $statusResponse->getEndDate();
                    } else {
                        \cli\line('   -> Certificate is NOT AVAILABLE now, its current state is "' . $statusResponse->getStatusMessage() . '".');
                        $this->logger->appendToBuffer('   -> Certificate is NOT AVAILABLE now, its current state is "' . $statusResponse->getStatusMessage() . '".', 'info');

                        //PHP 5.3 compat : empty needs a reference
                        $statusMessageExtra = $statusResponse->getStatusMessageExtra();
                        
                        if (!empty($statusMessageExtra)) {
                            \cli\line('   -> Extra information says "' . $statusResponse->getStatusMessageExtra() . '".');
                            $this->logger->appendToBuffer('   -> Extra information says "' . $statusResponse->getStatusMessageExtra() . '".', 'info');
                        }
                    }
                } else {
                    $newCert = false;
                }

                \TBSCertBot\Controllers\Profile::setProfile($profile);

            } elseif (\TBSCertBot\API\Helper::isErrorCodeNegative($statusResponse->getCode())) {
                //API error
                \cli\err('Could not retrieve information for certificate ' . $profile['CERTIFICATE']['tbsReference'] . '.');
                $this->logger->appendToBuffer('Could not retrieve information for certificate ' . $profile['CERTIFICATE']['tbsReference'] . '.', 'error');
                
                \cli\err('Error ' . $statusResponse->getCode() . ': ' 
                        . \TBSCertBot\API\Helper::selectErrorMessage($statusResponse->getCode() , $statusResponse->getMessage()));
                
                $this->logger->appendToBuffer('Error ' . $statusResponse->getCode() . ': ' 
                        . \TBSCertBot\API\Helper::selectErrorMessage($statusResponse->getCode() , $statusResponse->getMessage()), 'error');
            } elseif ($statusResponse->getXmlRpcCode != 0) {
                //XML RPC error
                \cli\err('Could not retrieve information for certificate ' . $profile['CERTIFICATE']['tbsReference'] . '.');
                $this->logger->appendToBuffer('Could not retrieve information for certificate ' . $profile['CERTIFICATE']['tbsReference'] . '.', 'error');
                \cli\err('Error ' . $statusResponse->getXmlRpcCode() . ': ' . $statusResponse->getXmlRpcMessage());
                $this->logger->appendToBuffer('Error ' . $statusResponse->getXmlRpcCode() . ': ' . $statusResponse->getXmlRpcMessage(), 'error');
            } else {
                \cli\err('Could not retrieve information for certificate ' . $profile['CERTIFICATE']['tbsReference'] . '.');
                $this->logger->appendToBuffer('Could not retrieve information for certificate ' . $profile['CERTIFICATE']['tbsReference'] . '.', 'error');
                \cli\line();
            }

            if ($newCert) {
                $this->runHook($profile['CERTIFICATE']['tbsReference']);
            }
            
            $this->logger->setFileName($profile['CERTIFICATE']['tbsReference'] . '.log');
            $this->logger->commit();
            $this->logger->resetFileName(); //just in case there's an issue with the tbsReference in the next iteration
        }
    }

    public static function setArguments(\cli\Arguments &$arguments)
    {
        $arguments->addOption('tbs-ref', array('description' => 'TBS Reference'));
    }

    private static function mergeOptionsWithConfig($options)
    {
        $config = array();

        if (isset($options['directory-']) && !is_null($options['directory-reqs'])) {
            $config['DIRECTORIES']['requests'] = $options['directory-reqs'];
        }

        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    protected function runHook($tbsReference = null)
    {
    if (!empty($this->hook)) {
            $profileHandler = new \TBSCertBot\Controllers\HookProfileHandler($tbsReference);
            $profileHandler->buildEnv();
            
            \cli\line("Running hook: " . $this->hook);
            $this->logger->appendToBuffer("Running hook: " . $this->hook, 'info');
            \passthru($this->hook);
            
            $profileHandler->destroyEnv();
        }
    }
}
