<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\CLI\Commands;

/**
 * Orders a certificate
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Order extends Command
{
    //const DCV_INTERVAL_TIME = 60;
    //const DCV_TOTAL_TRIES = 5;
    
    private $product;
    private $request;
    private $organization;
    private $billingOrganization;
    private $corporateContact;
    private $technicalContact;
    private $legalContact;
    private $payment;
    private $miscellaneous;
    
    private $key;
    private $csr;
    
    private $hook;
    protected $dcvHook;

    
    public function __construct()
    {
        parent::__construct();
        
        self::setArguments($this->arguments);
        $this->parseArguments();
                
        self::mergeOptionsWithConfig($this->arguments->getArguments());
        
        $this->product = \TBSCertBot\Config::getConfig()->getProduct();
        $this->request = \TBSCertBot\Config::getConfig()->getRequest();
        $this->organization = \TBSCertBot\Config::getConfig()->getOrganization();
        $this->billingOrganization = \TBSCertBot\Config::getConfig()->getBillingOrganization();
        $this->legalContact = \TBSCertBot\Config::getConfig()->getLegalContact();
        $this->corporateContact = \TBSCertBot\Config::getConfig()->getCorporateContact();
        
        
        $this->technicalContact = \TBSCertBot\Config::getConfig()->getTechnicalContact();
        
        //We want to use the corporate contact as fallback if the technical contact is entirely empty
        if (array_reduce($this->technicalContact->getXmlRpc(), function($carry, $item) {return $carry && empty($item);}, true)) {
            $this->technicalContact = \TBSCertBot\Config::getConfig()->getTechnicalContactFromCorporate();
        }
        
        $this->payment = \TBSCertBot\Config::getConfig()->getPayment();
        $this->miscellaneous = \TBSCertBot\Config::getConfig()->getMiscellaneous();
        $this->hook = \TBSCertBot\Config::getConfig()->getOrderHook();
        $this->dcvHook = \TBSCertBot\Config::getConfig()->getDcvHook();
        
        //We initialize the log file using the UUID. We will change if for the TBS Ref before writing if available then.
        $this->logger->setFileName($this->request->getUniqueIdentifier() . '.log');
    }
    
    /**
     * 
     * @throws \Exception
     * @todo Handle reference in the request
     * @todo Improve error handling after the api request
     * @todo Implement customer reference in \TBSCertBot\API\Methods\Order\Request
     */
    public function proceed()
    {   
        //Generating the DCV unique value and setting it into the request
        //This generation needs to happen here so that we can send it to the request
        $dcvUniqueValue = \TBSCertBot\Crypto\Helper::getAlphanumericToken(20);
        $this->request->setDomainControlValidationUniqueValue($dcvUniqueValue);
        
        \cli\line('Validating all parameters...');
        $this->logger->appendToBuffer('Validating all parameters...', 'info');
        
        try {
                $this->validate();
            } catch (\Exception $e) {
                $this->logger->appendToBuffer($e->getMessage(), 'error');
                throw $e;
            }

        \cli\line('All parameters are OK!');
        $this->logger->appendToBuffer('All parameters are OK!', 'info');
        \cli\line();
        
        $cn = \TBSCertBot\Controllers\Validation::getCN($this->request);


        if ($this->request->getCsr()) {
            
            $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                . DIRECTORY_SEPARATOR
                . $cn . '-' . date('Y-m-d') . '.csr'
            );
            
            try {
                $this->csr = new \TBSCertBot\Crypto\ExternalCSR(array(), $this->request->getCsr(), $csrFileName, $dcvUniqueValue);
                
                if ($this->csr->getCN() && $this->csr->getCN() != $cn) {
                    $this->logger->appendToBuffer('The CN provided in the CSR does not match the CN of the request', 'error');
                    throw new \Exception('The CN provided in the CSR does not match the CN of the request');
                    
                }
            } catch (\Exception $e) {
                throw $e;
            } 
        } else {        
            if ($cn === null) {
                $this->logger->appendToBuffer('No domain name provided.', 'error');
                throw new \Exception('No domain name provided.');
            }

            $dn = array(
                "CN" => $cn,
                "O" => $this->organization->getName(),
                "L" => $this->organization->getCity(),
                "ST" => $this->organization->getState(),
                "C" => $this->organization->getCountry()
            );

            // Key generation.
            $keyFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                \TBSCertBot\Config::getConfig()->getKeysDirectoryPath()
                . DIRECTORY_SEPARATOR
                . $cn . '-' . date('Y-m-d') . '.key'
                );
            $this->key = new \TBSCertBot\Crypto\RSA($keyFileName);
            try {
                \cli\line('Generating new cryptographic key...');
                $this->logger->appendToBuffer('Generating new cryptographic key...', 'info');
                \cli\line('   - Type = ' . \TBSCertBot\Crypto\RSA::TYPE);
                $this->logger->appendToBuffer('   - Type = ' . \TBSCertBot\Crypto\RSA::TYPE, 'info');
                \cli\line('   - Length = ' . \TBSCertBot\Crypto\RSA::DEFAULT_KEY_SIZE);
                $this->logger->appendToBuffer('   - Length = ' . \TBSCertBot\Crypto\RSA::DEFAULT_KEY_SIZE, 'info');
                $this->key->generate();
                \cli\line('New cryptographic key generated!');
                $this->logger->appendToBuffer('New cryptographic key generated!', 'info');
            } catch (\Exception $e) {
                $this->logger->appendToBuffer($e->getMessage(), 'error');
                throw $e;
            }
            \cli\line();

            // CSR generation.
            $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                . DIRECTORY_SEPARATOR
                . $cn . '-' . date('Y-m-d') . '.csr'
                );
            $this->csr = new \TBSCertBot\Crypto\CSR($dn, $csrFileName, $this->key);
            try {
                \cli\line('Generating new CSR...');
                $this->logger->appendToBuffer('Generating new CSR...', 'info');
                \cli\line('   - CN = ' . $dn['CN']);
                $this->logger->appendToBuffer('   - CN = ' . $dn['CN'], 'info');
                \cli\line('   - O = ' . $dn['O']);
                $this->logger->appendToBuffer('   - O = ' . $dn['O'], 'info');
                \cli\line('   - L = ' . $dn['L']);
                $this->logger->appendToBuffer('   - L = ' . $dn['L'], 'info');
                \cli\line('   - ST = ' . $dn['ST']);
                $this->logger->appendToBuffer('   - ST = ' . $dn['ST'], 'info');
                \cli\line('   - C = ' . $dn['C']);
                $this->logger->appendToBuffer('   - C = ' . $dn['C'], 'info');

                $this->csr->generate($dcvUniqueValue);
                \cli\line('New CSR generated!');
                $this->logger->appendToBuffer('New CSR generated!', 'info');
            } catch (\Exception $e) {
                $this->logger->appendToBuffer($e->getMessage(), 'error');
                throw $e;
            }
        }
        
        \cli\line();
        
        // Affect the CSR to the Request.
        $this->request->setCSR($this->csr->getASCII());

        $validation = new \TBSCertBot\Controllers\Validation($this->request, $this->csr);
        $validation->validate($this->product->needsDcv());

        $order = new \TBSCertBot\API\Methods\Order\Request(
          $this->product,
          $this->request,
          $this->organization,
          $this->corporateContact,
          $this->payment,
          $this->technicalContact,
          $this->legalContact,
          $this->billingOrganization,
          $this->miscellaneous,
          null
        );
        
        $orderInformation = $order->save();
        $orderInformation['CSR'] = $this->csr->save();
        if ($this->key) $orderInformation['PRIVATE_KEY'] = $this->key->save();
        
        $this->runDcvHook(null, $orderInformation);
        
        if ($this->product->needsDcv() && $this->product->isDcvMandatoryToOrder()
                && !\TBSCertBot\Config::getConfig()->getBatch()) {
            \cli\line("The product you are ordering requires the DCV challenge to be successful beforehand.");
            $this->logger->appendToBuffer('The product you are ordering requires the DCV challenge to be successful beforehand.', 'info');
            
            //Use of two variable for PHP 5.3 compatibility
            $dcvRelativeFileNameArr = array_slice(explode(DIRECTORY_SEPARATOR, $this->csr->getDcvHttpFileName()), -1, 1, false);
            $dcvRelativeFileName = $dcvRelativeFileNameArr[0];
            
            $dcvDomains = preg_split('/\s+/', $this->request->getDomains());
            
            \cli\line("Please make sure the DCV file is accessible at these addresses:");
            $this->logger->appendToBuffer('Please make sure the DCV file is accessible at these addresses:', 'info');
            
            foreach($dcvDomains as $dcvDomain) {
                if (!empty($dcvDomain)) {
                    \cli\line("http://$dcvDomain/.well-known/pki-validation/" . $dcvRelativeFileName);
                    $this->logger->appendToBuffer("http://$dcvDomain/.well-known/pki-validation/" . $dcvRelativeFileName, 'info');
                }
            }
            
            \cli\line("The DCV file is available at: " . $this->csr->getDcvHttpFileName());
            $this->logger->appendToBuffer("The DCV file is available at: " . $this->csr->getDcvHttpFileName(), 'info');

            $dcvContinueChoice = "n";
            while ($dcvContinueChoice !== "y") {
                $dcvContinueChoice = \cli\choose("Would you like to resume the order?", "yn", "n");
            }
        }
        
        \cli\line('Placing order...');
        $this->logger->appendToBuffer('Placing order...', 'info');
        $response = $order->send();
                
        //Let's check if we could get a response
        
        //PHP 5.3 compat: empty needs a reference
        $tmpReference = $response->getTBSReference();
        
        //PHP 5.3 compat: empty needs a reference
        $xmlRpcCode = $response->getXmlRpcCode();
        $xmlRpcMessage = $response->getXmlRpcMessage();
        
        if (\TBSCertBot\API\Helper::isErrorCodeNegative($response->getCode())) {
            \cli\line('Order failed');
            $this->logger->appendToBuffer('Order failed', 'error');
            throw new \Exception(\TBSCertBot\API\Helper::selectErrorMessage($response->getCode() , $response->getMessage()),
                    $response->getCode());
        } elseif (empty($tmpReference) && is_int($xmlRpcCode) && !empty($xmlRpcCode)) {
            \cli\line("Could not get a response.");
            $this->logger->appendToBuffer('Could not get a response.', 'warn');
            
            if (!empty($xmlRpcCode) && !empty($xmlRpcMessage)) {
                \cli\line('Network Error No ' . $xmlRpcCode
                    . ': ' . $xmlRpcMessage);
                $this->logger->appendToBuffer('Network Error No ' . $xmlRpcCode
                    . ': ' . $xmlRpcMessage, 'warn');
            }
            
            \cli\line("Waiting for the order to be processed before trying to recover it.");
            $this->logger->appendToBuffer('Waiting for the order to be processed before trying to recover it.', 'warn');
            
            //We have to wait because an order can take a long time to be processed.
            sleep(30);
            
            try {
                $recoverable = new \TBSCertBot\Controllers\Recoverable($this->request->getUniqueIdentifier());
            
                $recoverable->recover();
                
                $tbsReference = $recoverable->getTbsReference();
                $caReference = '';
                
                /*
                 * If we could not get a TBS Reference,
                 * it means there was a network error we could not identify.
                 * 
                 * We could throw an exception but we carry on in order to save the order.
                 */
                if (empty($tbsReference)) {
                    \cli\line("Could not recover order \"" 
                            . $this->request->getUniqueIdentifier()
                            . "\"."); 
                    
                    $this->logger->appendToBuffer("Could not recover order \"" 
                            . $this->request->getUniqueIdentifier()
                            . "\".", 'error');
                    
                    \cli\line("Please try using the recover command in a few minutes.");
                } else {
                    $this->logger->setFileName($tbsReference . '.log');
                }

            } catch (\Exception $e) {
                \cli\line("Could not get a response after ordering");
                $this->logger->appendToBuffer('Could not get a response after ordering', 'error');
                \cli\line("Error: " . $e->getMessage());
                $this->logger->appendToBuffer("Error: " . $e->getMessage(), 'error');
                
                /*
                 * If we could not get a TBS Reference,
                 * it means there was a network error we could not identify.
                 * 
                 * We could throw an exception but we carry on in order to save the order.
                 */
                \cli\line("Could not recover order \"" 
                            . $this->request->getUniqueIdentifier()
                            . "\"."); 
                
                $this->logger->appendToBuffer("Could not recover order \"" 
                            . $this->request->getUniqueIdentifier()
                            . "\".", 'error');
                    
                \cli\line("Please try using the recover command in a few minutes.");
                \cli\line();
                
                // Update and save profile
                $orderInformation = array_replace_recursive($orderInformation, $order->save());
                $certificateInformation = new \TBSCertBot\API\Entities\Certificate($caReference, $tbsReference);
                $orderInformation['CERTIFICATE'] = $certificateInformation->save();
                \TBSCertBot\Controllers\Profile::setProfile($orderInformation);
                
                exit();
            }
        } elseif (!\TBSCertBot\API\Helper::isErrorCodeNegative($response->getCode())) {
            $tbsReference = $response->getTBSReference();
            $caReference = $response->getCAReference();
            
            $this->logger->setFileName($tbsReference . '.log');
        } else {
            \cli\line('Order failed');
            $this->logger->appendToBuffer('Order failed', 'error');
            $this->logger->appendToBuffer(\TBSCertBot\API\Helper::selectErrorMessage($response->getCode() , $response->getMessage), 'error');
            throw new \Exception(\TBSCertBot\API\Helper::selectErrorMessage($response->getCode() , $response->getMessage()),
                    $response->getCode());
        }
        
        \cli\line('Order taken into account under the reference ' . $tbsReference . '.');
        $this->logger->appendToBuffer('Order taken into account under the reference ' . $tbsReference . '.', 'info');
        \cli\line();
        
        // Update and save profile
        $orderInformation = array_replace_recursive($orderInformation, $order->save());
        $certificateInformation = new \TBSCertBot\API\Entities\Certificate($caReference, $tbsReference);
        $orderInformation['CERTIFICATE'] = $certificateInformation->save();
        
        \TBSCertBot\Controllers\Profile::setProfile($orderInformation);

        //Running order hook before the download (that might trigger the download hook) 
        $this->runHook($tbsReference);

        $download = new Download($tbsReference);
        $download->proceed();

        $status = new Status($tbsReference);
        \cli\line("DCV:");
        $status->displayDcv(1);
        
    }
    
    /**
     * Validates the order information
     * 
     * @todo Handle empty technical contact properly
     */
    private function validate()
    {   
        $this->product->validate();
        $this->request->validate();
        $this->organization->validate();
        $this->payment->validate();
        
        $corporateContact = array_filter((array)$this->corporateContact);
        $legalContact = array_filter((array)$this->legalContact);
        $technicalContact = array_filter((array)$this->technicalContact);
        
        $billingOrganization = array_filter(\TBSCertBot\Helper::array_flatten((array)$this->billingOrganization));
        
        $miscellaneous = array_filter((array)$this->miscellaneous);
        
        if ($corporateContact) {
            $this->corporateContact->validate();
        } elseif ($legalContact) {
            $this->legalContact->validate();
        } else { //Falllback to the corporate contact
            $this->corporateContact->validate();
        }
        
        if ($billingOrganization) {
            $this->billingOrganization->validate();
        }
        
        //For now, always check the technical contact
        $this->technicalContact->validate();
        
        
        if ($miscellaneous) {
            $this->miscellaneous->validate();
        }
    }
    
    public static function setArguments(\cli\Arguments &$arguments)
    {
        \TBSCertBot\API\Entities\Product::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\Request::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\Organization::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\CorporateContact::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\Payment::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\TechnicalContact::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\LegalContact::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\BillingOrganization::setCLIoptions($arguments);
        \TBSCertBot\API\Entities\Miscellaneous::setCLIoptions($arguments);
    }
    
    private static function mergeOptionsWithConfig($options)
    {   
        \TBSCertBot\API\Entities\Product::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\Request::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\Organization::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\CorporateContact::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\Payment::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\TechnicalContact::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\LegalContact::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\BillingOrganization::mergeOptionsWithConfig($options);
        \TBSCertBot\API\Entities\Miscellaneous::mergeOptionsWithConfig($options);
    }
    
    protected function runHook($tbsReference = null)
    {
        if (!empty($this->hook) && !empty($tbsReference)) {
            $profileHandler = new \TBSCertBot\Controllers\HookProfileHandler($tbsReference);
            $profileHandler->buildEnv();
            
            \cli\line("Running hook: " . $this->hook);
            $this->logger->appendToBuffer("Running hook: " . $this->hook, 'info');
            \passthru($this->hook);
            
            $profileHandler->destroyEnv();
        }
        
    }
    
    protected function runDcvHook($tbsReference = null, $profile)
    {
        if (!empty($this->dcvHook)) {
            $profileHandler = new \TBSCertBot\Controllers\HookProfileHandler($tbsReference, $profile);
            $profileHandler->buildEnv();
            
            \cli\line("Running DCV hook: " . $this->dcvHook);
            $this->logger->appendToBuffer("Running DCV hook: " . $this->dcvHook, 'info');
            \passthru($this->dcvHook);
            
            $profileHandler->destroyEnv();
        }
    }
}
