<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\CLI\Commands;

/**
 * Renews one or more certificates
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Renewal extends Order
{
    //Note: If you edit these values, please edit the user messages, which are hardcoded
    const RENEWAL_MINIMUM_DATE_90d = 1728000;
    const RENEWAL_MINIMUM_DATE = 7780000;
    const RENEWAL_MAXIMUM_DATE = 2592000;
    private $hook;
    private $reuseKeys;

    /**
     * 
     * @param type $tbsReference optional TBS Reference to renew only one certificate
     */
    public function __construct($tbsReference = null)
    {
        parent::__construct();
        
        self::setArguments($this->arguments);
        $this->parseArguments();
        self::mergeOptionsWithConfig($this->arguments->getArguments());
        
        $this->profiles = \TBSCertBot\Controllers\Profile::getProfiles($tbsReference);
        
        $this->hook = \TBSCertBot\Config::getConfig()->getRenewalHook();
        $this->reuseKeys = \TBSCertBot\Config::getConfig()->getReuseKeys();
    }

    /**
     * 
     * Contains the main execution logic of the command
     * 
     * @throws \Exception
     * @todo Handle organization identifier in the DN for CSR?
     * @todo Handle reference in the request
     * @todo Improve error handling after the api request
     */
    public function proceed()
    {  
        foreach ($this->profiles as $profile) {

            //We create the keys to avoid warnings. Content and type checks are run using validation functions.
            $indicies = ['CERTIFICATE', 'PRODUCT', 'REQUEST', 'ORGANIZATION',
                'CORPORATE_CONTACT', 'PAYMENT', 'TECHNICAL_CONTACT', 'LEGAL_CONTACT', 
                'BILLING_ORGANIZATION', 'MISCELLANEOUS'];
            
            foreach($indicies as $index) {
                if (!array_key_exists($index, $profile) || !is_array($profile[$index])) {
                    $profile[$index] = [];
                }
            }
            
            //We want to filter out already renewed certificates and expired ones
            if ($profile['CERTIFICATE']['renewed'] == true) {   
                if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                    \cli\line();
                    \cli\line('Checking renewal for certificate reference ' 
                            . $profile['CERTIFICATE']['tbsReference'] 
                            . ' (CN = ' . $profile['CSR']['commonName'] . ')...');

                    
                        \cli\line('Certificate already renewed.');
                    }
                    
                    $this->logger->appendToBuffer('Checking renewal for certificate reference ' 
                            . $profile['CERTIFICATE']['tbsReference'] 
                            . ' (CN = ' . $profile['CSR']['commonName'] . ')...', 'info');
                
                  $this->logger->appendToBuffer('Certificate already renewed.', 'info');
                  
                  continue;
            } elseif (!empty($profile['CERTIFICATE']['endDate']) 
                    && time() - strtotime($profile['CERTIFICATE']['endDate']) > self::RENEWAL_MAXIMUM_DATE) {
                
                if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                    \cli\line();
                    \cli\line('Checking renewal for certificate reference ' 
                            . $profile['CERTIFICATE']['tbsReference'] 
                            . ' (CN = ' . $profile['CSR']['commonName'] . ')...');

                    
                        \cli\line('Certificate expired more than thirty days ago.');
                    }
                    
                $this->logger->appendToBuffer('Checking renewal for certificate reference ' 
                        . $profile['CERTIFICATE']['tbsReference'] 
                        . ' (CN = ' . $profile['CSR']['commonName'] . ')...', 'info');

                $this->logger->appendToBuffer('Certificate expired more than thirty days ago.', 'error');
                
                continue;
            }
            
            if (isset($profile['CERTIFICATE']['certificatePEM']) && !empty($profile['CERTIFICATE']['certificatePEM'])) {
                \cli\line();
                \cli\line('Checking renewal for certificate reference ' 
                        . $profile['CERTIFICATE']['tbsReference'] 
                        . ' (CN = ' . $profile['CSR']['commonName'] . ')...');
                
                $this->logger->appendToBuffer('Checking renewal for certificate reference ' 
                        . $profile['CERTIFICATE']['tbsReference'] 
                        . ' (CN = ' . $profile['CSR']['commonName'] . ')...', 'info');
                
                if ($profile['PRODUCT']['validity'] === '30d') {
                    \cli\line('Test certificates are not renewable.');
                    $this->logger->appendToBuffer('Test certificates are not renewable.', 'info');
                }elseif ($profile['PRODUCT']['validity'] === '90d' 
                        && strtotime($profile['CERTIFICATE']['endDate']) - time() > self::RENEWAL_MINIMUM_DATE_90d) {
                    
                    \cli\line('Certificate is not to be renewed yet (20 days).');
                    $this->logger->appendToBuffer('Certificate is not to be renewed yet (20 days).', 'info');
                } elseif (in_array($profile['PRODUCT']['validity'], array('1', '2', '3', '4', '5'))
                        && strtotime($profile['CERTIFICATE']['endDate']) - time() > self::RENEWAL_MINIMUM_DATE) {
                    
                    \cli\line('Certificate is not to be renewed yet (90 days).');
                    $this->logger->appendToBuffer('Certificate is not to be renewed yet (90 days).', 'info');
                }  else {
                    \cli\line('Certificate is to be renewed.');
                    $this->logger->appendToBuffer('Certificate is to be renewed.', 'info');
                    \cli\line();
                    
                    $this->certificate = \TBSCertBot\API\Entities\Certificate::load($profile['CERTIFICATE']);
                    $this->product = \TBSCertBot\API\Entities\Product::load($profile['PRODUCT']);
                    $this->request = \TBSCertBot\API\Entities\Request::load($profile['REQUEST']);
                    $this->organization = \TBSCertBot\API\Entities\Organization::load($profile['ORGANIZATION']);
                    $this->corporateContact = \TBSCertBot\API\Entities\CorporateContact::load($profile['CORPORATE_CONTACT']);
                    $this->payment = \TBSCertBot\API\Entities\Payment::load($profile['PAYMENT']);
                    $this->technicalContact = \TBSCertBot\API\Entities\TechnicalContact::load($profile['TECHNICAL_CONTACT']);
                    $this->legalContact = \TBSCertBot\API\Entities\LegalContact::load($profile['LEGAL_CONTACT']);
                    $this->billingOrganization = \TBSCertBot\API\Entities\BillingOrganization::load($profile['BILLING_ORGANIZATION']);
                    $this->miscellaneous = \TBSCertBot\API\Entities\Miscellaneous::load($profile['MISCELLANEOUS']);
                    
                    //Generating the DCV unique value and setting it into the request
                    //This generation needs to happen here so that we can send it to the request
                    $dcvUniqueValue = \TBSCertBot\Crypto\Helper::getAlphanumericToken(20);
                    $this->request->setDomainControlValidationUniqueValue($dcvUniqueValue);

                    \cli\line('Validating all parameters...');
                    $this->logger->appendToBuffer('Validating all parameters...', 'info');
                    
                    try {
                        $this->validate();
                    } catch (\Exception $e) {
                        $this->logger->appendToBuffer($e->getMessage(), 'error');
                        throw $e;
                    }
                    
                    
                    \cli\line('All parameters are OK!');
                    $this->logger->appendToBuffer('All parameters are OK!', 'info');
                    \cli\line();

                    $cn = \TBSCertBot\Controllers\Validation::getCN($this->request);

                    if ($cn === null) {
                        $this->logger->appendToBuffer('No domain name provided.', 'error');
                        throw new \Exception('No domain name provided.');
                    }

                    if ($this->reuseKeys) {
                        $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                            \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                            . DIRECTORY_SEPARATOR
                            . $cn . '-' . date('Y-m-d') . '.csr'
                        );
                        
                        //Importing previous CSR
                        $this->csr = new \TBSCertBot\Crypto\ExternalCSR(array(), 
                                file_get_contents($profile['CSR']['fileName']), 
                                $csrFileName, $dcvUniqueValue);
                        
                        //Display the HTTP DCV File if needed
                        if ($this->request->getDomainControlValidation() === 'HTTP_CSR_HASH'
                                    || $this->request->getDomainControlValidation() === 'HTTPS_CSR_HASH') {
                                \cli\line("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "");
                                $this->logger->appendToBuffer("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "", 'info');
                        }  
                    } elseif ($this->request->getCsr()) {
                        $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                            \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                            . DIRECTORY_SEPARATOR
                            . $cn . '-' . date('Y-m-d') . '.csr'
                        );
                        
                        try {
                            $this->csr = new \TBSCertBot\Crypto\ExternalCSR(array(), 
                                    $this->request->getCsr(), 
                                    $csrFileName, $dcvUniqueValue);

                            if ($this->csr->getCN() && $this->csr->getCN() != $cn) {
                                $this->logger->appendToBuffer('The CN provided in the CSR does not match the CN of the request', 'error');
                                throw new \Exception('The CN provided in the CSR does not match the CN of the request');
                            }
                        } catch (\Exception $e) {
                            $this->logger->appendToBuffer($e->getMessage(), 'error');
                            throw $e;
                        }
                    } else {
                        $dn = array(
                            "CN" => $cn,
                            "O" => $this->organization->getName(),
                            "L" => $this->organization->getCity(),
                            "ST" => $this->organization->getState(),
                            "C" => $this->organization->getCountry()
                        );

                        // Key generation.
                        $keyFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                                        \TBSCertBot\Config::getConfig()->getKeysDirectoryPath()
                                        . DIRECTORY_SEPARATOR
                                        . $cn . '-' . date('Y-m-d') . '.key'
                        );
                        $this->key = new \TBSCertBot\Crypto\RSA($keyFileName);
                        try {
                            \cli\line('Generating new cryptographic key...');
                            $this->logger->appendToBuffer('Generating new cryptographic key...', 'info');
                            \cli\line('   - Type = ' . \TBSCertBot\Crypto\RSA::TYPE);
                            $this->logger->appendToBuffer('   - Type = ' . \TBSCertBot\Crypto\RSA::TYPE, 'info');
                            \cli\line('   - Length = ' . \TBSCertBot\Crypto\RSA::DEFAULT_KEY_SIZE);
                            $this->logger->appendToBuffer('   - Length = ' . \TBSCertBot\Crypto\RSA::DEFAULT_KEY_SIZE, 'info');
                            
                            $this->key->generate($dcvUniqueValue);

                            \cli\line('New cryptographic key generated!');
                            $this->logger->appendToBuffer('New cryptographic key generated!', 'info');
                        } catch (\Exception $e) {
                            $this->logger->appendToBuffer($e->getMessage(), 'error');
                            throw $e;
                        }
                        \cli\line();

                        // CSR generation.
                        $csrFileName = \TBSCertBot\Helper::getOrIncrementFileName(
                                        \TBSCertBot\Config::getConfig()->getRequestsDirectoryPath()
                                        . DIRECTORY_SEPARATOR
                                        . $cn . '-' . date('Y-m-d') . '.csr'
                        );
                        $this->csr = new \TBSCertBot\Crypto\CSR($dn, $csrFileName, $this->key);
                        try {
                            \cli\line('Generating new CSR...');
                            $this->logger->appendToBuffer('Generating new CSR...', 'info');
                            \cli\line('   - CN = ' . $dn['CN']);
                            $this->logger->appendToBuffer('   - CN = ' . $dn['CN'], 'info');
                            \cli\line('   - O = ' . $dn['O']);
                            $this->logger->appendToBuffer('   - O = ' . $dn['O'], 'info');
                            \cli\line('   - L = ' . $dn['L']);
                            $this->logger->appendToBuffer('   - L = ' . $dn['L'], 'info');
                            \cli\line('   - ST = ' . $dn['ST']);
                            $this->logger->appendToBuffer('   - ST = ' . $dn['ST'], 'info');
                            \cli\line('   - C = ' . $dn['C']);
                            $this->logger->appendToBuffer('   - C = ' . $dn['C'], 'info');

                            $this->csr->generate();
                            \cli\line('New CSR generated!');
                            $this->logger->appendToBuffer('New CSR generated!', 'info');

                            //Display the HTTP DCV File if needed
                            if ($this->request->getDomainControlValidation() === 'HTTP_CSR_HASH'
                                    || $this->request->getDomainControlValidation() === 'HTTPS_CSR_HASH') {
                                
                                \cli\line("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "");
                                $this->logger->appendToBuffer("DCV HTTP file created: " . $this->csr->getDcvHttpFileName() . "", 'info');
                            }
                        } catch (\Exception $e) {
                            $this->logger->appendToBuffer($e->getMessage(), 'error');
                            throw $e;
                        }
                        \cli\line();   
                    }
                    
                    // Affect the newly generated CSR to the Request.
                    $this->request->setCSR($this->csr->getASCII());

                    $validation = new \TBSCertBot\Controllers\Validation($this->request, $this->csr);
                    $validation->validate($this->product->needsDcv());

                    $validation->checkDCV();

                    // TODO: Handle reference (last argument).
                    $renewal = new \TBSCertBot\API\Methods\Renewal\Request(
                             $this->certificate, 
                            $this->product, 
                            $this->request, 
                            $this->organization, 
                            $this->corporateContact, 
                            $this->payment, 
                            $this->technicalContact, 
                            $this->legalContact, 
                            $this->billingOrganization, 
                            $this->miscellaneous, 
                            null
                    );
                    
                    $renewalInformation = $renewal->save();
                    $renewalInformation['CSR'] = $this->csr->save();
                    if ($this->key) $renewalInformation['PRIVATE_KEY'] = $this->key->save();
                    
                    $this->runDcvHook(null, $renewalInformation);
                    
                    if ($this->product->needsDcv() && $this->product->isDcvMandatoryToOrder()
                            && !\TBSCertBot\Config::getConfig()->getBatch()) {
                        \cli\line("The product you are ordering requires the DCV challenge to be successful beforehand.");
                        $this->logger->appendToBuffer('The product you are ordering requires the DCV challenge to be successful beforehand.', 'info');

                        //Use of two variable for PHP 5.3 compatibility
                        $dcvRelativeFileNameArr = array_slice(explode(DIRECTORY_SEPARATOR, $this->csr->getDcvHttpFileName()), -1, 1, false);
                        $dcvRelativeFileName = $dcvRelativeFileNameArr[0];

                        $dcvDomains = preg_split('/\s+/', $this->request->getDomains());
            
                        \cli\line("Please make sure the DCV file is accessible at these addresses:");
                        $this->logger->appendToBuffer('Please make sure the DCV file is accessible at these addresses:', 'info');

                        foreach($dcvDomains as $dcvDomain) {
                            if (!empty($dcvDomain)) {
                                \cli\line("http://$dcvDomain/.well-known/pki-validation/" . $dcvRelativeFileName);
                                $this->logger->appendToBuffer("http://$dcvDomain/.well-known/pki-validation/" . $dcvRelativeFileName, 'info');
                            }
                        }

                        \cli\line("The DCV file is available at: " . $this->csr->getDcvHttpFileName());
                        $this->logger->appendToBuffer("The DCV file is available at: " . $this->csr->getDcvHttpFileName(), 'info');

                        $dcvContinueChoice = "n";
                        while ($dcvContinueChoice !== "y") {
                            $dcvContinueChoice = \cli\choose("Would you like to resume the order?", "yn", "n");
                        }
                    }

                    \cli\line('Placing order...');
                    $this->logger->appendToBuffer('Placing order...', 'info');
                    $response = $renewal->send();
                    
                    //Let's check if we could get a response
                    
                    //Pre PHP 5 empty limitation
                    $tmpReference = $response->getTBSReference();
                    
                    //PHP 5.3 compat: empty needs a reference
                    $xmlRpcCode = $response->getXmlRpcCode();
                    $xmlRpcMessage = $response->getXmlRpcMessage();
                    
                    if (\TBSCertBot\API\Helper::isErrorCodeNegative($response->getCode())) {
                        \cli\line('Order failed');
                        $this->logger->appendToBuffer('Order failed', 'error');
                        
                        $this->logger->appendToBuffer(\TBSCertBot\API\Helper::selectErrorMessage($response->getCode() , $response->getMessage()), 'error');
                        throw new \Exception(\TBSCertBot\API\Helper::selectErrorMessage($response->getCode() , $response->getMessage()),
                            $response->getCode());
                    } elseif (empty($tmpReference) && is_int($xmlRpcCode) && !empty($xmlRpcCode)) {
                        \cli\line("Could not get a reponse.");
                        $this->logger->appendToBuffer('Could not get a reponse.', 'warn');
                        
                        if (!empty($xmlRpcCode) && !empty($xmlRpcMessage)) {
                            \cli\line('Network Error No ' . $xmlRpcCode
                                . ': ' . $xmlRpcMessage);
                            
                            $this->logger->appendToBuffer('Network Error No ' . $xmlRpcCode
                                . ': ' . $xmlRpcMessage, 'warn');
                        }
                        
                        \cli\line("Waiting for the order to be processed before trying to recover it.");
                        $this->logger->appendToBuffer('Waiting for the order to be processed before trying to recover it.', 'info');

                        //We have to wait because an order can take a long time to be processed.
                        sleep(30);

                        try {
                            $recoverable = new \TBSCertBot\Controllers\Recoverable($this->request->getUniqueIdentifier());

                            $recoverable->recover();

                            $tbsReference = $recoverable->getTbsReference();
                            $caReference = '';

                            /*
                             * If we could not get a TBS Reference,
                             * it means there was a network error we could not identify.
                             * 
                             * We could throw an exception but we carry on in order to save the order.
                             */
                            if (empty($tbsReference)) {
                                \cli\line("Could not recover order \"" 
                                        . $this->request->getUniqueIdentifier()
                                        . "\".");
                                
                                $this->logger->appendToBuffer("Could not recover order \"" 
                                        . $this->request->getUniqueIdentifier()
                                        . "\".", 'error');
                                
                                \cli\line("Please try using the recover command in a few minutes.");
                                $this->logger->appendToBuffer("Please try using the recover command in a few minutes.", 'error');
                            } else {
                                $this->logger->setFileName($tbsReference . '.log');
                            }

                        } catch (\Exception $e) {
                            \cli\line("Could not get a response after ordering");
                            $this->logger->appendToBuffer('Could not get a response after ordering', 'error');
                            \cli\line("Error: " . $e->getMessage());
                            $this->logger->appendToBuffer("Error: " . $e->getMessage(), 'error');
                            
                            \cli\line("Could not recover order \"" 
                                . $this->request->getUniqueIdentifier()
                                . "\".");
                            
                            $this->logger->appendToBuffer("Could not recover order \"" 
                                . $this->request->getUniqueIdentifier()
                                . "\".", 'error');
                            
                            \cli\line("Please try using the recover command in a few minutes.");
                            $this->logger->appendToBuffer('Please try using the recover command in a few minutes.', 'error');
                            
                            \cli\line();
                            
                            // Update and save profile
                            $renewalInformation = array_replace_recursive($renewalInformation, $renewal->save());
                            $certificateInformation = new \TBSCertBot\API\Entities\Certificate($caReference, $tbsReference);
                            $renewalInformation['CERTIFICATE'] = $certificateInformation->save();

                            \TBSCertBot\Controllers\Profile::setProfile($renewalInformation);
                            
                            exit();
                        }
                    } elseif (!\TBSCertBot\API\Helper::isErrorCodeNegative($response->getCode())) {
                        $tbsReference = $response->getTBSReference();
                        $this->logger->setFileName($tbsReference . '.log');
                    } else {
                        \cli\line('Order failed');
                        $this->logger->appendToBuffer('Order failed', 'error');
                        
                        $this->logger->appendToBuffer(\TBSCertBot\API\Helper::selectErrorMessage($response->getMessage(), $response->getCode()), 'error');
                        throw new \Exception(\TBSCertBot\API\Helper::selectErrorMessage($response->getMessage(), $response->getCode()),
                                $response->getCode());
                    }
                    
                    \cli\line('Order taken into account under the reference ' . $tbsReference . '.');
                    $this->logger->appendToBuffer('Order taken into account under the reference ' . $tbsReference . '.', 'info');
                    \cli\line();

                    // Update and save profile
                    $renewalInformation = array_replace_recursive($renewalInformation, $renewal->save());
                    $certificateInformation = new \TBSCertBot\API\Entities\Certificate($caReference, $tbsReference);
                    $renewalInformation['CERTIFICATE'] = $certificateInformation->save();

                    \TBSCertBot\Controllers\Profile::setProfile($renewalInformation);

                    $profile['CERTIFICATE']['renewed'] = true;
                    \TBSCertBot\Controllers\Profile::setProfile($profile);

                    //Running order hook before the download (that might trigger the download hook) 
                    $this->runHook($tbsReference);

                    $download = new Download($tbsReference);
                    $download->proceed();
                }
            }
        }
    }
    
    public static function setArguments(\cli\Arguments &$arguments) 
    {
        $arguments->addFlag('reuse-keys', array('description' => 'Reuses keys from previous orders when renewing or reissueing'));
        $arguments->addFlag('no-reuse-keys', array('description' => 'Prevents reusing keys from previous orders when renewing or reissueing'));
    }
    
    private static function mergeOptionsWithConfig($options)
    {
        $config = array();
        
        if (!empty($options['reuse-keys'])) {
            $config['reuseKeys'] = true;
        } elseif ($options['no-reuse-keys']) {
            $config['reuseKeys'] = false;
        }
        
        if (!empty($config)) {
            \TBSCertBot\Config::getConfig()->update($config);
        }
    }
    
    private function validate()
    {   
        $this->product->validate();
        $this->request->validate();
        $this->organization->validate();
        $this->payment->validate();
        
        $corporateContact = array_filter((array)$this->corporateContact);
        $legalContact = array_filter((array)$this->legalContact);
        $technicalContact = array_filter((array)$this->technicalContact);
        
        $billingOrganization = array_filter(\TBSCertBot\Helper::array_flatten((array)$this->billingOrganization));
        
        $miscellaneous = array_filter((array)$this->miscellaneous);
        
        if ($corporateContact) {
            $this->corporateContact->validate();
        } elseif ($legalContact) {
            $this->legalContact->validate();
        } else { //Falllback to the corporate contact
            $this->corporateContact->validate();
        }
        
        if ($billingOrganization) {
            $this->billingOrganization->validate();
        }
        if ($technicalContact) {
            $this->technicalContact->validate();
        }
        
        if ($miscellaneous) {
            $this->miscellaneous->validate();
        }
    }
}
