<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\CLI\Commands;

/**
 * Tests a hooks
 *
 */
abstract class TestHook extends Command
{
    public static $validHooks = ['order', 'renewal', 'download', 'dcv'];
    
    private static $TEST_HOOKS_DIRECTORY;
    
    protected $fakeMode;
    protected $hookName;
    
    protected $tbsreference;
    
    
    
    /**
     *
     * @var \TBSCertBot\Crypto\RSA Private Key
     */
    protected $privateKey;
   
    /**
     *
     * @var type \TBSCertBot\Crypto\CSR
     */
    protected $csr;
    
    protected $privateKeyFileName;
    protected $csrFileName;
    protected $dcvHttpFileName;
    protected $certificateFileName;
    protected $chainFileName;
        
    public function __construct($hook, $tbsreference = null) {
        parent::__construct();
        
        $this->hookName = $hook;
        
        if (!empty($tbsreference)) {
            $this->tbsreference = $tbsreference;
            $this->fakeMode = false;
        } else {
            $this->fakeMode = true;
        }
    }
    
    private static function getTestHooksDirectory()
    {
        if (empty(self::$TEST_HOOKS_DIRECTORY)) {
            self::$TEST_HOOKS_DIRECTORY = 'data'. DIRECTORY_SEPARATOR . 'testhooks';
        }
        
        return self::$TEST_HOOKS_DIRECTORY;
    }
    
    public function __destruct() {
        $this->cleanUp();
    }
    
    public static function create($hook, $tbsreference = null)
    {
        if (in_array($hook, self::$validHooks)) {
            switch(strtolower($hook)) {
                case 'order':
                    return new TestHooks\Order($tbsreference);
                case 'renewal':
                    return new TestHooks\Renewal($tbsreference);
                case 'download':
                    return new TestHooks\Download($tbsreference);
                case 'dcv':
                    return new TestHooks\Dcv($tbsreference);
                default:
                    $this->logger->log("Missing hook. Please select a valid hook: \""
                        . implode(', ', self::$validHooks) 
                        . "\"\nUsage: php tbscerbot test-hook HOOK-NAME", 'error');
                    
                    throw new \Exception("Missing hook. Please select a valid hook: \""
                        . implode(', ', self::$validHooks) 
                        . "\"\nUsage: php tbscerbot test-hook HOOK-NAME");
            }
        } else {
            $this->logger->log("Missing hook. Please select a valid hook: \""
                . implode(', ', self::$validHooks) 
                    . "\"\nUsage: php tbscerbot test-hook HOOK-NAME [TBS Reference]", 'error');
            
            throw new \Exception("Missing hook. Please select a valid hook: \""
                . implode(', ', self::$validHooks) 
                    . "\"\nUsage: php tbscerbot test-hook HOOK-NAME [TBS Reference]");
    
        }
    }
    
    public function proceed()
    {        
        try {
            if (!empty($this->tbsreference)) {
                $profileHandler = new \TBSCertBot\Controllers\HookProfileHandler($this->tbsreference);
            } else {
                $profile = $this->makeFakeProfile();

                //We don't have a TBS reference so the first parameter is empty
                $profileHandler = new \TBSCertBot\Controllers\HookProfileHandler('1234567890', $profile);
            }

            $profileHandler->buildEnv();

            \cli\line("Running " . $this->hookName . " hook: " . $this->hook);
            $this->logger->log("Running " . $this->hookName . " hook: " . $this->hook, 'info');
            \passthru($this->hook);

            //This call should be moved to a finally if we upgrade the minimum PHP version to 5.5
            $profileHandler->destroyEnv();
        } catch (\Exception $e) {
            //This call should be moved to a finally if we upgrade the minimum PHP version to 5.5
            if ($profileHandler instanceof \TBSCertBot\Controllers\HookProfileHandler) {
                $profileHandler->destroyEnv();
            }

            $this->logger->log($e->getMessage(), 'error');
            throw $e;
        }
    }
    
    /**
     * Generates a private key and stores it with full path to the file
     *
     * @todo Ensure filename is unique
     */
    protected function generateKey()
    {
        $privateKeyFileName = \TBSCertBot\Helper::getDirectory(self::getTestHooksDirectory()). DIRECTORY_SEPARATOR . 'testKey-' . date('Y-m-d') . '.pkey';
        $this->privateKey = new \TBSCertBot\Crypto\RSA($privateKeyFileName);
        $this->privateKey->generate();
        
        $this->privateKeyFileName = realpath($privateKeyFileName);
    }
    
    /**
     * Generates the CSR and stores it with its full path
     * 
     * @param array $dn DN of the future CSR
     */
    protected function generateCsr()
    {
        //We try to set the CN using the profile, if we don't have a valid CN, we set it to a test default
        if (!($cn = \TBSCertBot\Controllers\Validation::getCN(\TBSCertBot\Config::getConfig()->getRequest()))) {
            $cn = 'example.org';
        }
        
        $dn = ['CN' => $cn];
        
        $csrFileName = \TBSCertBot\Helper::getDirectory(self::getTestHooksDirectory()) . DIRECTORY_SEPARATOR . 'testCsr-' . date('Y-m-d') . '.csr';
        
        $this->csr = new \TBSCertBot\Crypto\CSR($dn, $csrFileName, $this->privateKey);
        $this->csr->generate(\TBSCertBot\Crypto\Helper::getAlphanumericToken(20));
        $this->csrFileName = realpath($csrFileName);
    }
    
    /**
     * Gets the DCV file from the CSR object and normalizes it for hook testing
     * 
     * @throws \Exception
     */
    protected function generateDcv()
    {
        if (empty($this->csrFileName) 
                || !is_file($this->csrFileName)
                || !($this->csr instanceof \TBSCertBot\Crypto\CSR)) {
            
            $this->logger->log('Cannot generate DCV: no CSR', 'error');
            throw new \Exception("Cannot generate DCV: no CSR");
        }
        
        $tmpDcvFileName = $this->csr->getDcvHttpFileName();
        
        $dcvFileName = \TBSCertBot\Helper::getDirectory(self::getTestHooksDirectory()) . DIRECTORY_SEPARATOR . 'testDcv-' . date('Y-m-d') . '.txt';
        
        if (!empty($tmpDcvFileName) && is_file($tmpDcvFileName)) {
            rename($tmpDcvFileName, $dcvFileName);
        } else {
            $this->logger->log('DcvFile not generated', 'error');
            throw new \Exception("DcvFile not generated");
        }
        
        $this->dcvHttpFileName = realpath($dcvFileName);
    }
    
    /**
     * Generates a CA cert/chain and a certificate signed by it and stores full paths to files
     * 
     * @throws \Exception
     */
    protected function generateCertAndChain()
    {
        if (empty($this->privateKeyFileName) 
                || !is_file($this->privateKeyFileName)
                || !($this->privateKey instanceof \TBSCertBot\Crypto\RSA)) {
            
            $this->logger->log('Cannot generate certificate: no private Key', 'error');
            throw new \Exception("Cannot generate certificate: no private Key");
        }
        
        //CA part
        $caPrivateKeyFileName = \TBSCertBot\Helper::getDirectory(self::getTestHooksDirectory()) . DIRECTORY_SEPARATOR . 'testCa-' . date('Y-m-d') . '.pkey';
        $caPrivateKey = new \TBSCertBot\Crypto\RSA($caPrivateKeyFileName);
        $caPrivateKey->generate();
        $caDn = ['CN' => 'TBSCertBot Test Certificate Authority'];
        $caCsrFileName = \TBSCertBot\Helper::getDirectory(self::getTestHooksDirectory()) . DIRECTORY_SEPARATOR . 'testCa-' . date('Y-m-d') . '.csr';
        $caCsr = $csr = new \TBSCertBot\Crypto\CSR($caDn, $caCsrFileName, $caPrivateKey);
        $caCsr->generate(\TBSCertBot\Crypto\Helper::getAlphanumericToken(20));
        $caCertFileName = \TBSCertBot\Helper::getDirectory(self::getTestHooksDirectory()) . DIRECTORY_SEPARATOR . 'testCa-' . date('Y-m-d') . '.cer';
        $caCert = new \TBSCertBot\Crypto\Certificate($csr, $caCertFileName);
        $caCert->generate();
        
        $this->chainFileName = realpath($caCertFileName);
        
        //Certificate part
        
        //First, we'll need to generate a CSR, that we will later sign
        $this->generateCsr();
        
        $certFileName = \TBSCertBot\Helper::getDirectory(self::getTestHooksDirectory()) . DIRECTORY_SEPARATOR . 'testCert-' . date('Y-m-d') . '.cer';
        
        //Then we start the certificate generation
        $cert = new \TBSCertBot\Crypto\Certificate($this->csr, $certFileName);
        
        //Setting our test CA a the CA
        $cert->setAuthority($caPrivateKey, $caCert);
        
        $cert->generate();
        
        
        $this->certificateFileName = realpath($certFileName);
        
        //Deleting the files which are not needed for the rest of the test hook
        if (!empty($caPrivateKeyFileName) && is_file($caPrivateKeyFileName)) unlink($caPrivateKeyFileName);
        if (!empty($caCsrFileName) && is_file($caCsrFileName)) unlink($caCsrFileName);
    }
    
    /**
     * Cleans up the temporary files if a fake dossier is used
     */
    protected function cleanUp()
    {
        //If we are working with real data, we leave the function
        if (!$this->fakeMode) return;

        if (!empty($this->privateKeyFileName) && is_file($this->privateKeyFileName)) {
            unlink($this->privateKeyFileName);
        }
        
        if (!empty($this->csrFileName) && is_file($this->csrFileName)) {
            unlink($this->csrFileName);
        }
        
        if (!empty($this->certificateFileName) && is_file($this->certificateFileName)) {
            unlink($this->certificateFileName);
        }
        
        if (!empty($this->chainFileName) && is_file($this->chainFileName)) {
            unlink($this->chainFileName);
        }
    }

   
    protected abstract function makeFakeProfile();

    /**
     * 
     * Does nothing for now. We could imagine runing a hook after we are done testing a hook but that seems a bit strange.
     * 
     * @param type $tbsReference
     */    
    protected function runHook($tbsReference = null)
    {

    }
}
