<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\CLI;

/**
 * Main routing class to determine which command will be used
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
abstract class Router
{
    public static function createCommand($arguments)
    {
        if (count($arguments) < 2) {
            self::printUsage();
            exit();
        } else {
            $commandName = $arguments[1];

            switch ($commandName) {
                case 'show':
                    $command = new Commands\Show();
                    break;
                case 'order':
                    $command = new Commands\Order();
                    break;
                case 'renew':
                    if ($arguments[2]) {
                        $command = new Commands\Renewal($arguments[2]);
                    } else {
                        $command = new Commands\Renewal();
                    }
                    break;
                case 'copy':
                    if ($arguments[2]) {
                        $command = new Commands\Copy($arguments[2]);
                    } else {
                        throw new \Exception("You must pass a TBS Reference to copy");
                    }
                    break;
                case 'resend':
                    $command = new Commands\Resend();
                    break;
                case 'download':
                    if ($arguments[2]) {
                        $command = new Commands\Download($arguments[2]);
                        
                    } else {
                        $command = new Commands\Download();

                    }
                    break;
                case 'cron':
                    $command = new Commands\Cron();
                    break;
                case 'status':
                    if ($arguments[2]) {
                        $command = new Commands\Status($arguments[2]);
                    } else {
                        echo PHP_EOL . 'No parameter provided for status command.' . PHP_EOL;
                        self::printUsage();
                        exit();
                    }
                    break;
                case 'recover':
                    if ($arguments[2]) {
                        $command = new Commands\Recover($arguments[2]);
                    } else {
                        $command = new Commands\Recover();
                    }
                    break;
                case 'test-hook':
                    if ($arguments[2]) {
                        if ($arguments[3]) {
                            //Create will return the test hook for the hook type
                            $command = Commands\TestHook::create($arguments[2], $arguments[3]);
                        } else {
                           //Create will return the test hook for the hook type
                            $command = Commands\TestHook::create($arguments[2]);
                        }
                    } else {
                        $command = Commands\TestHook::create(null);
                    }
                    break;
                case 'archive':
                    if ($arguments[2]) {
                        $command = new Commands\Archive($arguments[2]);
                    } else {
                        $command = new Commands\Archive();
                    }
                    break;
                case 'unarchive':
                    if ($arguments[2]) {
                        $command = new Commands\Unarchive($arguments[2]);
                    } else {
                        $command = new Commands\Unarchive();
                    }
                    break;
                case 'show-archived':
                    $command = new Commands\ShowArchived();
                    break;
                case 'export':
                    if ($arguments[2] && $arguments[3]) {
                        $command = new Commands\Export($arguments[2], $arguments[3]);
                    } else {
                        throw new \Exception("You must provide a profile name to export and an export file path");
                    }
                    break;
                    
                case 'import':
                    if ($arguments[2]) {
                        $command = new Commands\Import($arguments[2]);
                    } else {
                        throw new \Exception("You must provide an import file path");
                    }
                    break;
                case 'version':
                    $command = new Commands\Version();
                    break;
                case 'help':
                    self::displayHelp($arguments[2]);
                    exit();
                default:
                    echo PHP_EOL . 'Invalid command.' . PHP_EOL;
                    self::printUsage();
                    exit();
            }
            $command->proceed();
        }
    }
    
    /**
     * 
     * Displays the generic helps or route to a specifc subject
     * 
     * @param type $subject section on which the help is required
     */
    public static function displayHelp($subject)
    {
        switch($subject) {
            case 'payment-mode':
                \TBSCertBot\API\Entities\Payment::modeHelp();
                break;
            case 'country-code':
                \TBSCertBot\API\Entities\Organization::countryHelp();
                break;
            case 'organization-type':
                \TBSCertBot\API\Entities\Organization::typeHelp();
                break;
            case 'product':
                \TBSCertBot\API\Entities\Product::productHelp();
                break;
            case 'software':
                \TBSCertBot\API\Entities\Request::softwareHelp();
                break;
            case 'naf-code':
                \TBSCertBot\API\Entities\Organization::nafCodeHelp();
                break;
            default:
                self::printUsage();
        }
    }
    
    /**
     * Displays the help screen
     * 
     * @return HelpScreen
     * @link https://github.com/wp-cli/php-cli-tools/blob/master/lib/cli/arguments/HelpScreen.php
     */
    private static function setHelpScreen()
    {
        $arguments = new \cli\Arguments();
        
        Commands\Command::setArguments($arguments);
        Commands\Order::setArguments($arguments);
        Commands\Renewal::setArguments($arguments);
        
        return $arguments->getHelpScreen();
    }
    
    /**
     * Prints the usage
     */
    public static function printUsage()
    {
        echo PHP_EOL . Commands\Version::getVersion() . PHP_EOL;
        echo PHP_EOL . 'Usage: php tbscertbot.php command [command parameter] [--config | --select-config | --help] ... [flags | options] ...' . PHP_EOL;
        
        echo PHP_EOL;
        echo 'Commands:' . PHP_EOL;
        echo "    show \t\t\t Show handled certificates by TBSCertBot." . PHP_EOL;
        echo "    download \t\t\t Download certificates that are issued." . PHP_EOL;
        echo "    status TBS_Ref \t\t Displays the status of a certificate" . PHP_EOL;
        echo "    order \t\t\t Order a new certificate." . PHP_EOL;
        echo "    renew [TBS_Ref] \t\t Renews a certificate." . PHP_EOL;
        echo "    cron \t\t\t Runs the cron mode to automatically download/renew certificate" . PHP_EOL;
        echo "    recover [orderId] \t\t Recovers one or multiple orders in case of network issues." . PHP_EOL;
        //echo "    resend \t\t\t Resend the DCV e-mail." . PHP_EOL;
        echo "    test-hook HOOK [TBS_Ref] \t Runs a hook, on an existing certificate if provided" . PHP_EOL;
        echo "    archive TBS_Ref \t\t Archives a certificate to remove it from listing and cron" . PHP_EOL;
        echo "    unarchive TBS_Ref \t\t Unarchives a certificate" . PHP_EOL;
        echo "    show-archived \t\t Shows archived certificates" . PHP_EOL;
        echo "    export TBS_Ref /path/my.ini  Exports a profile to ini conf file" . PHP_EOL;
        echo "    import /path/profile.json \t Imports a profile" . PHP_EOL;
        echo "    version \t\t\t Displays the TBSCertBot version." . PHP_EOL;
        echo "    help \t\t\t Display TBSCertBot's help." . PHP_EOL;
        echo "    help product \t\t Display help for product codes." . PHP_EOL;
        echo "    help software \t\t Display help for software codes." . PHP_EOL;
        echo "    help country-code \t\t Display help for country codes." . PHP_EOL;
        echo "    help organization-type \t Display help for organization types." . PHP_EOL;
        echo "    help payment-mode \t\t Display help for payments modes." . PHP_EOL;
        echo "    help naf-code \t\t Display help for naf codes." . PHP_EOL;
        echo PHP_EOL;
        echo self::setHelpScreen();
        echo PHP_EOL . PHP_EOL;
    }
}
