<?php

/* 
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot;

use Piwik\Ini as ConfigFileHelper;

/**
 * Singleton class providing configuration values through INI files and allows
 * overwrite these values at runtime through CLI.
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 * @todo Add a settings validator to be sure we're using consistent data.
 * @todo Add logging and error management.
 */
class Config
{
    private static $DEFAULT_CONFIG_FILE_PATH; 
    
    /**
     *
     * @var Config Instance of the Config singleton.
     */
    private static $instance;
    
    private $filePath;
    private $settings = [];
    
    private $billingOrganization;
    private $certificate;
    private $corporateContact;
    private $legalContact;
    private $miscellaneous;
    private $organization;
    private $payment;
    private $product;
    private $request;
    private $technicalContact;
    
    /**
     * @param string $specificConfigFilePath Specific INI config file path.
     * @return Config Instance of the Config singleton.
     */
    private function __construct($specificConfigFilePath = null)
    {
        if (!is_null($specificConfigFilePath)) {
            $this->filePath = $specificConfigFilePath;
            
            $this->setMainSettings();
        } else {
            $this->filePath = self::getDefaultConfigFilePath();
        }
        
        $this->setSettings();
    }
    
    private static function getDefaultConfigFilePath()
    {
        if (empty(self::$DEFAULT_CONFIG_FILE_PATH)) {
            self::$DEFAULT_CONFIG_FILE_PATH = 'data'. DIRECTORY_SEPARATOR . 'conf.ini';
        }
        
        return self::$DEFAULT_CONFIG_FILE_PATH;
    }
    
    /**
     * Function filling the internal settings array with default or specific
     * INI file.
     * @todo Improve error management
     */
    private function setSettings()
    {
        try {
            $reader = new ConfigFileHelper\IniReader();
            //settings should contain main config data like server adderss, login, whereas filepath may not
            $this->settings = array_merge($this->settings, $reader->readFile($this->filePath));
            unset($reader);
        } catch (ConfigFileHelper\IniReadingException $ex) {
            throw $ex;
        }
    }
    
    /**
     * Sets the main configuration from the default configuration file. Used when we provide a secondary ini file
     * 
     * @throws \Piwik\Ini\IniReadingException
     */
    private function setMainSettings()
    {
        try {
            $reader = new ConfigFileHelper\IniReader();
            
            $defaultSettings = $reader->readFile(self::getDefaultConfigFilePath());
            
            $mainSettings = [
                'debug' => $defaultSettings['debug'],
                'batch' => $defaultSettings['batch'],
                'reuseKeys' => $defaultSettings['reuseKeys'],
                'help' => $defaultSettings['help'],
                'API' => $defaultSettings['API'], //DIRECTORY!
                'DIRECTORIES' => $defaultSettings['DIRECTORIES'], //DIRECTORY!
                'HOOKS' => $defaultSettings['HOOKS'] //DIRECTORY!
            ];
            
            $this->settings = $mainSettings;
            unset($reader);
        } catch (ConfigFileHelper\IniReadingException $ex) {
            throw $ex;
        }
    }
    
    //General settings
    
    public function getDebug()
    {
        return $this->settings['debug'];
    }
    
    public function getBatch()
    {
        return (bool)$this->settings['batch'];
    }
    
    public function getHelp()
    {
        return (bool)$this->settings['help'];
    }
    
    public function getLogLevel()
    {
        if (in_array(strtolower($this->settings['logLevel']), ['error', 'warn', 'info', 'debug'])) {
            return strtolower($this->settings['logLevel']);
        } else {
            return 'error';
        }
    }
    
    //API settings
    
    public function getAPIProtocol()
    {
        return $this->settings['API']['protocol'];
    }
    
    public function getAPITimeout()
    {
        return $this->settings['API']['timeout'];
    }
    
    public function getAPIAddress()
    {
        return $this->settings['API']['address'];
    }
    
    public function getAPIPort()
    {
        return $this->settings['API']['port'];
    }
    
    public function getAPIUsername()
    {
        return $this->settings['API']['username'];
    }
    
    public function getAPIPassword()
    {
        return $this->settings['API']['password'];
    }

    public function getReuseKeys()
    {   
        if (empty($this->settings['reuseKeys'])) return false;
        else return true;
    }
    
    //Directory settings
    
    public function getConfigurationDirectoryPath()
    {
        return Helper::getDirectory($this->settings['DIRECTORIES']['config']);
    }
    
    public function getLogsDirectoryPath()
    {
        return Helper::getDirectory($this->settings['DIRECTORIES']['logs']);
    }
    
    public function getCertificatesDirectoryPath()
    {
        return Helper::getDirectory($this->settings['DIRECTORIES']['certificates']);
    }
    
    public function getKeysDirectoryPath()
    {
        return Helper::getDirectory($this->settings['DIRECTORIES']['keys']);
    }
    
    public function getRequestsDirectoryPath()
    {
        return Helper::getDirectory($this->settings['DIRECTORIES']['requests']);
    }
    
    //Hook settings
    public function getDcvHook()
    {
        if (!empty($this->settings['HOOKS']['dcv'])) {
           return $this->settings['HOOKS']['dcv']; 
        } else {
            return "";
        }
        
    }
    
    public function getOrderHook()
    {
        if (!empty($this->settings['HOOKS']['order'])) {
           return $this->settings['HOOKS']['order']; 
        } else {
            return "";
        }
        
    }
    
    public function getRenewalHook()
    {
        if (!empty($this->settings['HOOKS']['renewal'])) {
            return $this->settings['HOOKS']['renewal'];
        } else {
            return "";
        }
        
    }
    
    public function getDownloadHook()
    {
        if (!empty($this->settings['HOOKS']['download'])) {
            return $this->settings['HOOKS']['download'];
        } else {
            return "";
        }
    }
    
    //Request and product Settings
    
    public function getMainDomain()
    {
        return $this->settings['REQUEST']['mainDomain'];
    }
    
    public function getDomains()
    {
        return $this->settings['REQUEST']['domains'];
    }
    
    public function getProduct()
    {
        if (is_null($this->product)) {
            $information = $this->settings['PRODUCT'];
            $this->product = new API\Entities\Product(
                    $information['code'],
                    $information['validity']
            );
        }
        return $this->product;
    }
    
    public function getRequest()
    {
        if (is_null($this->request)) {
            $information = $this->settings['REQUEST'];
            
            $this->request = new API\Entities\Request(
                    $information['software'],
                    (array_key_exists('csr', $information) ? $information['csr'] : null),
                    $information['mainDomain'],
                    $information['domains'],
                    (array_key_exists('domainControlValidation', $information) ? $information['domainControlValidation'] : ''),
                    (array_key_exists('licenseNumber', $information) ? $information['licenseNumber'] : 0),
                    (array_key_exists('cloningReference', $information) ? $information['cloningReference'] : null),
                    (array_key_exists('guidanceOption', $information) ? $information['guidanceOption'] : null),
                    (array_key_exists('hashAlgorithm', $information) ? $information['hashAlgorithm'] : null)
            );
        }
        return $this->request;
    }
    
    //Organization settings
    
    public function getOrganization()
    {
        if (is_null($this->organization)) {
            $information = $this->settings['ORGANIZATION'];
            
            $this->organization = new API\Entities\Organization(
                    $information['name'], 
                    $information['address'],
                    $information['city'],
                    $information['postalCode'], 
                    $information['state'],
                    $information['country'],
                    $information['phoneNumber'],
                    $information['type'], 
                    $information['number'],
                    (array_key_exists('nafCode', $information) ? $information['nafCode'] : null),
                    (array_key_exists('website', $information) ? $information['website'] : null),
                    (array_key_exists('vatNumber', $information) ? $information['vatNumber'] : null),
                    (array_key_exists('accountingEmailAddress', $information) ? $information['accountingEmailAddress'] : null)
            );
        }
        return $this->organization;
    }
    
    public function getBillingOrganization()
    {
        if (is_null($this->billingOrganization)) {
            if (array_key_exists('BILLING_ORGANIZATION', $this->settings) && !empty($this->settings['BILLING_ORGANIZATION'])) {
                $information = $this->settings['BILLING_ORGANIZATION'];
            } else {
                trigger_error("Billing information was not supplied" , E_USER_NOTICE);
                
                $indices = ['name', 'address','city' ,'postalCode' , 'state',
                    'country' ,'phoneNumber' ,'type' , 'number', 'nafCode',
                    'website', 'vatNumber', 'accountingEmailAddress'];
                
                foreach ($indices as $index) {
                    $information[$index] = null;
                }
            }

            $this->billingOrganization = new API\Entities\BillingOrganization(
                    $information['name'], 
                    $information['address'],
                    $information['city'],
                    $information['postalCode'], 
                    $information['state'],
                    $information['country'],
                    $information['phoneNumber'],
                    $information['type'], 
                    $information['number'],
                    $information['nafCode'],
                    (array_key_exists('website', $information) ? $information['website'] : null),
                    $information['vatNumber'],
                    $information['accountingEmailAddress']
            );
        }
        return $this->billingOrganization;
    }
    
    //Contact settings
    
    public function getCorporateContact()
    {
        if (is_null($this->corporateContact)) {
            $information = $this->settings['CORPORATE_CONTACT'];
            
            $this->corporateContact = new API\Entities\CorporateContact(
                    $information['title'],
                    $information['firstName'],
                    $information['lastName'],
                    $information['function'],
                    $information['phoneNumber'],
                    $information['emailAddress'],
                    $information['language']
            );
        }
        return $this->corporateContact;
    }
    
    public function getLegalContact()
    {
        if (is_null($this->legalContact)) {
            //Legal contact may be empty for non RGS certificates
            if (array_key_exists('LEGAL_CONTACT', $this->settings) && !empty($this->settings['LEGAL_CONTACT'])) {
                $information = $this->settings['LEGAL_CONTACT'];
            } else {
                trigger_error("Legal contact was not supplied" , E_USER_NOTICE);
                
                $indices = ['title', 'firstName', 'lastName', 'function', 
                    'phoneNumber', 'emailAddress', 'language'];
                
                foreach ($indices as $index) {
                    $information[$index] = null;
                }
            }
            
            $this->legalContact = new API\Entities\LegalContact(
                    $information['title'],
                    $information['firstName'],
                    $information['lastName'],
                    $information['function'],
                    $information['phoneNumber'],
                    $information['emailAddress'],
                    $information['language']
            );
        }
        return $this->legalContact;
    }
    
    public function getTechnicalContact()
    {
        if (is_null($this->technicalContact)) {
            $information = $this->settings['TECHNICAL_CONTACT'];
            $this->technicalContact = new API\Entities\TechnicalContact(
                    $information['title'],
                    $information['firstName'],
                    $information['lastName'],
                    $information['function'],
                    $information['phoneNumber'],
                    $information['emailAddress'],
                    $information['language']
            );
        }
        return $this->technicalContact;
    }
    
    /**
     * 
     * Fallback for when the technical contact is not set
     * 
     * @return TechnicalContact
     */
    public function getTechnicalContactFromCorporate()
    {

        $information = $this->settings['CORPORATE_CONTACT'];
        $this->technicalContact = new API\Entities\TechnicalContact(
                $information['title'],
                $information['firstName'],
                $information['lastName'],
                $information['function'],
                $information['phoneNumber'],
                $information['emailAddress'],
                $information['language']
        );

        return $this->technicalContact;
    }
    
    //Payment settings
    
    public function getPayment()
    {
        if (is_null($this->payment)) {
            $information = $this->settings['PAYMENT'];
            $this->payment = new API\Entities\Payment(
                    $information['mode'], 
                    $information['quotation']
            );
        }
        return $this->payment;
    }
    
    //Miscellaneous settings
    
    public function getMiscellaneous()
    {
        if (is_null($this->miscellaneous)) {
            $information = $this->settings['MISCELLANEOUS'];
            $this->miscellaneous = new API\Entities\Miscellaneous(
                    (array_key_exists('promotionalCode', $information) ? $information['promotionalCode'] : null),
                    (array_key_exists('customerNote', $information) ? $information['customerNote'] : null),
                    (array_key_exists('customerReference', $information) ? $information['customerReference'] : null),
                    (array_key_exists('reissueReason', $information) ? $information['reissueReason'] : null)
            );
        }
        return $this->miscellaneous;
    }

    //Certificate settings
    
    public function getCertificate()
    {
        if (is_null($this->certificate)) {
            $information = $this->settings['CERTIFICATE'];
            $this->certificate = new API\Entities\Certificate(
                    $information['caReference'],
                    $information['tbsReference']
            );
        }
        return $this->certificate;
    }
    
    /**
     * 
     * This function updates the current config settings to allow default values
     * to be overwritten by options.
     * 
     * @param array $options
     */
    public function update(array $options)
    {
        $oldSettings = $this->settings;
        
        if (!empty($options)) {
            $this->settings = array_replace_recursive($oldSettings, $options);
            
           //We need to force update the objects so we delete them
           //They will then be generated upon the next call to them
           $params = ['billingOrganization', 'certificate', 'corporateContact',
                      'legalContact', 'miscellaneous', 'organization', 'payment',
                      'product', 'request', 'technicalContact'];
           
           foreach($params as $param) unset($this->{$param});
        }
        
        unset($oldSettings);
    }
    
    /**
     * @param string $specificConfigFilePath Specific INI config file path.
     * @return Config instance of the Config singleton.
     */
    public static function getConfig($specificConfigFilePath = null)
    {
        if (is_null(self::$instance) || !empty($specificConfigFilePath)) {
            self::$instance = new Config($specificConfigFilePath);
        }
        
        return self::$instance;
    }
    
    /**
     * Reads the configuration directory (from the main config) and lists available
     * configuration files in the configuration directory.
     * 
     * @return array
     */
    public static function readConfigs()
    {
        $dir = self::getConfig()->getConfigurationDirectoryPath();
        self::$instance = null;
        
        $files = scandir($dir);
        
        $configs = [];
        
        $reader = new ConfigFileHelper\IniReader();
        
        foreach ($files as $file) {
            //We only want files that end with .ini
            if (!is_dir($file) && preg_match('/\.[iI][nN][iI]$/', $file)) {
                                
                $tmpConfig = $reader->readFile($dir . DIRECTORY_SEPARATOR . $file);
                
                $tmpConfig['fileName'] = $file;
                
                array_push($configs, $tmpConfig);        
            }
        }
        
        return $configs;
    }
}
