<?php

/* 
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\Controllers;

/**
 *  Handles the Hooks, the environment considerations
 */
class HookProfileHandler
{
    //Identifier
    protected $tbsReference;
    
    //Files
    protected $privateKeyFile;
    protected $certificateFile;
    protected $chainFile;
    protected $dcvFile;
    
    //Certificate Properties
    protected $commonName;
    protected $sanList;
    
    /**
     * 
     * @param string $tbsReference TBS Reference
     * @todo Create and use a general purpose certificate class for better abstraction
     */
    public function __construct($tbsReference, $profile = null)
    {
        if (empty($tbsReference) && empty($profile)) {
            throw new \Exception("No profile or TBS Reference provided to run hook");
        }
        
        $this->tbsReference = $tbsReference;
        
        if (empty($profile)) {
            $profile = \TBSCertBot\Controllers\Profile::getProfiles($tbsReference);
        
            //Since we use the TBS Reference to filter profiles, we can only get one.
            //Therefore, we can safely remove that dimension from the array.
            if (is_array($profile)) $profile = $profile[0];
        }

         
        if ($profile['PRIVATE_KEY']) {
            $this->privateKeyFile = $profile['PRIVATE_KEY']['fileName'];
        }
        
        //If the certificate is delivered
        if (array_key_exists('CERTIFICATE', $profile) && !empty($profile['CERTIFICATE']) 
                && array_key_exists('certificatePEM', $profile['CERTIFICATE']) && !empty($profile['CERTIFICATE']['certificatePEM'])) {
            $certAnalysis = openssl_x509_parse($profile['CERTIFICATE']['certificatePEM']);
            
            //Setting the CN
            if ($certAnalysis['subject']['CN']) {
                $this->commonName = $certAnalysis['subject']['CN'];
            } elseif ($certAnalysis['subject']['commonName']) {
                $this->commonName = $certAnalysis['subject']['commonName'];
            } elseif ($profile['REQUEST']['mainDomain']) {
                $this->commonName = $profile['REQUEST']['mainDomain'];
            } else {
                $this->commonName = $profile['CSR']['commonName'];
            }
            
            //Getting the San List
            //We make the "dirty" san list into a comma separated list
            $this->sanList = preg_replace('/[\n\r\0\x0B\s]/', '', str_replace('DNS:', '', $certAnalysis['extensions']['subjectAltName']));
            
            
            $this->certificateFile = $profile['CERTIFICATE']['certificatePEMFile'];
            $this->chainFile = $profile['CERTIFICATE']['chainPEMFile'];
        } else {
            if ($profile['REQUEST']['mainDomain']) {
                $this->commonName = $profile['REQUEST']['mainDomain'];
            } elseif (!empty($profile['CSR']['commonName'])) {
                $this->commonName = $profile['CSR']['commonName'];
            }
            
            if (($profile['REQUEST']['domainControlValidation'] === 'HTTP_CSR_HASH'
              || $profile['REQUEST']['domainControlValidation'] === 'HTTPS_CSR_HASH')
            && !empty($profile['CSR']['dcvFileName'])) {
            //If we can do DCV operations
                $this->dcvFile = $profile['CSR']['dcvFileName'];
                
            }
        }
    }

    
    /**
     *
     * Builds the environment variables to use in the scripts called by the hook
     *  
     * @throws \Exception
     */
    public function buildEnv()
    {
        try {
            if (!putenv('PHP_TBS_REFERENCE=' . $this->tbsReference)) {
                throw new \Exception('Could not export PHP_TBS_REFERENCE environment variable');
            }
            
            if ($this->commonName) {
                if (!putenv('PHP_TBS_CN=' . $this->commonName)) {
                    throw new \Exception('Could not export PHP_TBS_CN environment variable');
                }
            }
            
            if ($this->sanList) {
                if (!putenv('PHP_TBS_SAN=' . $this->sanList)) {
                    throw new \Exception('Could not export PHP_TBS_SAN environment variable');
                }
            }
            
            if ($this->privateKeyFile) {
                if (!putenv('PHP_TBS_KEY=' . $this->privateKeyFile)) {
                    throw new \Exception('Could not export PHP_TBS_KEY environment variable');
                }
            }
            
            if ($this->certificateFile) {
                if (!putenv('PHP_TBS_CERT=' . $this->certificateFile)) {
                    throw new \Exception('Could not export PHP_TBS_CERT environment variable');
                }
            }
            
            if ($this->chainFile) {
                if (!putenv('PHP_TBS_CHAIN=' . $this->chainFile)) {
                    throw new \Exception('Could not export PHP_TBS_CHAIN environment variable');
                }
            }
            
            if ($this->dcvFile) {
                if (!putenv('PHP_TBS_DCV=' . $this->dcvFile)) {
                    throw new \Exception('Could not export PHP_TBS_DCV environment variable');
                }
            }
        } catch (\Exception $e) {
            \cli\line($e->getMessage());
        }
    }
    
    /**
     * 
     * Cleans the environment variables created by TBSCertBot
     * 
     * @throws \Exception
     */
    public function destroyEnv()
    {
        $envVarList = array('PHP_TBS_REFERENCE', 'PHP_TBS_CN', 'PHP_TBS_SAN', 
            'PHP_TBS_KEY', 'PHP_TBS_CERT', 'PHP_TBS_CHAIN', 'PHP_TBS_DCV');
        
        try {
            foreach ($envVarList as $envVar) {
                if (getenv($envVar)) {
                    //First we try unsetting it using php
                    putenv($envVar);
            
                    //Then we try unsetting it using sh
                    if (getenv($envVar)) {
                        exec('unset ' . $envVar);
                
                        if (getenv($envVar)) {
                            throw new \Exception('Could not unset environment variable ' 
                                    . $envVar . '. Please manually unset PHP_DCV_* environment variables.');
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            \cli\line($e->getMessage());
        }
    }
}