<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\Controllers;

/**
 * 
 * Profile manipulation class
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Profile
{
    private static $PROFILES_DIRECTORY;
    
    private static $PROFILES_ARCHIVE_DIRECTORY;
    
    private static function getProfilesDirectory()
    {
        if (empty(self::$PROFILES_DIRECTORY)) {
            self::$PROFILES_DIRECTORY = 'data' . DIRECTORY_SEPARATOR
            . 'internal'. DIRECTORY_SEPARATOR . 'profiles';
        }
        
        return self::$PROFILES_DIRECTORY;
    }
    
    private static function getDefaultProfilesArchiveDirectory()
    {
        if (empty(self::$PROFILES_ARCHIVE_DIRECTORY)) {
            self::$PROFILES_ARCHIVE_DIRECTORY = 'data' . DIRECTORY_SEPARATOR
            . 'internal' . DIRECTORY_SEPARATOR . 'profiles' . DIRECTORY_SEPARATOR . 'archive';
        }
        
        return self::$PROFILES_ARCHIVE_DIRECTORY;
    }
    
    /**
     * 
     * Loads a specific profile. getProfiles() is preferred for most cases.
     * 
     * @param type $tbsReference
     * @throws \Exception
     * @todo Add logging when the function is used
     */
    public function load($tbsReference)
    {
        $json = file_get_contents(\TBSCertBot\Helper::getDirectory(self::getProfilesDirectory()) . DIRECTORY_SEPARATOR . $tbsReference . 'json');
        
        if ($json !== false) {
            $data = \TBSCertBot\JsonDriver::getJsonDriver()->decode($json);
  
        } else {
            throw new \Exception('Could not load profile');
        }
    }
    
    public function save()
    {
        
    }
    
    
    /**
     * 
     * Gets an array of all profiles or an array of one profile if a profile name was passed
     * 
     * @param string $profileName Profile Name: TBS Reference or UUID
     * @return array Profiles
     */
    public static function getProfiles($profileName = null)
    {
        return self::retrieveProfiles(\TBSCertBot\Helper::getDirectory(self::getProfilesDirectory()), $profileName);
    }
    
    /**
     * Gets an array of all archived profiles or an array of on archived profile if a profile name was passed
     * 
     * @param string $profileName
     * @return array Archived profiles
     */
    public static function getArchivedProfiles($profileName = null)
    {
        return self::retrieveProfiles(\TBSCertBot\Helper::getDirectory(self::getDefaultProfilesArchiveDirectory()), $profileName);
    }

    /**
     * Writes the profile to a file
     * 
     * @param array $profile
     */
    public static function setProfile($profile)
    {
        $logger = new \TBSCertBot\Logger();
        
        $json = \TBSCertBot\JsonDriver::getJsonDriver()->encode($profile);
        
        if (!empty($profile['CERTIFICATE']['tbsReference'])) {
        
            $filename = $profile['CERTIFICATE']['tbsReference'];
        } elseif (!empty($profile['REQUEST']['uniqueIdentifier'])) {
            $filename = $profile['REQUEST']['uniqueIdentifier'];
        } else {
            $logger->log("Cannot set filename for profile.", 'error');
            throw new \Exception("Cannot set filename for profile."); 
        }
        
        file_put_contents(
            realpath(\TBSCertBot\Helper::getDirectory(self::getProfilesDirectory()))
            . '/'
            . $filename
            . '.json'
            , $json
        );
    }
    
    /**
     * 
     * Deletes a profile file from disk.
     * 
     * @param string $profileName Profile Name: TBS Reference or UUID
     * @throws \Exception
     * @todo restore error handler in a finally instead of both the try and the catch when we move to PHP5.5 min
     */
    public static function deleteProfile($profileName) {
        try {
            set_error_handler("\TBSCertBot\Helper::exception_error_handler");
            
            $logger = new \TBSCertBot\Logger();
            
            if (empty($profileName)) {
                $logger->log('No filename was provided', 'error');
                throw new \Exception("No filename was provided");
            }
            
            $fileName = realpath(\TBSCertBot\Helper::getDirectory(self::getProfilesDirectory()))
                    . '/'
                    . $profileName
                    . '.json';
            
            if (is_file($fileName)) {
                                
                unlink($fileName);
            } else {
                $logger->log("There is no profile named $profileName", 'error');
                throw new \Exception("There is no profile named $profileName");
            }
            
            restore_error_handler();
        } catch (\Exception $e) {
            restore_error_handler();
            $logger->log($e->getMessage(), 'error');
            throw $e;
        }
    }
    
    public static function archive($profileName)
    {
        $logger = new \TBSCertBot\Logger();
        
        if (empty($profileName)) {
            $logger->log('Impossible to archive: no profile name passed', 'error');
            throw new \Exception("Impossible to archive: no profile name passed");
        }
        
        $source = \TBSCertBot\Helper::getDirectory(self::getProfilesDirectory()) . DIRECTORY_SEPARATOR . $profileName . '.json';
        $destination = \TBSCertBot\Helper::getDirectory(self::getDefaultProfilesArchiveDirectory()) .  DIRECTORY_SEPARATOR . $profileName . '.json';
        
        if (!is_file($source)) {
            throw new \Exception("The profile $profileName does not exist");
        }
        
        if (is_file($destination)) {
            if (\TBSCertBot\Config::getConfig()->getBatch()) {
                $logger->log("The profile $profileName already exists in the archive and"
                        . "cannot therefore be moved", 'error');
                
                throw new \Exception("The profile $profileName already exists in the archive and"
                        . "cannot therefore be moved");
            } else {
                \cli\line("The profile $profileName already exists in the archive");
                $logger->log("The profile $profileName already exists in the archive", 'warn');
                
                $choice = \cli\choose("Would you like to overwrite it?", "yn", "n");
                
                //If the user does not want to overwrite the file, we exit TBSCertBot
                if ($choice == n) exit();
            }
        }
        
        $status = rename($source, $destination);
        
        if ($status === false) {
            $logger->log("Error archiving profile $profileName", 'error');
            throw new \Exception("Error archiving profile $profileName");
        } else {
            \cli\line("Profile $profileName has been archived");
            $logger->log("Profile $profileName has been archived", 'info');
        }
    }
    
    public static function unarchive($profileName)
    {
        $logger = new \TBSCertBot\Logger();
        
        if (empty($profileName)) {
            $logger->log('Impossible to unarchive: no profile name passed', 'error');
            throw new \Exception("Impossible to unarchive: no profile name passed");
        }
        
        $source = \TBSCertBot\Helper::getDirectory(self::getDefaultProfilesArchiveDirectory()) .  DIRECTORY_SEPARATOR . $profileName . '.json';
        $destination = \TBSCertBot\Helper::getDirectory(self::getProfilesDirectory()) . DIRECTORY_SEPARATOR . $profileName . '.json';
        
        if (!is_file($source)) {
            $logger->log("The profile $profileName does not exist in the archive", 'error');
            throw new \Exception("The profile $profileName does not exist in the archive");
        }
        
        if (is_file($destination)) {
            if (\TBSCertBot\Config::getConfig()->getBatch()) {
                $logger->log("An active $profileName profile already exists: "
                        . "Impossible to unarchive $profileName", 'error');
                
                throw new \Exception("An active $profileName profile already exists: "
                        . "Impossible to unarchive $profileName");
            } else {
                \cli\line("An active $profileName profile already exists");
                $logger->log("An active $profileName profile already exists", 'warn');
                
                $choice = \cli\choose("Would you like to overwrite it?", "yn", "n");
                
                //If the user does not want to overwrite the file, we exit TBSCertBot
                if ($choice == n) exit();
            }
        }
        
        $status = rename($source, $destination);
        
        if ($status === false) {
            $logger->log("Error unarchiving profile $profileName", 'error');
            throw new \Exception("Error unarchiving profile $profileName");
        } else {
            $logger->log("Profile $profileName has been unarchived", 'info');
            \cli\line("Profile $profileName has been unarchived");
        }
    }
    
    
    /**
     * Retrieves profiles from specified directory
     * 
     * @param string $directory
     * @param string $profileName
     * @return array Profiles
     * @throws Exception
     * @throws \Exception
     */
    private static function retrieveProfiles($directory, $profileName = null)
    {
        $logger = new \TBSCertBot\Logger();
         
        
        if (empty($directory) || !is_dir($directory)) {
            $logger->log('Invalid profile directory', 'error');
            throw new \Exception("Invalid profile directory");
        }
        
        $profiles = array();
        
        if (is_null($profileName)) {
            $files = scandir(realpath($directory));
        } else {
            $files[0] = $profileName . '.json';
            
            if (!is_file($directory . DIRECTORY_SEPARATOR . $files[0])) {
                $logger->log('The provided profile name ("' . $profileName . '.json") is invalid', 'error');
                throw new \Exception('The provided profile name ("' . $profileName . '.json") is invalid');
            }
        }

        foreach ($files as $profile) {
            //We only want files that end with .json and that are valid json
            if (!is_dir($profile) && preg_match('/\.[jJ][sS][oO][nN]$/', $profile)) {
                $info = \TBSCertBot\JsonDriver::getJsonDriver()->decode(file_get_contents(
                    realpath($directory)
                    . '/'
                    . $profile
                    ), true);
                
                array_push($profiles, $info);
            }
            
        }
        return $profiles;
    }
    
}
