<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\Controllers;

/**
 * A recoverable entity (order, renewal), stored in a json.
 * This class will be used to handle recovering orders with lost references.
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Recoverable
{
    private $uuid;
    private $tbsReference;
    
    /**
     * 
     * @param string $uuid A string reprensentation of the UUID associated with the order
     */
    public function __construct($uuid)
    {
         try {
            if (\TBSCertBot\Crypto\Helper::isUuid($uuid)) {
                $this->uuid = $uuid;
            } else {
                throw new \Exception("Parameter is not a valid UUID.");
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * 
     * @return string The UUID associated with this Recoverable Object
     */
    public function getUuid()
    {
        return $this->uuid;
    }
    
    /**
     * 
     * @return string The recovered TBS Reference
     */
    public function getTbsReference()
    {
        return $this->tbsReference;
    }
    
    /**
     * Recovers the TBS reference
     * 
     * @throws \Exception
     */
    public function recover()
    {
        try {
            $recover = new \TBSCertBot\API\Methods\Recover\Request($this->uuid);
            $recoverResponse = $recover->send();
            
            //PHP 5.3 compat: empty needs a reference
            $xmlRpcCode = $recoverResponse->getXmlRpcCode();
            $xmlRpcMessage = $recoverResponse->getXmlRpcMessage();
            
            if (is_int($xmlRpcCode) && !empty($xmlRpcCode)) {
                $errMess = 'Could not get a response.';
                
                if (!empty($xmlRpcCode) && !empty($xmlRpcMessage)) {
                    $errMess .= PHP_EOL . 'Network Error No ' . $xmlRpcCode
                        . ': ' . $xmlRpcMessage;
                }
                
                throw new \Exception($errMess);
            } elseif (\TBSCertBot\API\Helper::isErrorCodeNegative($recoverResponse->getCode())) {
                //If we get an API error, we throw it as an exception
                throw new \Exception(\TBSCertBot\API\Helper::selectErrorMessage($recoverResponse->getCode(), $recoverResponse->getMessage()),
                        $recoverResponse->getCode());
            } 
            
            $this->tbsReference = $recoverResponse->getTBSReference();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    

    /**
     * 
     * Attempts to add the TBS Reference to the profile, write it into the correct
     * filename and delete the old file
     * 
     * @throws \Exception
     */
    public function fixFile()
    {
        try {
            if (empty($this->tbsReference)) {
                if (\TBSCertBot\Config::getConfig()->getBatch()) {                
                    throw new \Exception("Cannot fix file if there is no TBS Reference");
                }
                
                \cli\line("There is no TBS Reference corresponding to this file.");
                $choice = \cli\choose("Would you like to archive this file?", 'yn', 'y');
                
                if ($choice == 'y') {
                    Profile::archive($this->uuid);
                } 
                
                $skip = true;
            }

            if (!$skip) {
                /*
                 * getProfiles always returns an array of profiles,
                 * even if we pass a parameter.
                 */
                //PHP 5.3 compat: no array dereferencing
                $allProfiles = Profile::getProfiles($this->uuid);

                $profile = $allProfiles[0];


                $profile['CERTIFICATE']['tbsReference'] = $this->tbsReference;

                Profile::setProfile($profile);

                //Now, we check if wrote the fixed profile correctly.
                $profile2 = Profile::getProfiles($this->tbsReference);

                if (is_array($profile2[0])) {
                    Profile::deleteProfile($this->uuid);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
