<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\Controllers;

/**
 * Main validator
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
class Validation
{
    const HTTP_DCV_SUFFIX = 'trust-provider';
    
    static private $defaultDCV = array(
        'HTTP_CSR_HASH' => "Upload the following content to your website: ** sha256 \\n trust-provider.com \\n uniqueValue ** at http://domain/.well-known/pki-validation/md5.txt",
        'HTTPS_CSR_HASH' => "Upload the following content to your website: ** sha256 \\n trust-provider.com \\n uniqueValue ** at https://domain/.well-known/pki-validation/md5.txt",
        'CNAME_CSR_HASH' => 'Provide the following field in your DNS zone: md5.domain. CNAME sha.256.uniqueValue.comodoca.com.'
        );
    
    private $request;
    private $csr;
    
    private $mainDomain;
    private $domains;
    private $dcv;
    
    public function __construct(
            \TBSCertBot\API\Entities\Request &$request,
            \TBSCertBot\Crypto\CSRInterface &$csr
            ) {
        $this->request = $request;
        $this->csr = $csr;
    }
    
    /**
     * Parses domains
     * @return array List of the domains
     */
    private function parseDomains()
    {
        $domains = $this->request->getDomains();
        
        if (!empty($domains)) {
            $domains = preg_split('/[\s]+/', $domains);
            if (sizeof($domains) > 0) {
                foreach ($domains as $key => $domain) {
                    $domains[$key] = trim($domain);
                }
            } else {
                $domains = array();
            }
        } else {
            $domains = array();
        }
        
        $this->domains = $domains;
    }
    
    /**
     * Parses the DCV
     */
    private function parseDCV()
    {
        $dcv = $this->request->getDomainControlValidation();
        
        if (!empty($dcv)) {
            $dcv = preg_split('/[\s]+/', $dcv);
            if (sizeof($dcv) > 0) {
                foreach ($dcv as $key => $method) {
                    $dcv[$key] = trim($method);
                }
            } else {
                $dcv = null;
            }
        } else {
            $dcv = null;
        }
        
        $this->dcv = $dcv;
    }
    
    /**
     * Checks if the main domain exists.
     * If not, uses the first domain.
     * @return string Main domain
     */
    private function checkMainDomain()
    {
        $mainDomain = $this->request->getMainDomain();
        
        if ($mainDomain === null) {
            $mainDomain = $this->domains[0];
        } else {
            if (!in_array($mainDomain, $this->domains)) {
                array_push($this->domains, $mainDomain);
            }
        }
        $this->mainDomain = $mainDomain;
    }
    
    /**
     * - Parses domains.
     * - Parses the DCV
     * - If there is no main domain, get it from the first domain.
     * - If there's a main domain, checks that it is in the list.
     * - Check that each domain has a DCV method selected.
     * - In case of an issue, call an interactive fallback.
     */
    public function checkDCV()
    {   
        for ($i = 0; $i < sizeof($this->domains); $i++) {
            $acceptableEmailAddressesRequest = new \TBSCertBot\API\Methods\DCVEmailInformation\Request($this->domains[$i]);
            $acceptableEmailAddressesResponse = $acceptableEmailAddressesRequest->send();
            
            $acceptableEmailAddresses = $acceptableEmailAddressesResponse->getEmailAddresses();
            $acceptableEmailAddresses = array_flip($acceptableEmailAddresses);
            
            foreach ($acceptableEmailAddresses as $address => $description) {
                $acceptableEmailAddresses[$address] = "Validation e-mail sent to $address";
            }
            
            $acceptableDCV = array_merge(
                    self::$defaultDCV,
                    $acceptableEmailAddresses
                );

            while (!array_key_exists($this->dcv[$i], $acceptableDCV)) {
                \cli\line('An issue occured with the Domain Control Validation parameter for domain ' . $this->domains[$i] . '. Please provide or correct the following information');
                if (!\TBSCertBot\Config::getConfig()->getBatch()) {
                    $this->displayAcceptableDCV($acceptableDCV, $this->domains[$i]);
                    $this->interactiveFallback($i);
                } else {
                    // TODO: Error management.
                }
            }
        }
    }
    
    /**
     * 
     * @param array $acceptableDCV
     * @param string $domain
     * @todo Check CNAME validation
     */
    private function displayAcceptableDCV(array $acceptableDCV, $domain)
    {
        $table = new \cli\Table();
        $table->setHeaders(array('value', 'validation'));
        $rows = array();
        
        foreach ($acceptableDCV as $value => $validation) {
            if (array_key_exists($value, self::$defaultDCV)) {
                if ($value == 'CNAME_CSR_HASH') { // TODO: Remove this exception as it is in uppercase in TBS system?
                   $validation = str_replace('md5', $this->csr->getMD5(), $validation); 
                } else {
                    $validation = str_replace('md5', strtoupper($this->csr->getMD5()), $validation);
                }
                $validation = str_replace('sha1', $this->csr->getSHA1(), $validation);
                $validation = str_replace('domain', $domain, $validation);
            }
            
            array_push($rows, array(
                $value,
                $validation,
                )
            );
        }
        
        $table->setRows($rows);
        $table->display();
    }
    
    private function interactiveFallback($index)
    {
        $this->dcv[(int)$index] = \cli\prompt('   - DCV');
    }
    
    private function updateRequest()
    {
        $this->request->setMainDomain($this->mainDomain);
        $this->request->setDomains(implode(' ', $this->domains));
        
        if (!empty($this->dcv) && is_array($this->dcv)) {
            $this->request->setDomainControlValidation(implode(' ', $this->dcv));
        } else {
            $this->request->setDomainControlValidation($this->dcv);
        }
    }
    
    public function validate($dcvRelevancy = true)
    {
        $this->parseDomains();
        $this->parseDCV();
        $this->checkMainDomain();
        
        if ($dcvRelevancy) {
            $this->checkDCV();
        }
        
        $this->updateRequest();
    }
    
    /**
     * 
     * @param \TBSCertBot\API\Entities\Request $request
     * @return string List of domains or null in case of failure
     * @todo handle cloning reference
     */
    public static function getCN(\TBSCertBot\API\Entities\Request $request)
    {
        if ($request->getMainDomain() !== null) {
            $cn = $request->getMainDomain();
        } else {
            $domains = $request->getDomains();
            if (!empty($domains)) {
                $domains = preg_split('/[\s]+/', $domains);
                if (sizeof($domains) > 0) {
                    foreach ($domains as $key => $domain) {
                        $domains[$key] = trim($domain);
                    }

                    $request->setMainDomain($domains[0]);
                    $cn = $domains[0];
                } else {
                    $cn = null;
                }
            } else {
                $cn = null;
            }
        }
        return $cn;
    }
}
