<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\Crypto;

/**
 * Certificate class
 *
 */
class Certificate {
    private $fileName;
    
    private $authorityCertificate;
    private $authorityKey;
    
    /**
     *
     * @var \TBSCertBot\Crypto\CSR
     */
    private $csr;
    
    private $certificateResource;
    
    /**
     * 
     * @param \TBSCertBot\Crypto\CSR $csr
     * @param string $fileName Full path to the future certificate
     * @throws \Exception
     */
    public function __construct($csr, $fileName)
    {
        $publicKey = openssl_csr_get_public_key($csr->getASCII());
        
        if ($publicKey === false) {
            throw new \Exception("Impossible to create certificate: invalid CSR");
        }
        
        if (empty($fileName)) {
            throw new \Exception("Impossible to create certificate: empty filename");
        }
        
        $this->fileName = $fileName;
        $this->csr = $csr;
        
        //Initializing to null in case we do not set the authority
        $this->authorityCertificate = null;
        
        //Setting to the CSR private key in case we fallback to self signed
        $this->authorityKey = $this->csr->getPrivateKey();
    }
   
    /**
     * Generates the ceriticate, also exports it to file
     * 
     * @throws \Exception
     */
   public function generate()
   {
       try {
           //If we are trying to make a self signed certificate, the authority certificate should be null
           if ($this->authorityCertificate instanceof Certificate) {
               $caCertificateAscii = $this->authorityCertificate->getASCII();
           } elseif ($this->authorityCertificate == null) {
               $caCertificateAscii = null;
           } else {
               throw new \Exception("Impossible to generate certificate: invalid ca certificate format");
           }
           
           //Regardless of what the PHP documentation might say, this function specifically requires a strings containing PEM
            $certificate = openssl_csr_sign($this->csr->getASCII(), 
                   $caCertificateAscii, 
                   $this->authorityKey->getASCII(),
                    30);

           if ($certificate !== false) {
               $this->certificateResource = $certificate;
           } else {
               throw new \Exception("Error generating certificate");
           }
           
           $this->exportToFile();
       } catch (\Exception $e) {
           throw $e;
       }
   }
   
   /**
    * Sets the Authority used to issue the certificate
    * 
    * @param \TBSCertBot\Crypto\PrivateKey $key
    * @param \TBSCertBot\Crypto\Certificate $cert
    * @throws \Exception
    */
   public function setAuthority($key, $cert)
   {
       if (!($key instanceof PrivateKey)) {
           throw new \Exception("Impossible to create certificate chain: invalid ca private key format");
       }
       
       if (!($cert instanceof Certificate)) {
           throw new \Exception("Impossible to create certificate chain: invalid ca certificate format");
       }
       
       $keyResource = openssl_pkey_get_private($key->getASCII());
       
       if ($keyResource === false) {
           throw new \Exception("Impossible to create certificate chain: invalid private key");
       }
       
       if (!openssl_x509_check_private_key($cert->getCertificateResource(), $key->getKeyResource())) {
           throw new \Exception("Impossible to create certificate: "
                   . "the authority key does not match its certificate");
       }
       
       $this->authorityCertificate = $cert;
       $this->authorityKey = $key;
   }
   
   public function exportToFile()
   {
       $status = openssl_x509_export_to_file($this->certificateResource, $this->fileName);
        
        if (!$status) {
            throw new \Exception('CSR could not be exported to file.');
        }
   }
   
   public function getASCII()
   {
       $ascii = '';
        $status = openssl_x509_export($this->certificateResource, $ascii);
        
        if (!$status) {
            throw new \Exception('Certificate has not been generated yet.');
        }
        return trim($ascii);
   }
   
   public function getCertificateResource()
   {
       return $this->certificateResource;
   }
}
