<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\Crypto;

/**
 * Implementation of the CSR importation
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 * @todo Improve modularity relative to the necessity of DCV
 */
class ExternalCSR implements CSRInterface
{
    const DIGEST_ALGORITHM = 'sha256';
 
    private static $DCV_DIRECTORY; //TODO: Shouldn't this be a configurable directory?
    
    private $privateKey;
    
    private $commonName;
    private $organizationName;
    private $localityName;
    private $stateOrProvinceName;
    private $countryName;
    private $organizationalUnitName;
    private $organizationIdentifier;
    
    private $csr;
    
    /**
     *
     * @var string The full path to the CSR file
     */
    private $fileName;
    
    /**
     *
     * @var string The full path to the DCV HTTP File 
     */
    private $dcvHttpFileName;
    
    public function __construct(
            array $distinguishedName,
            $csr,
            $fileName,
            $dcvUniqueValue
            )
    {
        $this->csr = $csr;
        $this->fileName = $fileName;
        
        if ($distinguishedName) {
            $this->setSubject($distinguishedName);   
        } elseif (extension_loaded('mbstring')) {
            $this->setSubjectFromOpenssl();
        } else {
            \cli\line("WARNING: Cannot verify CSR due to plaform limitation. Using as provided.");
            $choice = \cli\choose("Would you like to continue");
            if ($choice === 'n') exit();
        }
        
        $this->exportToFile();
        
        $this->generateDCVHTTPFile($dcvUniqueValue);

    }
    
    private static function getDcvDirectory()
    {
        if (empty(self::$DCV_DIRECTORY)) {
            self::$DCV_DIRECTORY = 'data' . DIRECTORY_SEPARATOR . 'dcv';
        }
        
        return self::$DCV_DIRECTORY;
    }
    
    public function getCN()
    {
        return $this->commonName;
    }
    
    public function getO()
    {
        return $this->organizationName;
    }
    
    public function getOU()
    {
        return $this->organizationalUnitName;
    }
    
    public function getL()
    {
        return $this->stateOrProvinceName;
    }
    
    public function getST()
    {
        return $this->stateOrProvinceName;
    }
    
    public function getC()
    {
        return $this->countryName;
    }
    
    public function getOrganizationIdentifier()
    {
        return $this->organizationIdentifier;
    }
    
    /**
     * Sets the internal class structure to contain the subject from an array
     * 
     * @param array $subject
     */
    private function setSubject(array $subject)
    {
        if (isset($subject['commonName'])) {
            $this->commonName = $subject['commonName'];
        } elseif (isset($subject['CN'])) {
            $this->commonName = $subject['CN'];
        }
        if (isset($subject['organizationName'])) {
            $this->organizationName = $subject['organizationName'];
        } elseif (isset($subject['O'])) {
            $this->organizationName = $subject['O'];
        }
        if (isset($subject['organizationalUnitName'])) {
            $this->organizationalUnitName = $subject['organizationalUnitName'];
        } elseif (isset($subject['OU'])) {
            $this->organizationalUnitName = $subject['OU'];
        }
        if (isset($subject['localityName'])) {
            $this->localityName = $subject['localityName'];
        } elseif (isset($subject['L'])) {
            $this->localityName = $subject['L'];
        }
        if (isset($subject['stateOrProvinceName'])) {
            $this->stateOrProvinceName = $subject['stateOrProvinceName'];
        } elseif (isset($subject['ST'])) {
            $this->stateOrProvinceName = $subject['ST'];
        }
        if (isset($subject['countryName'])) {
            $this->countryName = $subject['countryName'];
        } elseif (isset($subject['C'])) {
            $this->countryName = $subject['C'];
        }
        if (isset($subject['organizationIdentifier'])) {
            $this->organizationIdentifier = $subject['organizationIdentifier'];
        } elseif (isset($subject['OI'])) {
            $this->organizationIdentifier = $subject['OI'];
        }
    }
    
    /**
     * 
     * @return string Rerturns false in case of error
     */
    private function setSubjectFromOpenssl() {
        exec("openssl version", $output);
        
        if (!$output) {
            \cli\line("WARNING: Cannot verify CSR due to plaform limitation. Using as provided.");
            $choice = \cli\choose("Would you like to continue (y/n)");
            if ($choice === 'n') exit();
            
            return false;
        }
        
        try {
            $table = \TBSCertBot\Crypto\Helper::csrAnalysis($this->getASCII());
        } catch (\Exception $e) {
            \cli\line($e->getMessage());
            exit();
        }
        
        $this->setSubject($table);
    }
    
    /**
     * 
     * Generates an DN (Subject) array from the internal class values
     * 
     * @return array
     */
    private function constructDN()
    {
        $dn = array();
        
        if (!empty($this->commonName)) {
            $dn['commonName'] = $this->commonName;
        }
        if (!empty($this->organizationName)) {
            $dn['organizationName'] = $this->organizationName;
        }
        if (!empty($this->organizationalUnitName)) {
            $dn['organizationalUnitName'] = $this->organizationalUnitName;
        }
        if (!empty($this->localityName)) {
            $dn['localityName'] = $this->localityName;
        }
        if (!empty($this->stateOrProvinceName)) {
            $dn['stateOrProvinceName'] = $this->stateOrProvinceName;
        }
        if (!empty($this->countryName)) {
            $dn['countryName'] = $this->countryName;
        }
        if (!empty($this->organizationIdentifier)) {
            $dn['organizationIdentifier'] = $this->organizationIdentifier;
        }
        
        return $dn;
    }
    
    /**
     * 
     * @return string Full path to the CSR file
     */
    public function getFileName()
    {
        return $this->fileName;
    }
    
    /**
     * 
     * @return string Full path to the DCV HTTP file
     */
    public function getDcvHttpFileName()
    {
        return realpath($this->dcvHttpFileName);
    }
    
    /**
     * Returns the base64 (aka pem) CSR
     * 
     * @return string
     * @throws \Exception
     */
    public function getASCII()
    {
        return $this->csr;
    }
    
    /**
     * Writes the CSR to a file
     * 
     * @throws \Exception
     */
    public function exportToFile()
    {
        $status = file_put_contents($this->fileName, $this->csr);
        
        if (!$status) {
            throw new \Exception('CSR could not be exported to file.');
        }
    }
    
    /**
     * 
     * Returns a MD5 Hash of the DER-encoded CSR for use in the DCV
     * 
     * @return string
     */
    public function getMD5()
    {
        return md5($this->getDER());
    }
    
    /**
     * 
     * Returns a SHA1 Hash of the DER-encoded CSR for use in the DCV
     * 
     * @return string
     * @deprecated since version 0.2
     */
    public function getSHA1()
    {
        return sha1($this->getDER());
    }
    
    /**
     * 
     * Returns a SHA256 Hash of the DER-encoded CSR for use in the DCV
     * 
     * @return string
     */
    public function getSHA256()
    {
        return hash('sha256', $this->getDER());
    }
    
    // http://php.net/manual/fr/ref.openssl.php
    private function getDER()
    {
        $pemData = $this->getASCII();
        
        $begin = "REQUEST-----";
        $end   = "-----END";
        
        $pemData = substr($pemData, strpos($pemData, $begin)+strlen($begin));
        $pemData = substr($pemData, 0, strpos($pemData, $end));
        $der = base64_decode($pemData);
        
        return $der;
    }
    
    /**
     * 
     * Formats the SHA256 DER CSR Hash for use in the DNS Challenge.
     * 
     * @return string Formatted SHA256 Hash
     */
    private function getDnsFormattedSHA256() {
        $initialHash = $this->getSHA256();
        $hashHalf = \strlen($initialHash) / 2;
        
        $formattedHash = substr($initialHash, 0, $hashHalf) . '.'
                .substr($initialHash, $hashHalf);
        
        return $formattedHash;
    }
    
    public function getDcvCnameRecord($dcvUniqueValue)
    {
        return strtoupper($this->getMD5()) . '.' . $this->getCN() . '. '
                . 'CNAME ' . strtolower($this->getDnsFormattedSHA256() 
                        . '.' . $dcvUniqueValue . '.trust-provider.com.');
    }
    
    /**
     * Generates the DCV HTTP File
     * 
     * @param string $dcvUniqueValue
     * @todo override the error handler to throw exceptions
     */
    private function generateDCVHTTPFile($dcvUniqueValue)
    {
        $this->dcvHttpFileName = \TBSCertBot\Helper::getDirectory(self::getDcvDirectory()) . DIRECTORY_SEPARATOR . strtoupper($this->getMD5()) . '.txt';
        try {
            file_put_contents($this->dcvHttpFileName,
                    $this->getSHA256() . "\n" . 'trust-provider.com'
                    . "\n" . $dcvUniqueValue . "\n");
        } catch (\Exception $e) {
            throw $e;
        }
        
    }

    /**
     * 
     * Creates the array needed for serialization
     * 
     * @return array
     */
    public function save()
    {
        $info = array();
        
        if (isset($this->commonName) && !is_null($this->commonName)) {
            $info['commonName'] = $this->commonName;
        }
        if (isset($this->organizationName) && !is_null($this->organizationName)) {
            $info['organizationName'] = $this->organizationName;
        }
        if (isset($this->localityName) && !is_null($this->localityName)) {
            $info['localityName'] = $this->localityName;
        }
        if (isset($this->stateOrProvinceName) && !is_null($this->stateOrProvinceName)) {
            $info['stateOrProvinceName'] = $this->stateOrProvinceName;
        }
        if (isset($this->countryName) && !is_null($this->countryName)) {
            $info['countryName'] = $this->countryName;
        }
        if (isset($this->organizationalUnitName) && !is_null($this->organizationalUnitName)) {
            $info['organizationalUnitName'] = $this->organizationalUnitName;
        }
        if (isset($this->organizationIdentifier) && !is_null($this->organizationIdentifier)) {
            $info['organizationIdentifier'] = $this->organizationIdentifier;
        }
        if (isset($this->fileName) && !is_null($this->fileName)) {
            $info['fileName'] = realpath($this->fileName);
        }
        
        if (isset($this->dcvHttpFileName) && !is_null($this->dcvHttpFileName)) {
            $info['dcvFileName'] = realpath($this->dcvHttpFileName);
        }
        
        try {
            $ascii = $this->getASCII();
        } catch (\Exception $e) {
            
        }
        
        if (!empty($ascii)) {
            $info['PEM'] = $ascii;
        }
        
        return $info;
    }
}
