<?php

/*
 * This file is part of TBSCertBot.
 * 
 * Copyright (c) 1996-2016 TBS Internet
 * 
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace TBSCertBot\Crypto;

/**
 * Helper Class for crypto features
 *
 * @author TBS Internet <webmaster@tbs-internet.com>
 */
abstract class Helper
{

    private static $CACHE_DIRECTORY; //TODO: Shouldn't this be a configurable directory?

    static $baseFields = array(
        'commonName' => 'CN',
        'localityName' => 'L',
        'postalCode' => 'PostalCode',
        'countryName' => 'C',
        'stateOrProvinceName' => 'ST',
        'streetAddress' => 'STREET',
        'organizationName' => 'O',
        'organizationalUnitName' => 'OU',
        'emailAddress' => 'Email',
        'SN' => 'surname',
        'GN' => 'givenName',
        'I' => 'initials',
        'UID' => 'userId',
        '2.5.4.3' => 'CN',
        '2.5.4.4' => 'surname',
        '2.5.4.5' => 'serialNumber',
        '2.5.4.6' => 'C',
        '2.5.4.7' => 'L',
        '2.5.4.8' => 'ST',
        '2.5.4.9' => 'STREET',
        '2.5.4.10' => 'O',
        '2.5.4.11' => 'OU',
        '2.5.4.12' => 'title',
        '2.5.4.13' => 'description',
        '2.5.4.17' => 'PostalCode',
        '2.5.4.18' => 'POBox',
        '2.5.4.20' => 'Phone',
        '2.5.4.42' => 'givenName',
        '2.5.4.43' => 'initials',
        '1.3.6.1.4.1.311.60.2.1.3' => 'IncorporationCountry',
        '1.3.6.1.4.1.311.60.2.1.1' => 'IncorporationLocality',
        '1.3.6.1.4.1.311.60.2.1.2' => 'IncorporationState',
        '0.9.2342.19200300.100.1.1' => 'userId',
        'serialNumber' => 'IncorporationNumber',
        '1.3.2' => 'sirene',
        'street' => 'STREET',
        'postOfficeBox' => 'POBox',
        '2.5.4.15' => 'businessCategory',
        '2.5.4.97' => 'organizationIdentifier'
    );
    static $encodingConversion = array(
        'UTF8STRING' => 'UTF-8',
        'BMPSTRING' => 'UCS-2');
    static $translation = array(
        "rsaEncryption" => "RSA",
        "id-ecPublicKey" => "ECC"
    );
    static $multipleFields = array('OU', 'STREET');

    private static function getCacheDirectory()
    {
        if (empty(self::$CACHE_DIRECTORY)) {
            self::$CACHE_DIRECTORY = 'data' . DIRECTORY_SEPARATOR . 'cache';
        }
        
        return self::$CACHE_DIRECTORY;
    }
    
    /**
     * Cleans a CSR, mostly from whitespace
     * @param string $csr
     * @return string csr
     */
    public static function cleanCsr($csr)
    {
        if (!empty($csr)) {
            $cleanCsr = "";
            $begin = "";
            $end = "";
            $csrData = "";

            $data = explode(PHP_EOL, $csr);

            foreach ($data as $line) {
                // Cleaning the end of line from PHP_EOL and special chars
                $chars = array("\r\n", "\n", "\r", "\t", "\0", "\x0B");
                $line = str_replace($chars, "", $line);
                $line = trim($line);
                //BEGIN/END lines case
                if (preg_match("#-#", $line)) {
                    if ($csrData == "") {
                        $begin .= $line;
                    } else {
                        $end .= $line;
                    }
                } else {
                    $line = str_replace(" ", "", $line);
                    $csrData .= $line;
                }
            }

            // $csrData contains all the data on one line without \n
            // Splitting every 64 characters
            $csrData = chunk_split($csrData, 64, PHP_EOL);
            $csr_clean = $begin . PHP_EOL . $csrData . $end;

            return $csr_clean;
        } else {
            return $csr;
        }
    }

    /**
     * Checks hyphens in CSR
     * @param string $csr
     * @return string Hyphen-valid CSR
     */
    public static function checkHyphens($csr)
    {
        //Cleanup
        //Looking for hyphens
        if (strstr($csr, '----')) {
            //Cutting by 5 hyphens
            $table = explode('-----', $csr);

            //Cutting by 4 hyphen if there aren't 5
            if (sizeof($table) == 1)
                $table = explode('----', $csr);


            //Updated the logic here.
            //Instead of breaking on the first string with length > 50,
            //we keep iterating until we find the longest string.
            //Some CSRs have comments before the ----BEGIN,
            //so we don't want to select that string as the CSR, rather than the actual CSR content.
            $maxChunkLength = 0;

            foreach ($table AS $ligne) {
                $currChunkLength = strlen(trim($ligne));

                if ($currChunkLength > 100) { //Removes the line BEGIN CERTIFICATE REQUEST
                    if ($currChunkLength > $maxChunkLength) {
                        $csrpro = trim($ligne);
                        $maxChunkLength = $currChunkLength;
                    }
                }
            }
        } else {
            //If there are no hyphens, we take everything
            $csrpro = trim($csr);
        }

        //Adding the hyphens
        $return = "-----BEGIN CERTIFICATE REQUEST-----\n"
                . $csrpro
                . "\n-----END CERTIFICATE REQUEST-----";
        return $return;
    }

    /**
     * 
     * Parses ASN data for X509 certificates
     * 
     * @param string $tempFile Filename
     * @return array
     */
    
    public static function asn2($tempFile)
    {
        exec('openssl asn1parse -in '
                . $tempFile . ' 2>/dev/null', $asn);

        $baseFields = array(
            'commonName' => 'CN',
            'localityName' => 'L',
            'stateOrProvinceName' => 'ST',
            'streetAddress' => 'STREET',
            'organizationName' => 'O',
            'organizationalUnitName' => 'OU',
            'countryName' => 'C');

        $format = array();

        for ($n = 0; $n < sizeof($asn); $n++) {
            //We look for an interesting line
            if (stristr($asn[$n], "object")) {
                //The field name is at the end of the line
                // ex : 18:d=5 hl=2 l= 3 prim: OBJECT :countryName
                $tmp = explode(":", $asn[$n]);
                $ch = array_pop($tmp);
                //on cherche si c'est un champ qui doit être traité
                if (isset($baseFields[$ch])) {
                    //The following line
                    //ex : 23:d=5 hl=2 l= 2 prim: PRINTABLESTRING :FR
                    preg_match_all("!\d+!", $asn[++$n], $tmp);
                    $format[$baseFields[$ch]] = $tmp[0][3];

                    //BMPSTRING ou UCS-2 special cases
                    $fmt = explode(":", $asn[$n]);
                    if (in_array(trim($fmt[2]), array('BMPSTRING', 'UCS-2')))
                        $format[$baseFields[$ch]]/=2;
                }
            }
        }
        return $format;
    }

    /**
     * 
     * Recursivelly call a function on each element of a multidimentional array
     * 
     * @param function $func Callback function that is run on each element of the array
     * @param array $arr Array on which we want to run the function
     * @return type
     */
    public static function array_map_r($func, $arr)
    {
        // Check the $arr variable to ensure it is an array,
        //then process each value of the array agains the user function ($func)
        //if it is a multidimensional array then recall the function and process
        // rebulding the original array structure with the processed values.
        $newArr = array();
        if (is_array($arr)) {
            foreach ($arr as $key => $value) {
                $newArr[$key] = ( is_array($value) 
                        ? array_map_r($func, $value) 
                        : ( is_array($func) 
                                ? call_user_func_array($func, $value) 
                                : $func($value) ) );
            }
            return $newArr;
        } else {
            return $arr;
        }
    }

    /**
     * 
     * Analyses CSR using binary openssl
     * 
     * @param string $csr
     * @return table fields of the CSR
     */
    public static function csrAnalysis($csr)
    {
        try {
            if (!strpos($csr, 'CERTIFICATE REQUEST')) {
                throw new \Exception("Not a CSR");
            }
            
            //Cleanup
            $csr = self::cleanCsr($csr);


            //nettoyage mise en forme
            $csrpro = self::checkHyphens($csr);

            //Temp file
            //Unsing a micro-timestamp as name for unicity
            $tempFile = \TBSCertBot\Helper::getDirectory(self::getCacheDirectory()) . round(microtime(true) * 1000) . '.txt';
            $pointer = fopen($tempFile, "w");
            fwrite($pointer, $csrpro);
            fclose($pointer);

            #exec openssl
            $modulus = " -modulus";
            exec("openssl req -text -in " . $tempFile
                    . $modulus . " -nameopt sep_multiline,utf8,show_type", $get);

            $flag = false;

            $nbCN = 0;
            foreach (self::$multipleFields AS $z)
                $$z = 0; // init multiple fields line number
            for ($n = 0; $n < sizeof($get); $n++) {
                if (strstr($get[$n], "Subject:")) {
                    $n++;
                    do {
                        $temp = explode("=", $get[$n]);
                        $champ = (!empty(self::$baseFields[trim($temp[0])])) ? self::$baseFields[trim($temp[0])] : trim($temp[0]);
                        if (in_array($champ, self::$multipleFields)) {
                            $champ .= ++$$champ;
                        }

                        $tmp = explode(":", $temp[1], 2);

                        $tableBrut[$champ] = $tmp[1]; //pour le contrÃ´le de longueur de chaine

                        $encodage = mb_detect_encoding($tmp[1], mb_detect_order(), true);
                        $tmp[1] = ($encodage != "UTF-8") ? utf8_encode($tmp[1]) : $tmp[1];

                        //Some data appears to get doubly encoded, this saves decoding on the pages.
                        $tempUTFDecode = utf8_decode($tmp[1]);
                        $tempEncodage = mb_detect_encoding($tempUTFDecode, mb_detect_order(), true);

                        if ($tempEncodage == "UTF-8" && $tmp[0] != 'CN') {
                            $tmp[1] = $tempUTFDecode;
                        }

                        $table[$champ] = $tmp[1];

                        #06-10-2011
                        if ($champ == 'CN')
                            $nbCN++;
                        $n++;
                    } while (!strstr($get[$n], "Subject"));
                }

                if (stristr($get[$n], "public key algorithm")) {
                    $temp = explode(":", $get[$n]);
                    if (in_array(trim($temp[1]), array("rsaEncryption", "id-ecPublicKey"))) {
                        $table['keyFormat'] = strtr(trim($temp[1]), self::$translation);
                    } else {
                        throw new \Exception("The private key must be RSA or ECC");
                    }
                } else if (strstr($get[$n], "RSA Public Key")) {
                    $temp = preg_match("/\d+/", $get[$n], $nombre);
                    $table['keyLength'] = $nombre[0];
                } else if (strstr($get[$n], "Modulus")) {
                    $temp = explode("=", $get[$n]);
                    $table['modulus'] = wordwrap($temp[1], 32, "<br />", true);
                } else if (strpos($get[$n], "challengePassword") !== false) {
                    $temp = explode(":", $get[$n]);
                    $table['challengePassword'] = $temp[1];
                } else if (strpos($get[$n], "unstructuredName") !== false) {
                    $temp = explode(":", $get[$n]);
                    $table['unstructuredName'] = $temp[1];
                } else if (strstr($get[$n], "Subject Alternative Name") && !$table['DNS']) {
                    $n++;
                    $ligne = str_replace("DNS:", "", $get[$n]);
                    $tableTemp = explode(",", $ligne);

                    foreach ($tableTemp AS $dns)
                        $table['DNS'][] = trim($dns);
                } else if (strpos($get[$n], "Signature Algorithm") !== false) {
                    $temp = explode(":", $get[$n]);
                    $table['algorithm'] = $temp[1];
                }
            }


            if ($nbCN > 1 && $profile->commonName != 0) {
                throw new \Exception("There are multiple CN fields in the CSR");
            }

            //Supplementary analysis
            if (sizeof($table) > 0) {
                //Lowercase c
                if (isset($table['C']) && !preg_match("![A-Z]{2}!", $table['C'])) {
                    throw new \Exception("The C field is not ISO 3166-compliant (two capital letters).");
                }

                //Fields with invisible characters
                $lgChamp = self::asn2($tempFile);

                if ($lgChamp['CN'] != strlen(trim($tableBrut['CN']))) {
                    throw new \Exception("The CN field is not RFC 5280-compliant. Probable invisible character at the end or invalid encoding");
                }
            }

            /* clean all of the csr values to prevent xss
              and sql injection further down the csr prossessing
              removing all tags and escaping all delimiters etc */
            if (is_array($table)) {
                $table = self::array_map_r('strip_tags', $table);
            }

            @unlink($tempFile);
            return $table;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public static function generateUuid()
    {
        try {
            $myUuid = \Ramsey\Uuid\Uuid::uuid4();
            
            if (!($myUuid instanceof \Ramsey\Uuid\Uuid)) throw new \Exception("UUID generation failed");
            
        } catch (\Exception $ex) {
            //We can also choose to catch a Ramsey\Uuid\Exception\UnsatisfiedDependencyException
            //If we want to treat them differently
            throw $ex;
        }
        
        return $myUuid->toString();
    }  
 
    /*
     * @param string $uuid
     */
    public static function isUuid($uuid)
    {
        if (!is_string($uuid)) return false;
        
        //$uuid must be a string
        return \Ramsey\Uuid\Uuid::isValid($uuid);
    }
    
    /*
     * Uses openssl to generate a value in a range that will be used
     * in getAlphanumericToken
     * 
     * @param $min int
     * @param $max int
     * 
     */
    protected static function crypto_rand_secure($min, $max)                                                                                                                      
    {                                                                                                                                                            
        $range = $max - $min;                                                                                                                                    
        if ($range < 1) return $min; // not so random...                                                                                                         
        
        $log = ceil(log($range, 2));                                                                                                                             
        $bytes = (int) ($log / 8) + 1; // length in bytes                                                                                                        
        $bits = (int) $log + 1; // length in bits                                                                                                                
        $filter = (int) (1 << $bits) - 1; // set all lower bits to 1                                                                                             
    
        do {                                                                                                                                                     
            $rnd = hexdec(bin2hex(openssl_random_pseudo_bytes($bytes)));                                                                                         
            $rnd = $rnd & $filter; // discard irrelevant bits                                                                                                    
        } while ($rnd > $range);                                                                                                                                 
    
        return $min + $rnd;                                                                                                                                      
    }
    
    /**
     * Generates an alphanumeric random string of $length length
     * 
     * @param type $length Length of the string
     * @return string An alphanumeric random string
     */
    public static function getAlphanumericToken($length)                                                                                                                                   
    {                                                                                                                                                            
        $token = "";                                                                                                                                             
        $codeAlphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";                                                                                                            
        $codeAlphabet.= "abcdefghijklmnopqrstuvwxyz";                                                                                                            
        $codeAlphabet.= "0123456789";                                                                                                                            
        $max = strlen($codeAlphabet); // edited                                                                                                                  
                                                                                                                                                             
        for ($i=0; $i < $length; $i++) {                                                                                                                         
            $token .= $codeAlphabet[self::crypto_rand_secure(0, $max-1)];                                                                                              
        }                                                                                                                                                        
                                                                                                                                                             
        return $token;                                                                                                                                           
   }
}
